/* ############################ MALP License ############################## */
/* # Fri Jan 18 14:00:00 CET 2013                                         # */
/* # Copyright or (C) or Copr. Commissariat a l'Energie Atomique          # */
/* #                                                                      # */
/* # This software is governed by the CeCILL-C license under French law   # */
/* # and abiding by the rules of distribution of free software.  You can  # */
/* # use, modify and/ or redistribute the software under the terms of     # */
/* # the CeCILL-C license as circulated by CEA, CNRS and INRIA at the     # */
/* # following URL http://www.cecill.info.                                # */
/* #                                                                      # */
/* # The fact that you are presently reading this means that you have     # */
/* # had knowledge of the CeCILL-C license and that you accept its        # */
/* # terms.                                                               # */
/* #                                                                      # */
/* # Authors:                                                             # */
/* #   - BESNARD Jean-Baptiste jean-baptiste.besnard@cea.fr               # */
/* #                                                                      # */
/* ######################################################################## */
#include <svut.h>
#include <iostream>
#include <stdint.h>

using namespace svUnitTest;

#include "Bloom_Filter.h"

using namespace std;

SVUT_DECLARE_FLAT_TEST(Bloom_Filter,Init_Release)
{
	struct Bloom_Filter bf;

	Bloom_Filter_init( &bf, 4241, sizeof( uint64_t ) );

	SVUT_ASSERT_EQUAL( bf.lock, 0u );
	SVUT_ASSERT_EQUAL( bf.size, 4241u );
	SVUT_ASSERT_EQUAL( bf.sizeof_payload, sizeof( uint64_t ) );

	Bloom_Filter_release( &bf );

	SVUT_ASSERT_EQUAL( bf.lock, 0u );
	SVUT_ASSERT_EQUAL( bf.size, 0u );
	SVUT_ASSERT_EQUAL( bf.sizeof_payload, 0u );
}

SVUT_DECLARE_FLAT_TEST(Bloom_Filter,Bloom_Filter_push_and_test)
{
	struct Bloom_Filter bf;
	int ret;
	int i,j;

	uint64_t *val = (uint64_t *)malloc( 1024*sizeof( uint64_t ) );


	Bloom_Filter_init( &bf, 4241, 1024*sizeof( uint64_t ) );


	for( i = 0 ; i < 128 ;i++ )
	{
		for( j = 0 ; j < 1024 ; j++ )
		{
			val[j] = rand();
		}

		Bloom_Filter_push( &bf , val );

		ret = Bloom_Filter_test( &bf , val );

		SVUT_ASSERT_EQUAL( 1, ret );
	}


	Bloom_Filter_release( &bf );
	free( val );

}

SVUT_DECLARE_FLAT_TEST(Bloom_Filter,Bloom_Filter_replicate)
{
	struct Bloom_Filter bf;
	struct Bloom_Filter bf2;
	int ret;
	int i,j;

	uint64_t *val = (uint64_t *)malloc( 1024*sizeof( uint64_t ) );


	Bloom_Filter_init( &bf, 4241, 1024*sizeof( uint64_t ) );
	Bloom_Filter_init( &bf2, 4241, 1024*sizeof( uint64_t ) );


	for( i = 0 ; i < 128 ;i++ )
	{
		for( j = 0 ; j < 1024 ; j++ )
		{
			val[j] = rand();
		}

		Bloom_Filter_push( &bf , val );
	}

	Bloom_Filter_replicate( &bf2 , &bf );

	ret = Bloom_Filter_test( &bf2 , val );

	SVUT_ASSERT_EQUAL( 1, ret );

	Bloom_Filter_release( &bf );
	Bloom_Filter_release( &bf2 );
	free( val );

}
