/* ############################ MALP License ############################## */
/* # Fri Jan 18 14:00:00 CET 2013                                         # */
/* # Copyright or (C) or Copr. Commissariat a l'Energie Atomique          # */
/* #                                                                      # */
/* # This software is governed by the CeCILL-C license under French law   # */
/* # and abiding by the rules of distribution of free software.  You can  # */
/* # use, modify and/ or redistribute the software under the terms of     # */
/* # the CeCILL-C license as circulated by CEA, CNRS and INRIA at the     # */
/* # following URL http://www.cecill.info.                                # */
/* #                                                                      # */
/* # The fact that you are presently reading this means that you have     # */
/* # had knowledge of the CeCILL-C license and that you accept its        # */
/* # terms.                                                               # */
/* #                                                                      # */
/* # Authors:                                                             # */
/* #   - BESNARD Jean-Baptiste jean-baptiste.besnard@cea.fr               # */
/* #                                                                      # */
/* ######################################################################## */
#include <svut.h>
#include <iostream>
#include <stdint.h>

#include <omp.h>

using namespace svUnitTest;

#include "Buffered_FIFO.h"


using namespace std;

SVUT_DECLARE_FLAT_TEST(Buffered_FIFO,Buffered_FIFO_chunk_init_release)
{
	struct Buffered_FIFO_chunk ch;
	Buffered_FIFO_chunk_init( &ch, 1024, sizeof(uint64_t) );
	SVUT_ASSERT_EQUAL( 0u, ch.lock );
	SVUT_ASSERT_NOT_NULL( ch.payload );
	SVUT_ASSERT_EQUAL( sizeof(uint64_t), ch.elem_size );
	SVUT_ASSERT_EQUAL( 1024u, ch.chunk_size );
	SVUT_ASSERT_EQUAL( 0u, ch.start_offset );
	SVUT_ASSERT_EQUAL( 0u, ch.end_offset );
	Buffered_FIFO_chunk_release( &ch);
	SVUT_ASSERT_EQUAL( 0u, ch.lock );
	SVUT_ASSERT_NULL( ch.payload );
	SVUT_ASSERT_EQUAL( 0u, ch.elem_size );
	SVUT_ASSERT_EQUAL( 0u, ch.chunk_size );
	SVUT_ASSERT_EQUAL( 0u, ch.start_offset );
	SVUT_ASSERT_EQUAL( 0u, ch.end_offset );
}

SVUT_DECLARE_FLAT_TEST(Buffered_FIFO,Buffered_FIFO_chunk_push_pop)
{
	void *ret;
	struct Buffered_FIFO_chunk ch;
	Buffered_FIFO_chunk_init( &ch, 1024, sizeof(uint64_t) );

	uint64_t i;
	
	for( i = 0 ; i < 1024 ; i++ )
	{
		ret = Buffered_FIFO_chunk_push( &ch, (void *)&i );
		SVUT_ASSERT_NOT_NULL( ret );
		SVUT_ASSERT_EQUAL( i, *((uint64_t *)ret) );
	}
	
	uint64_t val;
	for( i = 0 ; i < 1024 ; i++ )
	{
		ret = Buffered_FIFO_chunk_pop( &ch, (void *)&val );
		SVUT_ASSERT_NOT_NULL( ret );
		SVUT_ASSERT_EQUAL( i, val );
		SVUT_ASSERT_EQUAL( i, *((uint64_t *)ret) );
	}


	Buffered_FIFO_chunk_release( &ch);
}

SVUT_DECLARE_FLAT_TEST(Buffered_FIFO,Buffered_FIFO_chunk_overflow)
{
	void *ret;
	struct Buffered_FIFO_chunk ch;
	Buffered_FIFO_chunk_init( &ch, 1024, sizeof(uint64_t) );

	uint64_t i;
	
	/* FILL it */
	for( i = 0 ; i < 1024 ; i++ )
	{
		ret = Buffered_FIFO_chunk_push( &ch, (void *)&i );
		SVUT_ASSERT_NOT_NULL( ret );
		SVUT_ASSERT_EQUAL( i, *((uint64_t *)ret) );
	}

	/* Overflow */
	for( i = 0 ; i < 1024 ; i++ )
	{
		ret = Buffered_FIFO_chunk_push( &ch, (void *)&i );
		SVUT_ASSERT_NULL( ret );
	}
	
	uint64_t val;
	/* POP one */
	ret = Buffered_FIFO_chunk_pop( &ch, (void *)&val );
	SVUT_ASSERT_NOT_NULL( ret );
	SVUT_ASSERT_EQUAL( 0u, val );
	SVUT_ASSERT_EQUAL( 0u, *((uint64_t *)ret) );

	/* Sill overflow ! */
	for( i = 0 ; i < 1024 ; i++ )
	{
		ret = Buffered_FIFO_chunk_push( &ch, (void *)&i );
		SVUT_ASSERT_NULL( ret );
	}	

	Buffered_FIFO_chunk_release( &ch);
}

SVUT_DECLARE_FLAT_TEST(Buffered_FIFO,Buffered_FIFO_chunk_underflow)
{
	void *ret;
	struct Buffered_FIFO_chunk ch;
	Buffered_FIFO_chunk_init( &ch, 1024, sizeof(uint64_t) );

	uint64_t i;
	
	for( i = 0 ; i < 1024 ; i++ )
	{
		ret = Buffered_FIFO_chunk_push( &ch, (void *)&i );
		SVUT_ASSERT_NOT_NULL( ret );
		SVUT_ASSERT_EQUAL( i, *((uint64_t *)ret) );
	}
	
	uint64_t val;
	for( i = 0 ; i < 1024 ; i++ )
	{
		ret = Buffered_FIFO_chunk_pop( &ch, (void *)&val );
		SVUT_ASSERT_NOT_NULL( ret );
		SVUT_ASSERT_EQUAL( i, val );
		SVUT_ASSERT_EQUAL( i, *((uint64_t *)ret) );
	}


	val = 0;
	/* Underflow is detected */
	for( i = 0 ; i < 1024 ; i++ )
	{
		ret = Buffered_FIFO_chunk_pop( &ch, (void *)&val );
		SVUT_ASSERT_NULL( ret );
		SVUT_ASSERT_EQUAL( 0u, val );
	}

	Buffered_FIFO_chunk_release( &ch);
}


SVUT_DECLARE_FLAT_TEST(Buffered_FIFO,Buffered_FIFO_chunk_parallel)
{
	uint64_t i;
	struct Buffered_FIFO_chunk ch;
	Buffered_FIFO_chunk_init( &ch, 1024*1024, sizeof(uint64_t) );



#pragma omp parallel
{	
	void *ret;
	
	#pragma omp for private(i)
	for( i = 0 ; i  < 1024*1024 ; i++ )
	{
		ret = Buffered_FIFO_chunk_push( &ch, (void *)&i );
	}
}

	/* Try to get elems */
	uint64_t val;
	for( i = 0 ; i < 1024*1024 ; i++ )
	{
		void *uret = Buffered_FIFO_chunk_pop( &ch, (void *)&val );
		SVUT_ASSERT_NOT_NULL( uret );
	}

	Buffered_FIFO_chunk_release( &ch);
}


SVUT_DECLARE_FLAT_TEST(Buffered_FIFO,Init_Release)
{
	struct Buffered_FIFO f;
	
	Buffered_FIFO_init(&f, 2, sizeof( uint64_t ));
	
	SVUT_ASSERT_NULL( f.head );
	SVUT_ASSERT_NULL( f.tail );
	SVUT_ASSERT_EQUAL( sizeof( uint64_t ), f.elem_size );
	SVUT_ASSERT_EQUAL( 2u, f.chunk_size );
	SVUT_ASSERT_EQUAL( 0u, f.elem_count );

	Buffered_FIFO_release( &f, NULL);

	SVUT_ASSERT_NULL( f.head );
	SVUT_ASSERT_NULL( f.tail );
	SVUT_ASSERT_EQUAL( 0u, f.elem_size );
	SVUT_ASSERT_EQUAL( 0u, f.chunk_size );
	SVUT_ASSERT_EQUAL( 0u, f.elem_count );
}


SVUT_DECLARE_FLAT_TEST(Buffered_FIFO,Push_repeat)
{
	struct Buffered_FIFO f;
	
	Buffered_FIFO_init(&f, 2, sizeof( uint64_t ));

	uint64_t j;
	uint64_t elem_count;
	
	for( j = 0 ; j < 1000 ; j++ )
	{
		uint64_t i;
		elem_count = rand() % 5000;

		for( i = 0 ; i < elem_count ; i++ )
		{
			Buffered_FIFO_push( &f, (void*)&i);
		}

		uint64_t elem;
		for( i = 0 ; i < elem_count ; i++ )
		{
			Buffered_FIFO_pop( &f, &elem );
			SVUT_ASSERT_EQUAL( i, elem );
		}
		
		for( i = 0 ; i < elem_count ; i++ )
		{
			SVUT_ASSERT_NULL( Buffered_FIFO_pop( &f, &elem ) );
		}
	}


	Buffered_FIFO_release( &f, NULL);
}

SVUT_DECLARE_FLAT_TEST(Buffered_FIFO,Push_repeat_unclean)
{
	struct Buffered_FIFO f;
	
	Buffered_FIFO_init(&f, 9, sizeof( uint64_t ));

	uint64_t j;
	uint64_t elem_count;
	
	uint64_t pushed = 0;
	uint64_t popped = 0;
	
	for( j = 0 ; j < 1000 ; j++ )
	{
		uint64_t i;
		elem_count = rand() % 5000;

		for( i = 0 ; i < elem_count ; i++ )
		{
			Buffered_FIFO_push( &f, (void*)&i);
			pushed++;
		}
		
		elem_count = rand() % 5000;
		
		
		for( i = 0 ; i < elem_count ; i++ )
		{
			Buffered_FIFO_push( &f, (void*)&i);
			pushed++;
		}

		uint64_t elem;
		for( i = 0 ; i < elem_count ; i++ )
		{
			SVUT_ASSERT_NOT_NULL( Buffered_FIFO_pop( &f, &elem ) );
			popped++;
		}
		
		while( Buffered_FIFO_pop( &f, &elem ) )
		{
			popped++;
		}
	}

	printf("Pushed %lld & popped %lld\n", (long long int)pushed, (long long int)popped );
	SVUT_ASSERT_EQUAL( pushed, popped );

	Buffered_FIFO_release( &f, NULL);
}


