/* ############################ MALP License ############################## */
/* # Fri Jan 18 14:00:00 CET 2013                                         # */
/* # Copyright or (C) or Copr. Commissariat a l'Energie Atomique          # */
/* #                                                                      # */
/* # This software is governed by the CeCILL-C license under French law   # */
/* # and abiding by the rules of distribution of free software.  You can  # */
/* # use, modify and/ or redistribute the software under the terms of     # */
/* # the CeCILL-C license as circulated by CEA, CNRS and INRIA at the     # */
/* # following URL http://www.cecill.info.                                # */
/* #                                                                      # */
/* # The fact that you are presently reading this means that you have     # */
/* # had knowledge of the CeCILL-C license and that you accept its        # */
/* # terms.                                                               # */
/* #                                                                      # */
/* # Authors:                                                             # */
/* #   - BESNARD Jean-Baptiste jean-baptiste.besnard@cea.fr               # */
/* #                                                                      # */
/* ######################################################################## */
#include <svut.h>
#include <iostream>
#include <stdint.h>

#include <omp.h>

using namespace svUnitTest;


#include "Buffered_LIFO.h"


SVUT_DECLARE_FLAT_TEST(Buffered_LIFO,Buffered_LIFO_init_release)
{
	struct Buffered_LIFO lifo;
	Buffered_LIFO_init( &lifo, sizeof(int), 1024 );
	
	SVUT_ASSERT_EQUAL( sizeof(int), lifo.sizeof_payload );
	SVUT_ASSERT_EQUAL( 1024u, lifo.chunk_size );
	SVUT_ASSERT_EQUAL( 0u, lifo.lock );
	SVUT_ASSERT_NULL( lifo.head );

	Buffered_LIFO_release( &lifo, NULL );

	SVUT_ASSERT_EQUAL( 0, lifo.sizeof_payload );
	SVUT_ASSERT_EQUAL( 0u, lifo.chunk_size );
	SVUT_ASSERT_EQUAL( 0u, lifo.lock );
	SVUT_ASSERT_NULL( lifo.head );
}



SVUT_DECLARE_FLAT_TEST(Buffered_LIFO,Buffered_LIFO_push_then_pop)
{
	struct Buffered_LIFO lifo;
	Buffered_LIFO_init( &lifo, sizeof(int), 1024 );
	
	int i;
	
	#define LIFO_PP_SIZE 2048
	
	for( i = 0 ; i < LIFO_PP_SIZE ; i++ )
	{
		Buffered_LIFO_push( &lifo, (void *)&i );
	}
	
	int out = -1;
	void * ret = NULL;
	
	for( i = 0 ; i < LIFO_PP_SIZE ; i++ )
	{
		ret = Buffered_LIFO_pop( &lifo, (void *)&out );
		SVUT_ASSERT_NOT_NULL( ret );
		SVUT_ASSERT_EQUAL( LIFO_PP_SIZE - i -1 , out );
	}
	

	Buffered_LIFO_release( &lifo, NULL );
}


SVUT_DECLARE_FLAT_TEST(Buffered_LIFO,Buffered_LIFO_push_then_pop_many)
{
	struct Buffered_LIFO lifo;
	Buffered_LIFO_init( &lifo, sizeof(int), 4096 );
	
	int i;
	
	#define LIFO_PP_SIZE_MANY 1024*1024*10
	
	for( i = 0 ; i < LIFO_PP_SIZE_MANY ; i++ )
	{
		Buffered_LIFO_push( &lifo, (void *)&i );
	}
	
	int out = -1;
	void * ret = NULL;
	
	for( i = 0 ; i < LIFO_PP_SIZE_MANY ; i++ )
	{
		ret = Buffered_LIFO_pop( &lifo, (void *)&out );
		SVUT_ASSERT_NOT_NULL( ret );
		SVUT_ASSERT_EQUAL( LIFO_PP_SIZE_MANY - i -1 , out );
	}
	

	Buffered_LIFO_release( &lifo, NULL );
}



SVUT_DECLARE_FLAT_TEST(Buffered_LIFO,Buffered_LIFO_chunk_parallel)
{
	struct Buffered_LIFO lifo;
	Buffered_LIFO_init( &lifo, sizeof(int), 1024 );
	
	int i;
	
	#pragma omp parallel
	{	
		void *ret;
		
		#pragma omp for private(i)
		for( i = 0 ; i  < 1024*1024 ; i++ )
		{
			Buffered_LIFO_push( &lifo, (void *)&i );
		}
	}

	
	int out = -1;
	void * ret = NULL;
	
	for( i = 0 ; i < 024*1024 ; i++ )
	{
		ret = Buffered_LIFO_pop( &lifo, (void *)&out );
		SVUT_ASSERT_NOT_NULL( ret );
	}
	

	Buffered_LIFO_release( &lifo, NULL );
}
