/* ############################ MALP License ############################## */
/* # Fri Jan 18 14:00:00 CET 2013                                         # */
/* # Copyright or (C) or Copr. Commissariat a l'Energie Atomique          # */
/* #                                                                      # */
/* # This software is governed by the CeCILL-C license under French law   # */
/* # and abiding by the rules of distribution of free software.  You can  # */
/* # use, modify and/ or redistribute the software under the terms of     # */
/* # the CeCILL-C license as circulated by CEA, CNRS and INRIA at the     # */
/* # following URL http://www.cecill.info.                                # */
/* #                                                                      # */
/* # The fact that you are presently reading this means that you have     # */
/* # had knowledge of the CeCILL-C license and that you accept its        # */
/* # terms.                                                               # */
/* #                                                                      # */
/* # Authors:                                                             # */
/* #   - BESNARD Jean-Baptiste jean-baptiste.besnard@cea.fr               # */
/* #                                                                      # */
/* ######################################################################## */
#include <svut.h>
#include <iostream>
#include <stdint.h>

using namespace svUnitTest;

#include "Chained_List.h"


using namespace std;

SVUT_DECLARE_FLAT_TEST(Chained_List,Init_Release)
{
	struct Chained_List cl;
	Chained_List_init( &cl, 1024, sizeof( uint64_t ) );
	
	SVUT_ASSERT_EQUAL( cl.count, 0u );
	SVUT_ASSERT_NULL( cl.list );
	
	SVUT_ASSERT_EQUAL( cl.chunk_size, 1024u );
	SVUT_ASSERT_EQUAL( cl.payload_size, sizeof( uint64_t ) );
	
	Chained_List_release( &cl );
	
	SVUT_ASSERT_EQUAL( cl.count, 0u );
	SVUT_ASSERT_NULL( cl.list );
}


SVUT_DECLARE_FLAT_TEST(Chained_List,Chained_List_count_1K)
{
	struct Chained_List cl;
	Chained_List_init( &cl, 2, sizeof( uint64_t ) );
	
	uint64_t i = 0;
	for( i = 0 ; i < 1024 ; i++ )
	{
		SVUT_ASSERT_NOT_NULL( Chained_List_push( &cl, (void *)&i ) );
	}
	
	SVUT_ASSERT_EQUAL( cl.count, 1024u );
	
	Chained_List_release( &cl );
}


SVUT_DECLARE_FLAT_TEST(Chained_List,Chained_List_count_100K)
{
	struct Chained_List cl;
	Chained_List_init( &cl, 1024, sizeof( uint64_t ) );
	
	uint64_t i = 0;
	for( i = 0 ; i < 100000 ; i++ )
	{
		SVUT_ASSERT_NOT_NULL( Chained_List_push( &cl, (void *)&i ) );
	}
	
	SVUT_ASSERT_EQUAL( cl.count, 100000u );
	
	Chained_List_release( &cl );
}


SVUT_DECLARE_FLAT_TEST(Chained_List,Chained_List_get_nth)
{
	struct Chained_List cl;
	Chained_List_init( &cl, 500, sizeof( uint64_t ) );
	
	//Push
	uint64_t i = 0;
	for( i = 0 ; i < 100000 ; i++ )
	{
		SVUT_ASSERT_NOT_NULL( Chained_List_push( &cl, (void *)&i ) );
	}
	
	SVUT_ASSERT_EQUAL( Chained_List_count( &cl ) , 100000 );
	
	//Retrieve
	for( i = 0 ; i < 100000 ; i++ )
	{
		uint64_t *ret = (uint64_t *)Chained_List_get_nth(&cl, i);
		SVUT_ASSERT_NOT_NULL( (void *)ret );
		
		// Se should get the elements in inversed order
		SVUT_ASSERT_EQUAL( *ret , ( 100000 - 1 - i ) );
	}
	
	
	//Overflow
	for( i = 100000 ; i < 500000 ; i++ )
	{
		uint64_t *ret = (uint64_t *)Chained_List_get_nth(&cl, i);
		SVUT_ASSERT_NULL( (void *)ret );
	}
	
	
	Chained_List_release( &cl );
}


SVUT_DECLARE_FLAT_TEST(Chained_List,Chained_List_get_nth_seq_empty_list)
{
	struct Chained_List cl;
	struct chained_list_view clv;

	Chained_List_init( &cl, 500, sizeof( uint64_t ) );

		
	chained_list_view_init( &clv );

	SVUT_ASSERT_NULL( (void *)clv.last_chunk );
	SVUT_ASSERT_ZERO( clv.last_chunk_offset );
	
	int i = 0;
	
	//Retrieve
	for( i = 0 ; i < 100000 ; i++ )
	{
		uint64_t *ret = (uint64_t *)Chained_List_get_nth_seq(&cl, &clv, i);
		SVUT_ASSERT_NULL( (void *)ret );
	}

	
	Chained_List_release( &cl );
}

SVUT_DECLARE_FLAT_TEST(Chained_List,Chained_List_get_nth_seq)
{
	struct Chained_List cl;
	struct chained_list_view clv;

	Chained_List_init( &cl, 500, sizeof( uint64_t ) );

	
	//Push
	uint64_t i = 0;
	for( i = 0 ; i < 100000 ; i++ )
	{
		SVUT_ASSERT_NOT_NULL( Chained_List_push( &cl, (void *)&i ) );
	}
	
		
	chained_list_view_init( &clv );

	SVUT_ASSERT_NULL( (void *)clv.last_chunk );
	SVUT_ASSERT_ZERO( clv.last_chunk_offset );
	
	
	//Retrieve
	for( i = 0 ; i < 100000 ; i++ )
	{
		uint64_t *ret = (uint64_t *)Chained_List_get_nth_seq(&cl, &clv, i);
		SVUT_ASSERT_NOT_NULL( (void *)ret );
		
		// Se should get the elements in inversed order
		SVUT_ASSERT_EQUAL( *ret , ( 100000 - 1 - i ) );
	}

	
	//Overflow
	for( i = 100000 ; i < 500000 ; i++ )
	{
		uint64_t *ret = (uint64_t *)Chained_List_get_nth(&cl, i);
		SVUT_ASSERT_NULL( (void *)ret );
	}
	
	
	Chained_List_release( &cl );
}



SVUT_DECLARE_FLAT_TEST(Chained_List,Chained_List_get_elem)
{
	struct Chained_List cl;

	Chained_List_init( &cl, 1024, sizeof( uint64_t ) );

	
	//Push
	uint64_t i = 0;
	uint64_t needle = 42;
	uint64_t haystack = 0;
	uint64_t secret_off = ( rand() % 100000 );
	
	for( i = 0 ; i < 100000 ; i++ )
	{
		
		if( i == secret_off )
		{
			SVUT_ASSERT_NOT_NULL( Chained_List_push( &cl, (void *)&needle ) );
		}
		else
		{
			haystack = 43 + rand();
			SVUT_ASSERT_NOT_NULL( Chained_List_push( &cl, (void *)&haystack ) );
		}
	}
	
	uint64_t *ret = (uint64_t *)Chained_List_get_elem( &cl, NULL, &needle );
	SVUT_ASSERT_EQUAL( *ret , needle );
	
	needle = 1;
	ret = (uint64_t *)Chained_List_get_elem( &cl, NULL, &needle );
	SVUT_ASSERT_NULL( (void *)ret );
	

	Chained_List_release( &cl );
}


