/* ############################ MALP License ############################## */
/* # Fri Jan 18 14:00:00 CET 2013                                         # */
/* # Copyright or (C) or Copr. Commissariat a l'Energie Atomique          # */
/* #                                                                      # */
/* # This software is governed by the CeCILL-C license under French law   # */
/* # and abiding by the rules of distribution of free software.  You can  # */
/* # use, modify and/ or redistribute the software under the terms of     # */
/* # the CeCILL-C license as circulated by CEA, CNRS and INRIA at the     # */
/* # following URL http://www.cecill.info.                                # */
/* #                                                                      # */
/* # The fact that you are presently reading this means that you have     # */
/* # had knowledge of the CeCILL-C license and that you accept its        # */
/* # terms.                                                               # */
/* #                                                                      # */
/* # Authors:                                                             # */
/* #   - BESNARD Jean-Baptiste jean-baptiste.besnard@cea.fr               # */
/* #                                                                      # */
/* ######################################################################## */
#include <svut.h>
#include <iostream>
#include <stdint.h>
#include <stdlib.h>

using namespace svUnitTest;

#include "Hash_Tree.h"

using namespace std;


struct Request
{
	uint64_t id;
};


struct Payload
{
	uint64_t content[512];
};


SVUT_DECLARE_FLAT_TEST(Hash_Tree,Init_Release)
{
	MALP_Trace_crc64_init();
	struct Hash_Tree htr;
	
	Hash_Tree_init( &htr, sizeof(struct Request), sizeof(struct Payload) );

	SVUT_ASSERT_EQUAL(htr.payload_size, sizeof(struct Payload));
	SVUT_ASSERT_EQUAL(htr.request_size, sizeof(struct Request));
	SVUT_ASSERT_EQUAL(htr.node_count, 0u);
	SVUT_ASSERT_EQUAL(htr.lock, 0u);

	Hash_Tree_release( &htr, NULL );

	SVUT_ASSERT_EQUAL(htr.payload_size, 0);
	SVUT_ASSERT_EQUAL(htr.request_size, 0);
	SVUT_ASSERT_EQUAL(htr.node_count, 0u);
	SVUT_ASSERT_EQUAL(htr.lock, 0u);

}


SVUT_DECLARE_FLAT_TEST(Hash_Tree,Push_no_parent_and_request)
{
	MALP_Trace_crc64_init();
	struct Hash_Tree htr;
	
	Hash_Tree_init( &htr, sizeof(struct Request), sizeof(struct Payload) );

	struct Request req;
	struct Payload pl;

	req.id = 42;
	memset( &pl, rand(), sizeof(struct Payload) );


	struct Hash_Tree_node *node =  Hash_Tree_push( &htr, NULL, (void *)&req, (void *)&pl );

	SVUT_ASSERT_NOT_NULL( node );

	struct Hash_Tree_node *node_req = Hash_Tree_get_elem( &htr, (void *)&req );

	SVUT_ASSERT_NOT_NULL( node_req );

	int ret = memcmp( &pl, node_req->payload, sizeof(sizeof(struct Payload) ) );

	SVUT_ASSERT_EQUAL( ret, 0 );


	Hash_Tree_release( &htr, NULL );

}


SVUT_DECLARE_FLAT_TEST(Hash_Tree,Push_no_parent)
{
	MALP_Trace_crc64_init();
	struct Hash_Tree htr;
	
	Hash_Tree_init( &htr, sizeof(struct Request), sizeof(struct Payload) );

	struct Request req;
	struct Payload pl;

	req.id = 42;
	memset( &pl, rand(), sizeof(struct Payload) );


	struct Hash_Tree_node *node =  Hash_Tree_push( &htr, (void *)&req, (void *)&req, (void *)&pl );

	SVUT_ASSERT_NULL( node );

	Hash_Tree_release( &htr, NULL );

}


SVUT_DECLARE_FLAT_TEST(Hash_Tree,Push_parent_and_request)
{
	MALP_Trace_crc64_init();
	struct Hash_Tree htr;
	
	Hash_Tree_init( &htr, sizeof(struct Request), sizeof(struct Payload) );

	struct Request req, parent_req;
	struct Payload pl, parent_pl;

	req.id = 42;
	parent_req.id = 43;

	int seed1 = rand();
	int seed2 = rand();

	memset( &pl, seed1, sizeof(struct Payload) );
	memset( &parent_pl, seed2, sizeof(struct Payload) );

	struct Hash_Tree_node *node =  Hash_Tree_push( &htr, NULL, (void *)&parent_req, (void *)&parent_pl );
	SVUT_ASSERT_NOT_NULL( node );
	
	node =  Hash_Tree_push( &htr, (void *)&parent_req, (void *)&req, (void *)&pl );
	SVUT_ASSERT_NOT_NULL( node );

	/* Check parent  and child */
	struct Hash_Tree_node *node_parent = Hash_Tree_get_elem( &htr, (void *)&parent_req );
	SVUT_ASSERT_NOT_NULL( node_parent );
	struct Hash_Tree_node *node_req = Hash_Tree_get_elem( &htr, (void *)&req );
	SVUT_ASSERT_NOT_NULL( node_req );

	/* Check content */

	int ret = memcmp( &parent_pl, node_parent->payload, sizeof(sizeof(struct Payload) ) );
	SVUT_ASSERT_EQUAL( ret, 0 );

	ret = memcmp( &pl, node_req->payload, sizeof(sizeof(struct Payload) ) );
	SVUT_ASSERT_EQUAL( ret, 0 );

	/* Check linking */

	SVUT_ASSERT_EQUAL( node_req->parent, node_parent );	
	SVUT_ASSERT_EQUAL( Hash_Tree_node_get_child_count(node_parent), 1 );
	SVUT_ASSERT_EQUAL( Hash_Tree_node_get_nth_child(node_parent, 0), node_req );


	Hash_Tree_release( &htr, NULL );

}


