/* ############################ MALP License ############################## */
/* # Fri Jan 18 14:00:00 CET 2013                                         # */
/* # Copyright or (C) or Copr. Commissariat a l'Energie Atomique          # */
/* #                                                                      # */
/* # This software is governed by the CeCILL-C license under French law   # */
/* # and abiding by the rules of distribution of free software.  You can  # */
/* # use, modify and/ or redistribute the software under the terms of     # */
/* # the CeCILL-C license as circulated by CEA, CNRS and INRIA at the     # */
/* # following URL http://www.cecill.info.                                # */
/* #                                                                      # */
/* # The fact that you are presently reading this means that you have     # */
/* # had knowledge of the CeCILL-C license and that you accept its        # */
/* # terms.                                                               # */
/* #                                                                      # */
/* # Authors:                                                             # */
/* #   - BESNARD Jean-Baptiste jean-baptiste.besnard@cea.fr               # */
/* #                                                                      # */
/* ######################################################################## */
#include <svut.h>
#include <iostream>
#include <stdint.h>

using namespace svUnitTest;

#include "Data_Entry.h"

using namespace std;


SVUT_DECLARE_FLAT_TEST(Data_entry,Data_entry_init)
{

	void *dat =malloc(1024*1024);
	
	int i = 0;
	
	for( i = 0 ; i < 1024*1024 ; i++ )
	{
		((char*)dat)[i] = i%255;
	}
	
	
	SVUT_ASSERT_NOT_NULL( dat );
	
	struct Data_entry *ent = Data_entry_init( 123, dat, 1024*1024 );

	SVUT_ASSERT_NOT_NULL( ent );

	free( dat );
	
	SVUT_ASSERT_EQUAL(123u,ent->key);
	SVUT_ASSERT_EQUAL(0u,ent->ref_count);
	SVUT_ASSERT_EQUAL(1024u*1024u,ent->size);
	
	for( i = 0 ; i < 1024*1024 ; i++ )
	{
		SVUT_ASSERT_EQUAL(i%255, (unsigned char)ent->payload[i]);
	}
	
	Data_entry_release( ent );
	
}

SVUT_DECLARE_FLAT_TEST(Data_entry,Data_entry_release)
{

	void *dat =malloc(1024*1024);

	SVUT_ASSERT_NOT_NULL( dat );
	
	struct Data_entry *ent = Data_entry_init( 123, dat, 1024*1024 );

	SVUT_ASSERT_NOT_NULL( ent );

	free( dat );

	Data_entry_release( ent );
	
	/* Pointer is freed cannot check ! ... */
	
}

SVUT_DECLARE_FLAT_TEST(Data_entry,Data_entry_clone)
{

	void *dat =malloc(1024*1024);
	
	int i = 0;
	
	for( i = 0 ; i < 1024*1024 ; i++ )
	{
		((char*)dat)[i] = i%255;
	}
	
	SVUT_ASSERT_NOT_NULL( dat );
	
	struct Data_entry *ent = Data_entry_init( 123, dat, 1024*1024 );

	SVUT_ASSERT_NOT_NULL( ent );

	free( dat );
	
	struct Data_entry *new_ent = Data_entry_clone( ent );
	
	
	SVUT_ASSERT_EQUAL(ent->key,new_ent->key);
	SVUT_ASSERT_EQUAL(ent->size,new_ent->size);
	
	for( i = 0 ; i < 1024*1024 ; i++ )
	{
		SVUT_ASSERT_EQUAL((unsigned char)ent->payload[i], (unsigned char)new_ent->payload[i]);
	}

	Data_entry_release( ent );
	Data_entry_release( new_ent );
}

