/* ############################ MALP License ############################## */
/* # Fri Jan 18 14:00:00 CET 2013                                         # */
/* # Copyright or (C) or Copr. Commissariat a l'Energie Atomique          # */
/* #                                                                      # */
/* # This software is governed by the CeCILL-C license under French law   # */
/* # and abiding by the rules of distribution of free software.  You can  # */
/* # use, modify and/ or redistribute the software under the terms of     # */
/* # the CeCILL-C license as circulated by CEA, CNRS and INRIA at the     # */
/* # following URL http://www.cecill.info.                                # */
/* #                                                                      # */
/* # The fact that you are presently reading this means that you have     # */
/* # had knowledge of the CeCILL-C license and that you accept its        # */
/* # terms.                                                               # */
/* #                                                                      # */
/* # Authors:                                                             # */
/* #   - BESNARD Jean-Baptiste jean-baptiste.besnard@cea.fr               # */
/* #                                                                      # */
/* ######################################################################## */
#include <svut.h>
#include <iostream>
#include <stdint.h>

using namespace svUnitTest;

#include "MALP_Blackboard.h"
#include "Hash_Table.h"

using namespace std;

SVUT_DECLARE_FLAT_TEST(MALP_Blackboard,MALP_blackboard_init)
{
	struct MALP_blackboard bb;
	
	MALP_blackboard_init( &bb, 2, NULL );
	
	
	SVUT_ASSERT_EQUAL( sizeof( struct Ks_cell ), bb.ks_table.sizeof_payload );
	SVUT_ASSERT_EQUAL( sizeof( struct Key_cell ), bb.key_table.sizeof_payload );

	SVUT_ASSERT_EQUAL( KS_COLL_AVOID, bb.ks_table.coll_avoid );
	SVUT_ASSERT_EQUAL( KEY_COLL_AVOID, bb.key_table.coll_avoid );

	SVUT_ASSERT_EQUAL( 2u, bb.workers.worker_count );

	SVUT_ASSERT_EQUAL( 0u, bb.lock );

	MALP_blackboard_release( &bb );
}


SVUT_DECLARE_FLAT_TEST(MALP_Blackboard,MALP_blackboard_release)
{
	struct MALP_blackboard bb;
	
	MALP_blackboard_init( &bb, 0, NULL );
	MALP_blackboard_release( &bb );

	SVUT_ASSERT_EQUAL( 0u, bb.ks_table.sizeof_payload );
	SVUT_ASSERT_EQUAL( 0u, bb.key_table.sizeof_payload );

	SVUT_ASSERT_EQUAL( 0u, bb.workers.worker_count );

	SVUT_ASSERT_EQUAL( 1u, bb.lock );
}


void MALPBB_process_ks(  struct MALP_blackboard *bb, struct Data_entry **de, uint32_t entry_count, void *arg )
{

}



SVUT_DECLARE_FLAT_TEST(MALP_Blackboard,MALP_Blackboard_new_ks)
{
	char pl1[200];
	memset( pl1, 41, 200);

	char pl2[200];
	memset( pl2, 42, 200);


	uint64_t sens_list[2] =  { MALP_Blackboard_type_hash( 456 ), MALP_Blackboard_type_state_hash( 12, 345 ) };

	struct MALP_blackboard bb;

	MALP_blackboard_init( &bb, 0, NULL);

	MALP_Blackboard_new_ks( &bb, 1, sens_list, 2, MALPBB_process_ks, NULL );

	/* Check KS content */
	struct Knowledge_system *ks = (struct Knowledge_system *) Hash_Table_get( &bb.ks_table, 1, NULL, knowledge_system_test_func);
	SVUT_ASSERT_NOT_NULL( ks );
	
	SVUT_ASSERT_EQUAL( 1u, ks->type );
	SVUT_ASSERT_EQUAL( 2u, ks->sensivity_count);
	SVUT_ASSERT_EQUAL( MALP_Blackboard_type_hash( 456 ), ks->sensivity_keys[0] );
	SVUT_ASSERT_EQUAL( MALP_Blackboard_type_state_hash( 12, 345 ), ks->sensivity_keys[1] );
	SVUT_ASSERT_EQUAL( MALPBB_process_ks, ks->process );

	/* Check KE Type content */
	struct Key_entry *ke = (struct Key_entry *) Hash_Table_get( &bb.key_table, MALP_Blackboard_type_hash( 456 ), NULL, key_entry_test_func);
	SVUT_ASSERT_NOT_NULL( ks );
	SVUT_ASSERT_EQUAL( 1, Key_entry_has_KS_type( ke, 1) );

	ke = (struct Key_entry *) Hash_Table_get( &bb.key_table, MALP_Blackboard_type_state_hash( 12, 345 ), NULL, key_entry_test_func);
	SVUT_ASSERT_NOT_NULL( ks );
	SVUT_ASSERT_EQUAL( 1, Key_entry_has_KS_type( ke, 1) );

	MALP_blackboard_release( &bb );


}


int mpc_pbb_counter = 0;
MALP_Spinlock mpc_pbb_lock;



void MALPBB_process_ks_counter(  struct MALP_blackboard *bb, struct Data_entry **de, uint32_t entry_count, void *arg  )
{
	MALP_Spinlock_lock( &mpc_pbb_lock );
	mpc_pbb_counter++;
	MALP_Spinlock_unlock( &mpc_pbb_lock );
}


SVUT_DECLARE_FLAT_TEST(MALP_Blackboard,MALP_Blackboard_submit_data)
{
	char pl1[200];
	memset( pl1, 41, 200);

	char pl2[200];
	memset( pl2, 42, 200);

	uint64_t sens_list[2] =  { 456 , 12 };

	struct MALP_blackboard bb;

	MALP_blackboard_init( &bb, 0, NULL );

	MALP_Blackboard_new_ks( &bb, 1, sens_list, 2, MALPBB_process_ks_counter, NULL );

	int exp = 0;

	while( exp < 10 )
	{
		mpc_pbb_counter = 0;
		int num = rand()%5000;

		printf("EXP : %d NUM : %d\n", exp, num );

		int i = 0;
		for( i = 0 ; i < num ; i++ )
		{
			MALP_Blackboard_submit_data( &bb, 456, &pl1, 200 );
			MALP_Blackboard_submit_data( &bb, 12, &pl2, 200 );
		}

		MALP_blackboard_wait( &bb );

		SVUT_ASSERT_EQUAL( num, mpc_pbb_counter );
		exp++;
	}


	MALP_blackboard_release( &bb );


}


SVUT_DECLARE_FLAT_TEST(MALP_Blackboard,MALP_Blackboard_submit_data_similar)
{
	char pl1[200];
	memset( pl1, 41, 200);

	char pl2[200];
	memset( pl2, 42, 200);

	uint64_t sens_list[2] =  { 456 , 456 };

	struct MALP_blackboard bb;

	MALP_blackboard_init( &bb, 0, NULL );

	MALP_Blackboard_new_ks( &bb, 1, sens_list, 2, MALPBB_process_ks_counter, NULL );

	int exp = 0;

	while( exp < 10 )
	{
		mpc_pbb_counter = 0;
		int num = rand()%5000;

		//printf("EXP : %d NUM : %d\n", exp, num );

		int i = 0;
		for( i = 0 ; i < num ; i++ )
		{
			MALP_Blackboard_submit_data( &bb, 456, &pl1, 200 );
			MALP_Blackboard_submit_data( &bb, 456, &pl2, 200 );
		}

		MALP_blackboard_wait( &bb );

		SVUT_ASSERT_EQUAL( num, mpc_pbb_counter );
		exp++;
	}


	MALP_blackboard_release( &bb );

}

int self_push_cnt = 0;


void MALPBB_process_ks_selfpush(  struct MALP_blackboard *bb, struct Data_entry **de, uint32_t entry_count, void *arg )
{
	MALP_Spinlock_lock( &mpc_pbb_lock );

	self_push_cnt++;

	int pl = *( (int *)Data_entry_get_payload(de[0]) );
	
	pl--;
	
	if( 0 < pl )
	{
		MALP_Blackboard_submit_data( bb, 123, &pl, sizeof(int) );
	}
	else
	{
		MALP_Blackboard_submit_data( bb, 456, NULL, 0 );
	}

	MALP_Spinlock_unlock( &mpc_pbb_lock );
}

int seen_last_elem = 0;

void MALPBB_process_ks_last_elem(  struct MALP_blackboard *bb, struct Data_entry **de, uint32_t entry_count, void *arg  )
{
	MALP_Spinlock_lock( &mpc_pbb_lock );
	seen_last_elem = 1;
	MALP_Spinlock_unlock( &mpc_pbb_lock );
}



SVUT_DECLARE_FLAT_TEST(MALP_Blackboard,MALP_Blackboard_submit_data_selfpush)
{
	uint64_t sens_list[1] = {  123  };
	uint64_t sens_list2[2] = {  456 , 789 };

	struct MALP_blackboard bb;

	MALP_blackboard_init( &bb, 0, NULL);

	MALP_Blackboard_new_ks( &bb, 1, sens_list, 1, MALPBB_process_ks_selfpush, NULL);
	MALP_Blackboard_new_ks( &bb, 2, sens_list2, 2, MALPBB_process_ks_last_elem, NULL );

	int exp = 0;
	printf("BEGIN EXP \n");
	while( exp < 10 )
	{
		/* End token */
		MALP_Blackboard_submit_data( &bb, 789, NULL, 0 );

		self_push_cnt = 0;
		seen_last_elem = 0;
		int num = rand()%5000 + 1;

		printf("EXP : %d NUM : %d\n", exp, num );

		MALP_Blackboard_submit_data( &bb, 123, &num, sizeof(int) );

		MALP_blackboard_wait( &bb );

		SVUT_ASSERT_EQUAL( num, self_push_cnt );
		SVUT_ASSERT_EQUAL( 1, seen_last_elem );
		exp++;
	}

	MALP_blackboard_release( &bb );


}

