/* ############################ MALP License ############################## */
/* # Fri Jan 18 14:00:00 CET 2013                                         # */
/* # Copyright or (C) or Copr. Commissariat a l'Energie Atomique          # */
/* #                                                                      # */
/* # This software is governed by the CeCILL-C license under French law   # */
/* # and abiding by the rules of distribution of free software.  You can  # */
/* # use, modify and/ or redistribute the software under the terms of     # */
/* # the CeCILL-C license as circulated by CEA, CNRS and INRIA at the     # */
/* # following URL http://www.cecill.info.                                # */
/* #                                                                      # */
/* # The fact that you are presently reading this means that you have     # */
/* # had knowledge of the CeCILL-C license and that you accept its        # */
/* # terms.                                                               # */
/* #                                                                      # */
/* # Authors:                                                             # */
/* #   - BESNARD Jean-Baptiste jean-baptiste.besnard@cea.fr               # */
/* #                                                                      # */
/* ######################################################################## */
#ifndef ANALYZER_BB_H
#define ANALYZER_BB_H


#include "MALP_Blackboard.h"
#include "Event_Desc.h"
#include "Event_Dispatcher.h"
#include "Report.h"


/**
 * @ingroup Analyzer_BB_
 * @brief Stores the access to the @ref Event_Dispatcher_ and @ref MPI_Report_ used by an analyser, its partition id and
 */
struct Analyzer_BB
{
    struct Event_Dispatcher disp;   /**< @brief current event dispatcher */
    struct MALP_blackboard bb;      /**< @brief current black board */
    uint64_t partition_id;          /**< @brief MPI partition identifier */
    uint64_t max_flying_packs;      /**< @brief Maximum number of flying packets */
    struct Report *report;
};

/**
 * @ingroup Analyzer_BB_
 * @brief Initialisation of the Analyser by associating a BlackBoard registering it to the event dispatcher, and by initialising the report rendering environment
 *
 * Detailed initialisation steps:
 * -Register Meta data handling for the event dispatcher //Event_Dispatcher_register( &bb->disp, MALP_EVENT_META, _Meta_Data_Handler_push , NULL );
 * -Register Event sink (Event dispatcher)
 * -Blackboard initialisation
 * -Register unpacker KS and save dispatcher address
 * -Setup report rendering environment
 * -Time grid initialisation (if needed)
 * @param bb
 * @param max_pack_buff
 */
void Analyzer_BB_init( struct Analyzer_BB *bb, uint64_t max_pack_buff );

/**
 * @ingroup Analyzer_BB_
 * @brief Analyzer_BB's components closing and freeing (after packets releasing): Black board, rendering environment, time grid (if used)
 * @param bb
 */
void Analyzer_BB_release( struct Analyzer_BB *bb );

/**
 * @ingroup Analyzer_BB_
 * @brief (Unused) Wait for all packs to be processed, and send to the @ref MALP_Blackboard_ a stop signal for all event types and all partitions
 * @param bb
 */
void Analyzer_BB_read_done(struct Analyzer_BB *bb);

/**
 * @ingroup Analyzer_BB_
 * @brief Push an @link Event_Pack @endlink (see Event_Desc in @ref Data_Model_ ) in the @ref Analyzer_BB_ (see @ref MALP_Blackboard_)
 * @param bb
 * @param pack
 */
void Analyzer_BB_push_pack( struct Analyzer_BB *bb, struct Event_Pack *pack );


/**
 * @ingroup Analyzer_BB_
 * @brief Getter for the @ref Analyzer_BB_ from the current @ref Trace_Collector_
 * @return an @ref Analyzer_BB_ structure
 */
struct Analyzer_BB * Analyzer_BB();

#endif /* ANALYZER_BB_H */
