#include "Collector.h"

#include "Analyzer_BB.h"
#include "Event_Dispatcher.h"


void _Collector_push_event(struct MALP_Trace_Event *evt, void *pcoll)
{
	/* This is ineficient => FIND ANOTHER WAY */
	struct Collector *coll = (struct Collector *)pcoll;

	if( evt->partition_id != coll->partition_id )
		return;
	
	(coll->push_event)( evt, pcoll );
	
}

int Collector_partition_size( struct Collector *coll )
{
	VMPI_Partition_desc *target = VMPI_Get_desc_by_id(coll->partition_id);
	
	if( ! target )
	{
		return 0;
	}
	
	
	return target->size;
}


int Collector_init( struct Collector *coll, MALP_Trace_event_type type,
					uint64_t partition_id,
					char *name,
					int (*init_coll)( struct Collector * coll ),
					int (*release_coll)( struct Collector * coll ),
					void (*push_event)(struct MALP_Trace_Event *, void *),
					int (*reduce)( struct Collector * coll ),
					int (*accumulate)( struct Collector * coll ) )
{
	//printf("Initializing collector %s on partition %llu\n", name, (long long unsigned int)partition_id );
	
	coll->type = type;
	coll->partition_id = partition_id;
	coll->init_coll = init_coll;
	coll->release_coll = release_coll;
	coll->push_event = push_event;
	coll->reduce = reduce;
	coll->accumulate = accumulate;
	
	snprintf( coll->name, 128, "%s", name );

	/* WARNING handler is registered at report LEVEL */
	
	coll->internal_data = NULL;

	int rank;
	PMPI_Comm_rank( VMPI_Get_partition_comm(), &rank );
	
	if( rank == 0 )
	{
		/* Register the analysis */
		
		/* Here we want the analysis to be a root in the cache as only those can be commited independently */
		snprintf(coll->analysis_cache_name, 128, "%llu/%s", (long long unsigned int)partition_id, coll->name);
		
		/* Ceate the analysis object */
		coll->json = json_object();
		
		json_t *js_name = json_string( coll->name );
		
		json_object_set( coll->json , "name", js_name );
		
		json_decref( js_name );
		
		nodeSharedCache_set( &nsc, coll->analysis_cache_name, coll->json );
		
		json_decref( coll->json );

		
		/* Push in collectors */
		
		json_t * partition_table = nodeSharedCache_get( &nsc, "partitions" );
		
		if( !partition_table )
		{
			printf("Failed to locate partitions...\n");
			abort();
		}
		
		json_t * partition_entry = json_array_get( partition_table, partition_id );
		
		if( !partition_entry )
		{
			printf("Could not locate partition %llu in the partition table\n", (long long unsigned int)partition_id );
		}
		
		json_t * partition_collectors = json_object_get( partition_entry, "collectors" );
		
		if( !partition_collectors )
		{
			printf("Could not locate collectors in partitions \n");
			abort();
		}
		
		/* Register the collector note that we hold the ref on it */
		
		json_t * analysis_cache_name_json = json_string( coll->analysis_cache_name );
		
		json_array_push(partition_collectors, analysis_cache_name_json);
		
		json_decref( analysis_cache_name_json );
		
		json_decref( partition_collectors );
		json_decref( partition_entry );
		json_decref( partition_table );
		

		/* Update all the partition informations (final commit) */
		nodeSharedCache_commit( &nsc, "partitions" );
		nodeSharedCache_commit( &nsc, coll->analysis_cache_name );
	}
	
	return 0;
}




void Collector_release( struct Collector *coll )
{

	Collector_release_coll( coll );

	memset( coll, 0, sizeof( struct Collector ) );
	
}


void _Collector_release( void *coll )
{
	 Collector_release( (void *)coll );
}
