#include "POSIX_Profile.h"

#include "POSIX_Profile.h"
#include "MPI_Events.h"

#include "Meta_Data_Handler.h"
#include "Shared_Cache.h"
#include "Timer.h"

int  POSIX_Profile_init_coll( struct Collector * coll )
{
	coll->internal_data = malloc( sizeof( struct POSIX_Profile ) );
	
	if( !coll->internal_data )
	{
		perror("malloc");
		abort();
	}
	
	memset( coll->internal_data, 0, sizeof( struct POSIX_Profile ) );
	
	struct POSIX_Profile * prof = (struct POSIX_Profile *)coll->internal_data;
	
	Hash_Table_init( &prof->per_stream_profiles, sizeof(struct POSIX_Profile_acc_cell), 64, POSIX_PROF_COLL_AVOID );
	
	/* Create the stream array */
	int rank;
	PMPI_Comm_rank( VMPI_Get_partition_comm(), &rank );
	
	if( rank == 0 )
	{
		json_t * array = json_array();
		
		char uri[100];
		snprintf( uri, 100, "%s/streams", coll->analysis_cache_name);
		nodeSharedCache_set( &nsc, uri , array );
		
		json_decref(array);
	}
	
	return 0;
}


int  POSIX_Profile_release_coll( struct Collector * coll )
{
	struct POSIX_Profile * prof = (struct POSIX_Profile *)coll->internal_data;
	
	Hash_Table_release( &prof->per_stream_profiles, NULL );
	free( coll->internal_data );
	coll->internal_data = NULL;
	
	return 0;
}

int POSIX_Profile_acc_test(void *elem, uint64_t key, void *dummy)
{
	struct POSIX_Profile_acc *acc = (struct POSIX_Profile_acc *)elem;
	
	if( acc->associated_tid == key )
		return 1;
	
	return 0;
}


void POSIX_Profile_push_event(struct MALP_Trace_Event *event, void *pcoll)
{
	struct Collector *coll = (struct Collector *)pcoll;
	struct POSIX_Profile *prof = (struct POSIX_Profile *)coll->internal_data;


	if( event->type != MALP_EVENT_WRAPPED )
	{
		printf("BAD event type\n");
		return;
	}
	
	struct Wrapped_Event *wevent = &event->event.wrapped_event;
	int type = wevent->type;

	/* Update the global profile */
	
	prof->global_acc.hits[type]++;
	prof->global_acc.time[type]+= event->duration;
	prof->global_acc.size[type]+= wevent->size;
	
	/* Update the perrank profile */
	struct POSIX_Profile_acc *acc = Hash_Table_get( &prof->per_stream_profiles, event->tid , NULL, POSIX_Profile_acc_test);

	if(! acc )
	{
		struct POSIX_Profile_acc tmp;
		memset( &tmp, 0, sizeof( struct POSIX_Profile_acc ) );
		
		tmp.associated_tid = event->tid;
		
		acc = Hash_Table_push( &prof->per_stream_profiles, (void *)&tmp , event->tid );
	}

	acc->hits[type]++;
	acc->time[type]+= event->duration;
	acc->size[type]+= wevent->size;
}

int  POSIX_Profile_reduce( struct Collector * coll )
{
	uint64_t tmp_buff[T_POSIX_COUNT];
	
	struct POSIX_Profile *prof = (struct POSIX_Profile *)coll->internal_data;
	struct POSIX_Profile_acc *global_acc = &prof->global_acc;

	/* HITS */
	PMPI_Reduce(global_acc->hits, tmp_buff, T_POSIX_COUNT, MPI_UNSIGNED_LONG_LONG , MPI_SUM, 0, VMPI_Get_partition_comm());
	memcpy( global_acc->hits, tmp_buff, T_POSIX_COUNT * sizeof(uint64_t ) );


	/* TIME */
	PMPI_Reduce(global_acc->time, tmp_buff, T_POSIX_COUNT, MPI_UNSIGNED_LONG_LONG , MPI_SUM, 0, VMPI_Get_partition_comm());
	memcpy( global_acc->time, tmp_buff, T_POSIX_COUNT * sizeof(uint64_t ) );
	
	/* SIZE */
	PMPI_Reduce(global_acc->size, tmp_buff, T_POSIX_COUNT, MPI_UNSIGNED_LONG_LONG , MPI_SUM, 0, VMPI_Get_partition_comm());
	memcpy( global_acc->size, tmp_buff, T_POSIX_COUNT * sizeof(uint64_t ) );

	return 0;
}


json_t * gen_POSIX_Profile( struct POSIX_Profile_acc * acc, int render_min_max )
{
	json_t * profile = json_array();
	int i;
	for(i = 0 ; i < T_POSIX_COUNT ; i++ )
	{
		if( acc->hits[i] )
		{				

			json_t * function_entry = json_object();
			
			json_t * name = json_string( (char *)Wrapped_symbol_names[i] );

			json_t * hits = json_int( acc->hits[i] );
			
			json_t * time = json_real( (double)acc->time[i]/Timer_second() );

			json_t * size = json_real( acc->size[i] );

			json_object_set( function_entry, "name", name);
			
			json_object_set( function_entry, "hits", hits);
			
			json_object_set( function_entry, "time", time);
			
			
			json_object_set( function_entry, "size", size);
			
			
			json_decref(name);
			json_decref(hits);
			json_decref(time);
			json_decref(size);
			
			json_array_push( profile, function_entry );
			
			json_decref(function_entry);
			
		}
	}
	
	return profile;
}

void POSIX_Profile_render_per_rank_profile(void *pacc, void *pcoll)
{
	struct POSIX_Profile_acc *acc = (struct POSIX_Profile_acc *)pacc;

	json_t *profile = gen_POSIX_Profile( acc, 0 );
	struct Stream_Descriptor *stream_desc = Meta_Data_Handler_get_desc( acc->associated_tid );
	
	if( !stream_desc )
	{
		printf("Failed to locate associated stream desc\n");
		return;
	}

	
	/* Create an entry object to include the TID */	
	char entry_name[100];
	snprintf(entry_name, 100, "%ld/POSIX_Profile/streams/%lu", stream_desc->partition_id, acc->associated_tid );

	json_t * entry = json_object();
	
	char stid[100];
	snprintf( stid, 100, "%lu", acc->associated_tid );
	
	json_t * tid = json_string( stid );
	
	json_object_set( entry, "tid", tid );
	json_object_set( entry, "profile", profile );
	
	nodeSharedCache_reduce( &nsc, entry_name, entry);
	
	json_decref( profile );
	json_decref( tid );
	json_decref( entry );
}




int  POSIX_Profile_accumulate( struct Collector * coll )
{
	struct POSIX_Profile * prof = (struct POSIX_Profile *)coll->internal_data;
	struct POSIX_Profile_acc *global_acc = &prof->global_acc;

	int rank;
	PMPI_Comm_rank( VMPI_Get_partition_comm(), &rank );
	
	/* Each rank renders its own profiles */
	
	Hash_Table_walkthrought_2p( &prof->per_stream_profiles, (void *)coll, POSIX_Profile_render_per_rank_profile  );
	
	
	if( rank == 0 )
	{
	
		/* Do the JSON rendering */
		json_t * profile =  gen_POSIX_Profile( global_acc, 1 );
		
		char uri[100];
		snprintf(uri, 100, "%s/profile", coll->analysis_cache_name );
		nodeSharedCache_set( &nsc, uri, profile );
		
		json_decref( profile );
	}

	return 0;
}




void POSIX_Profile_init(struct Collector *coll, uint64_t partition_id)
{
		
	Collector_init( coll, MALP_EVENT_WRAPPED,
					partition_id,
					"POSIX_Profile",
					POSIX_Profile_init_coll,
					POSIX_Profile_release_coll,
					POSIX_Profile_push_event,
					POSIX_Profile_reduce,
					POSIX_Profile_accumulate );
	
}
