#ifndef TOPOLOGY_H
#define TOPOLOGY_H

#include <stdint.h>

#include "Event_Desc.h"
#include "Collector.h"
#include "Shared_Cache.h"
#include "Wrapped_Symbols.h"
#include "Hash_Table.h"
#include "Array.h"

enum
{
	COMM_MAT_T_MPI_SEND,
	COMM_MAT_T_MPI_ISEND,
	COMM_MAT_T_MPI_IBSEND,
	COMM_MAT_T_MPI_IRSEND,
	COMM_MAT_T_MPI_ISSEND,
	COMM_MAT_T_MPI_BSEND,
	COMM_MAT_T_MPI_SSEND,
	COMM_MAT_T_MPI_RSEND,
	COMM_MAT_T_MPI_RECV,
	COMM_MAT_T_MPI_IRECV,
	COMM_MAT_T_MPI_COUNT
};


static inline int MPI_Comm_translate_mpi_event_p2p_type( int type )
{
	if( type == T_MPI_SEND) 
		return COMM_MAT_T_MPI_SEND;
	if( type == T_MPI_ISEND )
		return COMM_MAT_T_MPI_ISEND;
	if( type == T_MPI_IBSEND )
		return COMM_MAT_T_MPI_IBSEND;
	if( type == T_MPI_IRSEND )
		return COMM_MAT_T_MPI_IRSEND;
	if( type == T_MPI_ISSEND )
		return COMM_MAT_T_MPI_ISSEND;
	if( type == T_MPI_BSEND)
		return COMM_MAT_T_MPI_BSEND;
	if( type == T_MPI_SSEND)
		return COMM_MAT_T_MPI_SSEND;
	if( type == T_MPI_RSEND )
		return COMM_MAT_T_MPI_RSEND;
	if( type == T_MPI_RECV )
		return COMM_MAT_T_MPI_RECV;
	if( type == T_MPI_IRECV )
		return COMM_MAT_T_MPI_IRECV;

	return -1;
}

static inline int MPI_Comm_translate_comm_mat_to_type( int type )
{
	if( type == COMM_MAT_T_MPI_SEND) 
		return T_MPI_SEND;
	if( type == COMM_MAT_T_MPI_ISEND )
		return T_MPI_ISEND;
	if( type == COMM_MAT_T_MPI_IBSEND )
		return T_MPI_IBSEND;
	if( type == COMM_MAT_T_MPI_IRSEND )
		return T_MPI_IRSEND;
	if( type == COMM_MAT_T_MPI_ISSEND )
		return T_MPI_ISSEND;
	if( type == COMM_MAT_T_MPI_BSEND)
		return T_MPI_BSEND;
	if( type == COMM_MAT_T_MPI_SSEND)
		return T_MPI_SSEND;
	if( type == COMM_MAT_T_MPI_RSEND )
		return T_MPI_RSEND;
	if( type == COMM_MAT_T_MPI_RECV )
		return T_MPI_RECV;
	if( type == COMM_MAT_T_MPI_IRECV )
		return T_MPI_IRECV;

	return -1;
}



struct Topology_acc
{
	struct Array hits;
	struct Array time;
	struct Array size;
	short filled;
};

void Topology_acc_init(struct Topology_acc *acc, uint64_t psize);
void Topology_acc_release(struct Topology_acc *acc);
void Topology_acc_reduce(struct Topology_acc *acc, uint64_t psize);

struct Topology
{
	struct Topology_acc acc[COMM_MAT_T_MPI_COUNT];
	uint64_t psize;
};



int  Topology_init_coll( struct Collector * coll );
int  Topology_release_coll( struct Collector * coll );
void Topology_push_event(struct MALP_Trace_Event *, void *);
int  Topology_reduce( struct Collector * coll );
int  Topology_accumulate( struct Collector * coll );

void Topology_init(struct Collector *coll, uint64_t partition_id);


#endif
