/* ############################ MALP License ############################## */
/* # Fri Jan 18 14:00:00 CET 2013                                         # */
/* # Copyright or (C) or Copr. Commissariat a l'Energie Atomique          # */
/* #                                                                      # */
/* # This software is governed by the CeCILL-C license under French law   # */
/* # and abiding by the rules of distribution of free software.  You can  # */
/* # use, modify and/ or redistribute the software under the terms of     # */
/* # the CeCILL-C license as circulated by CEA, CNRS and INRIA at the     # */
/* # following URL http://www.cecill.info.                                # */
/* #                                                                      # */
/* # The fact that you are presently reading this means that you have     # */
/* # had knowledge of the CeCILL-C license and that you accept its        # */
/* # terms.                                                               # */
/* #                                                                      # */
/* # Authors:                                                             # */
/* #   - BESNARD Jean-Baptiste jean-baptiste.besnard@cea.fr               # */
/* #                                                                      # */
/* ######################################################################## */
#ifndef EVENT_DISPATCHER
#define EVENT_DISPATCHER
#ifdef __cplusplus
extern "C"
{
#endif

#include <stdio.h>
#include <stdlib.h>
#include <stdint.h>
#include <string.h>
#include <assert.h>

#include "Chained_List.h"
#include "Event_Desc.h"


/**
 * @ingroup Event_Dispatcher_
 * @brief Define the Handler_Desc structure
 *
 * A handler descriptor is used to store the association of an MALP trace event and a function to call when the event is received
 */
struct Handler_Desc {
    void (*handler)(struct MALP_Trace_Event *, void *); /**< Handler for a @ref MALP_Trace_Event structure */
    void *arg;                                          /**< Pointer to arguments */
};

void Handler_Desc_init( struct Handler_Desc *disp, void (*handler)(struct MALP_Trace_Event *, void *), void *arg);
void Handler_Desc_release( struct Handler_Desc *disp );


/**
 * @ingroup Event_Dispatcher_
 * @brief Define the Event_Dispatcher structure for storing association between events and functions to call.
 */
struct Event_Dispatcher {
    struct Chained_List handler_lists[MALP_EVENT_TYPE_COUNT + 1];   /**< list of events handlers */
    int been_initialized;                                           /**< Initialisation flag */
};

/**
 * @ingroup Event_Dispatcher_
 * @brief Initialise an Event_Dispatcher structure
 * @param disp
 */
void Event_Dispatcher_init( struct Event_Dispatcher *disp );
/**
 * @ingroup Event_Dispatcher_
 * @brief Release a Event_Dispatcher structure
 * @param disp
 */
void Event_Dispatcher_release( struct Event_Dispatcher *disp );

/**
 * @ingroup Event_Dispatcher_
 * @brief Initialise an Event_Dispatcher structure
 * @param disp
 * @param type
 * @param handler handler for a @ref Handler_Desc structure
 * @param arg
 */
void Event_Dispatcher_register( struct Event_Dispatcher *disp, MALP_Trace_event_type type, void (*handler)(struct MALP_Trace_Event *, void *), void *arg);

/**
 * @brief Event_Dispatcher_has_handle 
 * @param disp
 * @param type
 * @return
 *
 * @todo Confirm if it works as intended.
 */
int Event_Dispatcher_has_handler( struct Event_Dispatcher *disp, MALP_Trace_event_type type );

/**
 * @ingroup Event_Dispatcher_
 * @brief Assign the number of event type as the value if MALP_EVENT_ANY
 */
#define MALP_EVENT_ANY MALP_EVENT_TYPE_COUNT

/**
 * @ingroup Event_Dispatcher_
 * @brief Call the handler associated to a MALP trace event in a Handler_desc (parameters are casted inside the function)(Internal use, see @ref Event_Dispatcher_call )
 * @param pelem Handler_Desc
 * @param pevent MALP_Trace_Event
 */
void __Event_Dispatcher_call(void * pelem, void * pevent);

/**
 * @ingroup Event_Dispatcher_
 * @brief Call the handler associated to a MALP trace event in the Handler_desc of a given event dispatcher
 * @param disp Event_Dispatcher
 * @param event MALP_Trace_Event
 */
static inline void Event_Dispatcher_call( struct Event_Dispatcher *disp, struct MALP_Trace_Event *event )
{
	if(MALP_EVENT_TYPE_COUNT <= event->type)
	{
		printf("Erroneous event type\n");
		return;
	}

	Chained_List_walkthrough_arg( &disp->handler_lists[(int)event->type], __Event_Dispatcher_call, event);
	Chained_List_walkthrough_arg( &disp->handler_lists[MALP_EVENT_ANY], __Event_Dispatcher_call, event);
}


#ifdef __cplusplus
}
#endif



#endif /* EVENT_DISPATCHER */
