/* ############################ MALP License ############################## */
/* # Fri Jan 18 14:00:00 CET 2013                                         # */
/* # Copyright or (C) or Copr. Commissariat a l'Energie Atomique          # */
/* #                                                                      # */
/* # This software is governed by the CeCILL-C license under French law   # */
/* # and abiding by the rules of distribution of free software.  You can  # */
/* # use, modify and/ or redistribute the software under the terms of     # */
/* # the CeCILL-C license as circulated by CEA, CNRS and INRIA at the     # */
/* # following URL http://www.cecill.info.                                # */
/* #                                                                      # */
/* # The fact that you are presently reading this means that you have     # */
/* # had knowledge of the CeCILL-C license and that you accept its        # */
/* # terms.                                                               # */
/* #                                                                      # */
/* # Authors:                                                             # */
/* #   - BESNARD Jean-Baptiste jean-baptiste.besnard@cea.fr               # */
/* #                                                                      # */
/* ######################################################################## */

#ifndef RED_HT_H
#define RED_HT_H

#include <stdio.h>
#include <stdint.h>
#include "Hash_Table.h"

/**
 * @ingroup Red_Ht_
 * @brief The Red_Ht structure
 */
struct Red_Ht
{
	struct Hash_Table ht;       /**< @brief the actual Hash_Table */
	uint64_t internal_elem_size;/** @brief the number of elements int the Hash_Table */
};


/**
 * @ingroup Red_Ht_
 * @brief Red_Ht_init
 * @param rht
 * @param payload_size
 * @param table_size
 * @param coll_avoid
 */
void Red_Ht_init( struct Red_Ht *rht, size_t payload_size, uint64_t table_size, uint64_t coll_avoid);
/**
 * @ingroup Red_Ht_
 * @brief Red_Ht_clone
 * @param rht
 * @param source
 */
void Red_Ht_clone(struct Red_Ht *rht, struct Red_Ht *source);
/**
 * @ingroup Red_Ht_
 * @brief Red_Ht_release
 * @param rht
 */
void Red_Ht_release(struct Red_Ht *rht, void (*free_f)(void *) );
/**
 * @ingroup Red_Ht_
 * @brief Red_Ht_reduce
 * @param rht
 * @param key_func
 */
void Red_Ht_reduce(struct Red_Ht *rht, void (*red_func)(void * ,void *), int (*test_func)(void *, uint64_t , void *),
					uint64_t key_func(void *), void (*free_f)(void *)  );

/**
 * @ingroup Red_Ht_
 * @brief Red_Ht_serialize
 * @param rht
 * @param elem_count
 * @param elem_size
 * @return
 */
void * Red_Ht_serialize(struct Red_Ht *rht, uint64_t *elem_count, uint64_t *elem_size);


/**
 * @ingroup Red_Ht_
 * @brief Red_Ht_Table_get
 * @param rht
 * @param key
 * @param test_elem
 * @return
 */
static inline void * Red_Ht_Table_get( struct Red_Ht *rht, uint64_t key , void *test_elem, int (*test_func)(void *, uint64_t , void *))
{
	return Hash_Table_get( &rht->ht, key , test_elem, test_func );
}

/**
 * @ingroup Red_Ht_
 * @brief Red_Ht_Table_push
 * @param rht
 * @param payload
 * @param key
 * @return
 */
static inline void * Red_Ht_Table_push(  struct Red_Ht *rht, void *payload , uint64_t key )
{
	return Hash_Table_push( &rht->ht, payload ,  key );
}

/**
 * @ingroup Red_Ht_
 * @brief Red_Ht_Table_get_an_entry
 * @param rht
 * @return
 */
static inline void* Red_Ht_Table_get_an_entry( struct Red_Ht *rht )
{
	return Hash_Table_get_an_entry( &rht->ht );
}

/**
 * @ingroup Red_Ht_
 * @brief Red_Ht_Table_push_unique
 * @param rht
 * @param payload
 * @param key
 * @param extra_arg
 * @return
 */
static inline void * Red_Ht_Table_push_unique( struct Red_Ht *rht, void *payload , uint64_t key, void *extra_arg, int (*test_func)(void *, uint64_t, void * ) )
{
	return Hash_Table_push_unique( &rht->ht, payload , key, extra_arg, test_func );
}


#endif /* RED_HT_H */
