#include "Report.h"

#include "Shared_Cache.h"
#include "VMPI.h"
#include "Analyzer_BB.h"

int Report_init( struct Report *cll )
{
	Chained_List_init( &cll->collectors, 16, sizeof(struct Collector ) );
	
	return 0;
}

void Report_release( struct Report *cll )
{
	 Chained_List_walkthrough(&cll->collectors, _Collector_release);
	 Chained_List_release( &cll->collectors );
}

int Report_register( struct Report *cll, struct Collector *cl )
{
	struct Collector *ret =  (struct Collector *)Chained_List_push( &cll->collectors, (void *)cl );
	
	/* Here we register the handler to the actual pointer */
	if( (ret->type != MALP_EVENT_NONE) && (ret->push_event != NULL) )
	{
		Event_Dispatcher_register( &Analyzer_BB()->disp, ret->type, _Collector_push_event, (void *)ret);
	}
	
	return 0;
}

int Report_init_coll( struct Report *cll )
{
	Chained_List_walkthrough(&cll->collectors, Collector_init_coll);
	
	return 0;
}


int Report_release_coll( struct Report *cll )
{
	Chained_List_walkthrough(&cll->collectors, Collector_release_coll);
	
	return 0;
}

int Report_reduce( struct Report *cll )
{
	Chained_List_walkthrough(&cll->collectors, Collector_reduce);
	
	return 0;
}

int Report_accumulate( struct Report *cll )
{
	Chained_List_walkthrough(&cll->collectors, Collector_accumulate);
	
	return 0;
}


/* Here we register modules and partitions */

int Report_populate( struct Report *cll, uint64_t partition_id )
{
	struct Collector cl;
	
	MPI_Profile_init(&cl, partition_id);
	Report_register( cll, &cl );
	
	POSIX_Profile_init(&cl, partition_id);
	Report_register( cll, &cl );

	Topology_init(&cl, partition_id);
	Report_register( cll, &cl );

	Time_Matrix_init(&cl, partition_id);
	Report_register( cll, &cl );
	
	Flat_Matrix_init(&cl, partition_id);
	Report_register( cll, &cl );
	
	OTF2_init(&cl, partition_id);
	Report_register( cll, &cl );
	
	Rank_Matrix_init(&cl, partition_id);
	Report_register( cll, &cl );
	
	//~ //Printer_init(&cl, partition_id);
	//~ //Report_register( cll, &cl );
	//~ 


	return 0;
}
