#include "Shared_Cache.h"
#include "Binary_Tree.h"

#include "VMPI.h"

#include <unistd.h>

struct nodeSharedCache nsc;

static int running_in_trace_mode = 0;

int Shared_Cache_is_in_trace_mode()
{
	return running_in_trace_mode;
}


void Shared_Cache_init()
{
	int rank;
	PMPI_Comm_rank( VMPI_Get_partition_comm(), &rank );
	int size;
	PMPI_Comm_size( VMPI_Get_partition_comm(), &size );
	
	
	int proxy_port = 0;
	char hostname[100];	
	char proxyadress[100];
	int ret = 0;
	int trials = 0;	

	if( gethostname(hostname, 100) == -1 )
	{
		perror("gethostname");
		abort();
	}
	
	/* Lets build the cache binary tree */
	struct Binary_Tree bt;
	Binary_Tree_init(&bt, rank, size);
	
	//printf("%d P %d LC %d RC %d CC %d\n", rank, Binary_Tree_parent(&bt), Binary_Tree_left_child(&bt), Binary_Tree_right_child(&bt), Binary_Tree_child_count(&bt));
	
	
	if( 0 <= Binary_Tree_parent(&bt) )
	{
		PMPI_Recv( proxyadress, 100, MPI_CHAR, Binary_Tree_parent(&bt), 0, VMPI_Get_partition_comm(), MPI_STATUS_IGNORE );
		
			
		//printf("%d will connect to %s\n", rank, proxyadress );
		
		do
		{
			/* We wait 10 seconds for the cache to start ... */
			ret = nodeSharedCache_proxy_init( &nsc, proxyadress, Binary_Tree_child_count(&bt) /*Expected clients */ );
			
			if( ret == -1 )
				sleep(1);
				
			trials++;
		}while( (ret == -1) && trials < 10 );
		
		proxy_port = ret;
	}
	else
	{
		 /*
		 * INIT NodeSharedCache for root proxy (connected to node.js)
		 */
		char * node_server_addr = getenv("NODE_SHARED_CACHE");

		if( !node_server_addr )
		{
			printf("INFO : Runtime analysis server not defined\n");
			running_in_trace_mode = 1;
		}
		else
		{
			printf("Analysis connecting to %s\n", node_server_addr );
		}
	
		do
		{
			/* We wait 10 seconds for the cache to start ... */
			ret = nodeSharedCache_proxy_init( &nsc, node_server_addr, Binary_Tree_child_count(&bt) /*Expected clients */ );
			
			if( ret == -1 )
				sleep(1);
	
			trials++;
		}while( (ret == -1) && trials < 10 );
		
		/* We now have a proxy port */
		proxy_port = ret;
	}

	/* Fill in the address for children */
	snprintf( proxyadress, 100, "%s:%d", hostname, proxy_port );
	//printf("%d ========> %s\n", rank , proxyadress ); 	
	
	if( 0 <= Binary_Tree_left_child(&bt) )
	{
		PMPI_Send( proxyadress, 100, MPI_CHAR, Binary_Tree_left_child(&bt), 0, VMPI_Get_partition_comm() );
		
	}
	
	if( 0 <= Binary_Tree_right_child(&bt) )
	{
		PMPI_Send( proxyadress, 100, MPI_CHAR, Binary_Tree_right_child(&bt), 0, VMPI_Get_partition_comm() );
	}
	
	/* Now accept children */
	nodeSharedCache_proxy_accept_clients( &nsc );
	
	
	Binary_Tree_release( &bt );
}


void Shared_Cache_to_file( char *filename )
{
	FILE *trace = fopen( filename, "w" );
	
	if( !trace )
	{
		perror("fopen");
		printf("Failed to open trace file %s\n", filename );
		abort();
	}
	
	json_dump_f(trace, nsc.cache.cache, JSON_PRETTY );
	
}

void Shared_Cache_wait()
{
	nodeSharedCache_wait( &nsc );
}


void Shared_Cache_release()
{
	nodeSharedCache_release( &nsc );
}
