/* ############################ MALP License ############################## */
/* # Fri Jan 18 14:00:00 CET 2013                                         # */
/* # Copyright or (C) or Copr. Commissariat a l'Energie Atomique          # */
/* #                                                                      # */
/* # This software is governed by the CeCILL-C license under French law   # */
/* # and abiding by the rules of distribution of free software.  You can  # */
/* # use, modify and/ or redistribute the software under the terms of     # */
/* # the CeCILL-C license as circulated by CEA, CNRS and INRIA at the     # */
/* # following URL http://www.cecill.info.                                # */
/* #                                                                      # */
/* # The fact that you are presently reading this means that you have     # */
/* # had knowledge of the CeCILL-C license and that you accept its        # */
/* # terms.                                                               # */
/* #                                                                      # */
/* # Authors:                                                             # */
/* #   - BESNARD Jean-Baptiste jean-baptiste.besnard@cea.fr               # */
/* #                                                                      # */
/* ######################################################################## */

/**
 * @addtogroup Binary_Tree_
 * @{
 */

#ifndef BINARY_TREE_H
#define BINARY_TREE_H

#ifdef __cplusplus
extern "C" {
#endif

/**
 * @brief Struct defining a binary tree
 *
 */
struct Binary_Tree
{
	int rank;           /**< @brief The rank of the element */
	int size;           /**< @brief The size of the element */
	int left_child;     /**< @brief The rank of the left child */
	int right_child;    /**< @brief The rank of the right child */
	int parent;         /**< @brief The rank of the parent */
};


/**
 * @brief Initialization of a binary tree
 * @param bt the tree to initialize
 * @param rank the rank of initialized tree 
 * @param size the size of the tree 
 */
void Binary_Tree_init(struct Binary_Tree *bt, int rank, int size);

/**
 * @brief releases a binary tree
 * @param bt the binary tree to release
 */
void Binary_Tree_release(struct Binary_Tree *bt);

/**
 * @brief Getter on the parent
 * @param bt the Binary_Tree
 * @return the parent of given Binary_Tree
 */
static inline int Binary_Tree_parent(struct Binary_Tree *bt)
{
	return bt->parent;
}

/**
 * @brief Getter on the left child
 * @param bt the Binary_Tree
 * @return the left_child of given Binary_Tree
 */
static inline int Binary_Tree_left_child(struct Binary_Tree *bt)
{
	return bt->left_child;
}

/**
 * @brief Getter on the right child
 * @param bt the Binary_Tree
 * @return the right_child of given Binary_Tree
 */
static inline int Binary_Tree_right_child(struct Binary_Tree *bt)
{
	return bt->right_child;
}

static inline int Binary_Tree_child_count(struct Binary_Tree *bt)
{
	int ret = 0;
	
	if( 0 <=  Binary_Tree_left_child(bt) )
		ret ++;
	
	if( 0 <=  Binary_Tree_right_child(bt) )
		ret ++;
	
	return ret;
}


#ifdef __cplusplus
}
#endif


#endif /* BINARY_TREE_H */

/**
 * @}
 */
