#ifndef COLOR_SCALE_H
#define COLOR_SCALE_H


typedef enum
{
	SCALE_TEMPERATURE,
	SCALE_FIRE,
	SCALE_ICE,
	SCALE_GREYSCALE,
	SCALE_THREE_LVL,
	SCALE_THREE_LVL_LINEAR,
	SCALE_THREE_LVL_LINEAR_NARROW,
	SCALE_GOOD_AND_EVIL,
	SCALE_COUNT
}Color_Scale_t;


static const char * const Color_Scale_names[SCALE_COUNT] =
{
	"temperature",
	"fire",
	"ice",
	"greyscale",
	"three levels",
	"three levels linear",
	"three levels linear narrow",
	"good and evil"
};


Color_Scale_t Color_Scale_from_name( char * name );

 
/**
 * @ingroup Grid_Handler_Color_
 * @brief Color structure containing RGB values.
 */
struct Color {
    int r;  /**< Red */
    int g;  /**< Green */
    int b;  /**< Blue */
};




/**
 * @ingroup Grid_Handler_Color_
 * @brief factor_to_color
 * @param col
 * @param factor
 */
static inline void factor_to_color( struct Color *col, double factor, Color_Scale_t scale_type )
{
	double h=factor * 255;
	double f = 0;

	switch( scale_type ) {
		case SCALE_TEMPERATURE : // TEMPERATURE
			if ( h < 60 ) {
				f = h /60.0;
				col->b = 255;

				col->g = f * 255;
				col->r = 0;

			}

			if ( 60 < h && h <= 120 ) {
				f = (h /60.0) - 1;
				col->b = (1 -f) *255;
				col->g = 255;
				col->r = 0;

			}

			if ( 120 < h && h <= 180 ) {
				f = (h /60.0) - 2;
				col->b = 0;
				col->g = 255;
				col->r = f * 255;

			}

			if ( h > 180) {
				f = ((h /60.0) - 3)/1.25;
				col->b = 0;
				col->g = (1-f)*255;
				col->r = 255;

			}
			break;
		case SCALE_FIRE : // FIRE
			if( factor <= 0.5 ) {
				col->r = 255 * factor;
				col->g = 0;
				col->b = 0;
			} else {
				col->r = 255 * factor;
				col->g = (255 * (factor - 0.5)) * 1.8;
				col->b = 0;
			}
			break;
		case SCALE_ICE : // ICE
			col->r = 0;
			col->g = 255 * factor;
			col->b = 255 * factor;
			break;
		case SCALE_GREYSCALE : //GREYSCALE
			col->r = 255 * (1-factor);
			col->g = 255 * (1-factor);
			col->b = 255 * (1-factor);
			break;
		case SCALE_THREE_LVL : //THREE LEVELS STRICT
			col->r = 0;
			col->g = 0;
			col->b = 0;

			if( factor < 0.3333 ) {
				col->b = 255;
			} else if( 0.6666 < factor ) {
				col->r = 255;
			} else {
				col->r = 128;
				col->g = 128;
				col->b = 128;
			}
			break;
		case SCALE_THREE_LVL_LINEAR : //THREE LEVELS LINEAR LARGE AVG
			col->r = 0;
			col->g = 0;
			col->b = 0;

			if( factor < 0.3333 ) {
				col->r = 0;
				col->g = 0;
				col->b = 255 * (1-(factor*3));
			} else if( 0.6666 < factor ) {
				col->r = 0;
				col->g = 0;
				col->r = 255 * (( factor - 0.6666 ) * 3);
			} else {
				col->r = 0;
				col->g = 0;
				col->b = 0;
			}
			break;
		case SCALE_THREE_LVL_LINEAR_NARROW : //THREE LEVELS LINEAR NARROW AVG
			col->r = 0;
			col->g = 0;
			col->b = 0;

			if( factor < 0.4705 ) {
				col->r = 0;
				col->g = 0;
				col->b = 255 * (1-(factor*2.125));
			} else if( 0.5294 < factor ) {
				col->r = 0;
				col->g = 0;
				col->r = 255 * (( factor - 0.5294 ) * 2.125);
			} else {
				col->r = 0;
				col->g = 0;
				col->b = 0;
			}
			break;
		case SCALE_GOOD_AND_EVIL : //GOOD AND EVIL
			col->r = 0;
			col->g = 0;
			col->b = 0;

			if( factor < 0.5 ) {
				col->b = 255;
			} else {
				col->r = 255;
			}

			break;
		default :
			f = 0;
	}


}

















#endif /* COLOR_SCALE_H */
