/* ############################ MALP License ############################## */
/* # Fri Jan 18 14:00:00 CET 2013                                         # */
/* # Copyright or (C) or Copr. Commissariat a l'Energie Atomique          # */
/* #                                                                      # */
/* # This software is governed by the CeCILL-C license under French law   # */
/* # and abiding by the rules of distribution of free software.  You can  # */
/* # use, modify and/ or redistribute the software under the terms of     # */
/* # the CeCILL-C license as circulated by CEA, CNRS and INRIA at the     # */
/* # following URL http://www.cecill.info.                                # */
/* #                                                                      # */
/* # The fact that you are presently reading this means that you have     # */
/* # had knowledge of the CeCILL-C license and that you accept its        # */
/* # terms.                                                               # */
/* #                                                                      # */
/* # Authors:                                                             # */
/* #   - BESNARD Jean-Baptiste jean-baptiste.besnard@cea.fr               # */
/* #                                                                      # */
/* ######################################################################## */

/**
 * @addtogroup Common_Debug_
 * @{
 */


#ifndef DEBUG_H
#define DEBUG_H

#include <stdio.h>

/**
 * @brief Warning macro
 * 
 * Prints : 
 * @code
 *  WARNING : filename:line [function name] : message
 * @endcode
 */
#define WARNING(string, ...) printf( "WARNING : %s:%d [%s] : " string , __FILE__, __LINE__, __FUNCTION__, __VA_ARGS__);

/**
 * @brief Error macro
 * 
 * Prints : 
 * @code
 *  ERROR : filename:line [function name] : message
 * @endcode
 */
#define ERROR(string, ...) printf( "ERROR : %s:%d [%s] : " string , __FILE__, __LINE__, __FUNCTION__, __VA_ARGS__);

/**
 * @brief Malp information macro
 * 
 * Prints : 
 * @code
 *  <## MALP INFO ##> : message
 * @endcode
 */
#define MALP(string, ...) printf( "<## MALP INFO ##> :" string , __VA_ARGS__);

/**
 * @brief PERROR macro
 * 
 * Prints : 
 * @code
 *  DEBUG : filename:line [function name] : perror string
 * @endcode
 */
#define PERROR(string) printf( "DEBUG : %s:%d [%s] : " , __FILE__, __LINE__, __FUNCTION__); \
                       perror( string );

/**
 * @brief Debug macro
 * 
 * Prints : 
 * @code
 *  DEBUG : file name:line [function name] : message
 * @endcode
 */
#define DEBUG(string, ...) printf( "DEBUG : %s:%d [%s] : " string , __FILE__, __LINE__, __FUNCTION__, __VA_ARGS__);

#endif /* DEBUG_H */

/**
 * @}
 */
