/* ############################ MALP License ############################## */
/* # Fri Jan 18 14:00:00 CET 2013                                         # */
/* # Copyright or (C) or Copr. Commissariat a l'Energie Atomique          # */
/* #                                                                      # */
/* # This software is governed by the CeCILL-C license under French law   # */
/* # and abiding by the rules of distribution of free software.  You can  # */
/* # use, modify and/ or redistribute the software under the terms of     # */
/* # the CeCILL-C license as circulated by CEA, CNRS and INRIA at the     # */
/* # following URL http://www.cecill.info.                                # */
/* #                                                                      # */
/* # The fact that you are presently reading this means that you have     # */
/* # had knowledge of the CeCILL-C license and that you accept its        # */
/* # terms.                                                               # */
/* #                                                                      # */
/* # Authors:                                                             # */
/* #   - BESNARD Jean-Baptiste jean-baptiste.besnard@cea.fr               # */
/* #                                                                      # */
/* ######################################################################## */

/**
 * @addtogroup Timer_
 * @{
 */

#ifndef TIMER_H
#define TIMER_H

#ifdef __cplusplus
  extern "C"
  {
#endif



#include <stdint.h>
#include <stdio.h>
#include <sys/time.h>

/**
 * @brief Initializes the timer (save current processor timer counter)
 */
void Timer_set_origin();

/**
 * @brief Sets the offset to apply to timer
 * @param offset the new offset
 */
void Timer_set_offset(long long int offset );

/**
 * @brief Timer_Init_scale
 */
void Timer_Init_scale( unsigned int (*real_sleep)(unsigned int) );

/**
 * @brief The offset of the timer
 */
extern long long int Process_Sync_Offset;

/**
 * @brief The origin of the timer
 */
extern uint64_t Process_time_origin;

/**
 * @brief A second in the timer scale
 */
extern double Process_timer_second;

/**
 * @brief Getter on the current timer scale second
 * @return the current timer scale second
 */
static inline double Timer_second()
{
	return Process_timer_second;
}

#if (defined(__GNUC__) || defined(__ICC))

static inline void sctk_rdtsc_barrier(void)
{
	asm("mfence");
	asm("lfence");
}

#if defined(__ia64__)
static inline uint64_t Timer_tsc (void)
{
	//if( Process_Sync_Offset )
	//printf("\n\nOFF : %d \n",Process_Sync_Offset);
	unsigned long t;
	__asm__ volatile ("mov %0=ar%1":"=r" (t):"i" (44));
	return (uint64_t) t + Process_Sync_Offset - Process_time_origin;
}
#elif defined(__i686__)
static inline uint64_t Timer_tsc (void)
{


	unsigned long long t;
	sctk_rdtsc_barrier();
	__asm__ volatile ("rdtsc":"=A" (t));
	return (uint64_t) t + Process_Sync_Offset - Process_time_origin;
}



#elif defined(__x86_64__)
static inline uint64_t Timer_tsc (void)
{

	unsigned int a;
	unsigned int d;
	unsigned long t;
	sctk_rdtsc_barrier();
	__asm__ volatile ("rdtsc":"=a" (a), "=d" (d));
	t = ((unsigned long) a) | (((unsigned long) d) << 32);
	uint64_t ret = (uint64_t) t + Process_Sync_Offset - Process_time_origin;

	return ret;
}
#else
#warning "Use get time of day for profiling, interprocess sync will not work !"

#include<sys/time.h>

static inline uint64_t Timer_tsc (void)
{
	struct timeval tp;
	gettimeofday (&tp, NULL);
	return tp.tv_usec + tp.tv_sec * 1000000;
}
#endif

#else
#error Compile with GCC or ICC

#endif

#ifdef __cplusplus
  }
#endif




#endif
  
/**
 * @}
 */
