/* ############################ MALP License ############################## */
/* # Fri Jan 18 14:00:00 CET 2013                                         # */
/* # Copyright or (C) or Copr. Commissariat a l'Energie Atomique          # */
/* #                                                                      # */
/* # This software is governed by the CeCILL-C license under French law   # */
/* # and abiding by the rules of distribution of free software.  You can  # */
/* # use, modify and/ or redistribute the software under the terms of     # */
/* # the CeCILL-C license as circulated by CEA, CNRS and INRIA at the     # */
/* # following URL http://www.cecill.info.                                # */
/* #                                                                      # */
/* # The fact that you are presently reading this means that you have     # */
/* # had knowledge of the CeCILL-C license and that you accept its        # */
/* # terms.                                                               # */
/* #                                                                      # */
/* # Authors:                                                             # */
/* #   - BESNARD Jean-Baptiste jean-baptiste.besnard@cea.fr               # */
/* #                                                                      # */
/* ######################################################################## */

/**
 * @brief @addtogroup Data_Model_
 *
 *@{
 */

#ifndef STREAM_DESCRIPTOR_H
#define STREAM_DESCRIPTOR_H

#ifdef __cplusplus
extern "C"
{
#endif

/**
 * @brief enum defining the type of the source of the stream
 * @todo Only MALP_PROCESS is used within MALP. 
 */
typedef enum
{
	MALP_NO_TYPE,   /**< @brief No type */
	MALP_NODE,      /**< @brief Source is a node */
	MALP_PROCESS,   /**< @brief Source is a process */
	MALP_VCPU,      /**< @brief Source is a virtual CPU */
	MALP_TASK,      /**< @brief Source is a task */
	MALP_THREAD,    /**< @brief Source is a thread */
	MALP_TYPES_COUNT
}Stream_type;

static const char * const Stream_type_names[MALP_TYPES_COUNT] =
{
	"unknown",
	"node",
	"process",
	"VCPU",
	"task",
	"thread"
};

/**
 * @brief enum defining a stream event type
 */
typedef enum
{
	DESC_BEGIN, /**< @brief event is stream creation */
	DESC_END    /**< @brief event is stream release */
}Stream_Descriptor_type;


/**
 * @brief Structure containing data related to a stream and intended to be sent as an event (see @link MALP_Trace_Event @endlink)
 */
struct Stream_Descriptor
{
	/* Global informations */
	uint64_t tid;                       /**< @brief id of the stream (see Stream_MGR_gen_id())*/
	uint64_t parent_tid;                /**< @brief id of parent stream */
	uint64_t partition_id;              /**< @brief id of the partition (mpi program that the stream belongs to) */
	Stream_Descriptor_type desc_type;   /**< @brief identifies whether this is a stream creation or release event */
	Stream_type type;                   /**< @brief the type of the source of the steram */
	uint64_t pid;
	uint64_t thread_id;

	/* Temporal informations */
	uint64_t start_time;                /**< @brief time at stream creation */
	uint64_t end_time;                  /**< @brief time at stream release */

	/* Contextual informations */
	uint64_t mpi_rank;                  /**< @brief mpi rank of the source of the stream */
	uint64_t omp_thread;                /**< @brief OpenMP thread id of the source of the stream */
	char hostname[32];                  /**< @brief name of the host running the source of the stream */
};




#ifdef __cplusplus
}
#endif

#endif /* STREAM_DESCRIPTOR_H */

/**
 * @}
 */
