/* ############################ MALP License ############################## */
/* # Fri Jan 18 14:00:00 CET 2013                                         # */
/* # Copyright or (C) or Copr. Commissariat a l'Energie Atomique          # */
/* #                                                                      # */
/* # This software is governed by the CeCILL-C license under French law   # */
/* # and abiding by the rules of distribution of free software.  You can  # */
/* # use, modify and/ or redistribute the software under the terms of     # */
/* # the CeCILL-C license as circulated by CEA, CNRS and INRIA at the     # */
/* # following URL http://www.cecill.info.                                # */
/* #                                                                      # */
/* # The fact that you are presently reading this means that you have     # */
/* # had knowledge of the CeCILL-C license and that you accept its        # */
/* # terms.                                                               # */
/* #                                                                      # */
/* # Authors:                                                             # */
/* #   - BESNARD Jean-Baptiste jean-baptiste.besnard@cea.fr               # */
/* #                                                                      # */
/* ######################################################################## */


/**
 * @brief @addtogroup Data_Model_
 *
 *@{
 */

#ifndef WRAPPED_SYMBOLS_H
#define WRAPPED_SYMBOLS_H

#include <stdint.h>

/* Wrapped symbols */
/**
 * @defgroup Wrapped_Symbols Wrapped symbols
 * @brief This section relates to "manually" wrapped symbols. 
 * 
 * @warning If you add something make sure to update all structures
 * in this page. you can use Wrap_dump_infos()
 * to debug Wrapped symbols meta-data
 * 
 * @{
 */

 
/* Rewrite macros */
#define T_POSIX_ENUM(a) T_##a
#define T_POSIX_NAME(a) #a
#define T_POSIX_ID(a) a


/***** Event_List ******/
/**
 * @brief enum indicating which symbols are wrapped
 */
typedef enum {
<MALPW>
MALPW_WNAME( T_POSIX_ENUM );
    ,
</MALPW>
	/* Variadic */
	T_POSIX_ENUM(fscanf),
	T_POSIX_ENUM(printf),
	T_POSIX_ENUM(fprintf),
	T_POSIX_ENUM(open),
	T_POSIX_ENUM(open64),
	/* Count */
    T_POSIX_COUNT
}Wrapped_symbol;


/***** Wrapped symbols names *****/

/**
 * @brief Array containing wrapped symbols names
 */
static const char * const Wrapped_symbol_names[T_POSIX_COUNT] =
{
<MALPW>
MALPW_WNAME( T_POSIX_NAME );
    ,
</MALPW>
	/* Variadic */
	T_POSIX_NAME(fscanf),
	T_POSIX_NAME(printf),
	T_POSIX_NAME(fprintf),
	T_POSIX_NAME(open64),
	T_POSIX_NAME(open)
};


/********** Footprints **********/
/**
 * @defgroup Footprints_ Wrapped Symbols footprints
 * @brief These are wrapped symbols footprints
 *
 * @todo [DOC] remove cond if we want to get all the footprints in the documentation
 * @{
 * @cond
 */
<MALPW>
	MALPW_FOOTPRINT(FOOTPRINT_);
</MALPW>

/* Variadic */
#define FOOTPRINT_fscanf int (*)(FILE *stream, const char *format, ...)
#define FOOTPRINT_printf int (*)(const char *format, ...)
#define FOOTPRINT_fprintf int (*)(FILE *stream, const char *format, ...)
#define FOOTPRINT_open int (*)(const char* path, int flags, ...)
#define FOOTPRINT_open64 int (*)(const char* path, int flags, ...)


/**
 * @endcond
 * @}
 */

/**** Event Footprint *****/
/**
 * @brief Structure defining event for wrapped symbols
 */
struct Wrapped_Event
{
	Wrapped_symbol type;            /**< @brief The symbol type */
	int64_t size;                  /**< @brief For functions that manipulates data, the size of the data (for example, with malloc, the size of allocated data) */
	void * ptr;                     /**< @brief For functions that manipulates a pointer, the pointer (for example, with malloc, the allocated data) */
};
/**
 * @}
 */

#endif /* WRAPPED_SYMBOLS_H */

/**
 * @}
 */
