/* ############################ MALP License ############################## */
/* # Fri Jan 18 14:00:00 CET 2013                                         # */
/* # Copyright or (C) or Copr. Commissariat a l'Energie Atomique          # */
/* #                                                                      # */
/* # This software is governed by the CeCILL-C license under French law   # */
/* # and abiding by the rules of distribution of free software.  You can  # */
/* # use, modify and/ or redistribute the software under the terms of     # */
/* # the CeCILL-C license as circulated by CEA, CNRS and INRIA at the     # */
/* # following URL http://www.cecill.info.                                # */
/* #                                                                      # */
/* # The fact that you are presently reading this means that you have     # */
/* # had knowledge of the CeCILL-C license and that you accept its        # */
/* # terms.                                                               # */
/* #                                                                      # */
/* # Authors:                                                             # */
/* #   - BESNARD Jean-Baptiste jean-baptiste.besnard@cea.fr               # */
/* #                                                                      # */
/* ######################################################################## */
#ifndef EVENT_PACK_H
#define EVENT_PACK_H

/**
 * @addtogroup Instrum_
 *
 * @{
 */

#include <stdint.h>
#include "Event_Desc.h"
#include "VMPI.h"
#include "VMPI_stream.h"
#include "Spinlock.h"


/**
 * @brief This is the struct used to send events throuh a stream
 */
struct Event_Packer
{
	struct Event_Pack *pack;    /**< @brief The event pack */
	VMPI_Stream stream;         /**< @brief The associated stream */
	MALP_Spinlock lock;         /**< @brief the lock for concurrent access */
};

/**
 * @brief The event packer of the current MPI rank
 */
extern struct Event_Packer _malp_e_packer;

/**
 * @brief Getter on the current event packer
 * @return the current event packer
 */
static inline struct Event_Packer * Event_Packer()
{
	return &_malp_e_packer;
}

/**
 * @brief Initializes the event packer, maps current process to existing Analyser or IO_WRITE partition and initializes the streams. 
 */
void Event_Packer_init();

/**
 * @brief Releases the event packer
 */
void Event_Packer_release();

/**
 * @brief Sends an event. 
 *
 * the event is stored into an internal FIFO until it gets full. 
 * @param event the event to send
 */
void Event_Packer_push(struct MALP_Trace_Event *event);

/**
 * @brief Processes the send of all events present in the internal FIFO
 */
void Event_Packer_flush();

/**
 * @}
 */

#endif /* EVENT_PACK_H */
