/* ############################ MALP License ############################## */
/* # Fri Jan 18 14:00:00 CET 2013                                         # */
/* # Copyright or (C) or Copr. Commissariat a l'Energie Atomique          # */
/* #                                                                      # */
/* # This software is governed by the CeCILL-C license under French law   # */
/* # and abiding by the rules of distribution of free software.  You can  # */
/* # use, modify and/ or redistribute the software under the terms of     # */
/* # the CeCILL-C license as circulated by CEA, CNRS and INRIA at the     # */
/* # following URL http://www.cecill.info.                                # */
/* #                                                                      # */
/* # The fact that you are presently reading this means that you have     # */
/* # had knowledge of the CeCILL-C license and that you accept its        # */
/* # terms.                                                               # */
/* #                                                                      # */
/* # Authors:                                                             # */
/* #   - BESNARD Jean-Baptiste jean-baptiste.besnard@cea.fr               # */
/* #                                                                      # */
/* ######################################################################## */
#include <VMPI.h>
#include "VMPI_Wrap.h"
#include "MPI_Events.h"
#include "MPI_Inst.h"
#include "Event_Desc.h"
#include "Timer.h"
#include "Event_Pack.h"
#include "Stream_MGR.h"
#include "OTF_Event_forge.h"
#include "Posix_Profile.h"
#include "MALP_Config.h"

#include <mpi.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#define PRINT_NAME( a ) printf( #a"\n");
#define SET_SOURCE( a ) _container.event.mpi_event.source = a;
#define SET_DEST( a ) _container.event.mpi_event.dest = a;
#define SET_TAG( a ) _container.event.mpi_event.tag = a;
#define SET_COUNT( a ) _container.event.mpi_event.count = a;
#define SET_COMM( a ) _container.event.mpi_event.comm = (uint64_t)a;
#define SET_DATATYPE( a ) _container.event.mpi_event.datatype = (uint64_t)a;
#define COMP_SIZE( a ) PMPI_Type_size( datatype ,&_size); \
					   _size *= a;

<MALPW !MPI_Init,!MPI_Init_thread,!MPI_Initialized,!MPI_Finalize>
MALPW_PROTOTYPE();
{
	
	/* Virtualize MPI */
	MALPW_WRAP( MPI_Comm , * , MACRO_MPI_Comm );

	/* Pre instrum */
	int _rank = Stream_desc_get()->mpi_rank;

	struct MALP_Trace_Event _container;

	/********************************/
	/* FILL MPI EVENT               */
	/********************************/

	_container.event.mpi_event.type =
	MALPW_WNAMECASE_UP( T_MPI_ENUM );
	;

	/* Set default source & dest before override */
	_container.event.mpi_event.source = _rank;
	_container.event.mpi_event.dest = _rank;

	MALPW_WRAP( int , source , SET_SOURCE );
	MALPW_WRAP( int , dest , SET_DEST );
	MALPW_WRAP( int , tag , SET_TAG );
	MALPW_WRAP( int , count , SET_COUNT );
	MALPW_WRAP( MPI_Datatype , datatype , SET_DATATYPE );
	MALPW_WRAP( MPI_Comm , comm , SET_COMM );
	
	/* Fill in size informations if needed */
	int _size = 0;
	MALPW_HAS_ARG_B( MPI_Datatype, datatype );
	MALPW_WRAP( int , count , COMP_SIZE );
	MALPW_HAS_ARG_E( MPI_Datatype, datatype );
	_container.event.mpi_event.size = _size;

	/********************************/
	/* EMIT OTF ENTER              */
	/********************************/
	
	struct MALP_Trace_Event otf_event;

	uint64_t _begin =  Timer_tsc ();

	if( TA_Conf_get_key_bool( TA_CONF_INSTRUMENT_MPI)
	&&  TA_Conf_get_key_bool( TA_CONF_INSTRUMENT_MPI_OTF) )
	{	

			MALP_Trace_Event_fill( &otf_event, MALP_EVENT_OTF, _begin, 0);
			OTF_Event_forge( &otf_event, OTF_EVENT_ENTER,
							 MALP_EVENT_MPI, _container.event.mpi_event.type );
			Event_Packer_push(&otf_event);
	}

	/********************************/
	/* ACTUAL FUNCTION CALL         */
	/********************************/

	Stream_Infos_enter_mpi();
	MALPW_CALL(P);
	Stream_Infos_leave_mpi();

	/********************************/
	/* EMIT OTF END                 */
	/********************************/
	
	uint64_t _end =  Timer_tsc();
	uint64_t _duration = _end - _begin;
	
	if( TA_Conf_get_key_bool( TA_CONF_INSTRUMENT_MPI) )
	{	
		if( TA_Conf_get_key_bool( TA_CONF_INSTRUMENT_MPI_OTF) )
		{	
			
			MALP_Trace_Event_fill( &otf_event, MALP_EVENT_OTF, _end, _duration);
			OTF_Event_forge( &otf_event, OTF_EVENT_LEAVE,
							 MALP_EVENT_MPI, _container.event.mpi_event.type );
			

			Event_Packer_push(&otf_event);
		}
	
	/********************************/
	/* EMIT MALP EVENT              */
	/********************************/

		MALP_Trace_Event_fill( &_container, MALP_EVENT_MPI, _begin, _duration );
	
		/* Submit event if MPI instrumented */
		Event_Packer_push(&_container);
	}
	
	
	return ret;
}
</MALPW>
