/* ############################ MALP License ############################## */
/* # Fri Jan 18 14:00:00 CET 2013                                         # */
/* # Copyright or (C) or Copr. Commissariat a l'Energie Atomique          # */
/* #                                                                      # */
/* # This software is governed by the CeCILL-C license under French law   # */
/* # and abiding by the rules of distribution of free software.  You can  # */
/* # use, modify and/ or redistribute the software under the terms of     # */
/* # the CeCILL-C license as circulated by CEA, CNRS and INRIA at the     # */
/* # following URL http://www.cecill.info.                                # */
/* #                                                                      # */
/* # The fact that you are presently reading this means that you have     # */
/* # had knowledge of the CeCILL-C license and that you accept its        # */
/* # terms.                                                               # */
/* #                                                                      # */
/* # Authors:                                                             # */
/* #   - BESNARD Jean-Baptiste jean-baptiste.besnard@cea.fr               # */
/* #                                                                      # */
/* ######################################################################## */
#ifndef POSIX_PROFILE_H
#define POSIX_PROFILE_H

#define _GNU_SOURCE
#include <dlfcn.h>

#include "Wrapped_Symbols.h"

#include <stdint.h>
#include <string.h>

#include "Spinlock.h"

struct Profile_Entry
{
	Wrapped_symbol id;
	
	uint64_t hits;
	uint64_t time;
	int64_t size;
	
	MALP_Spinlock lock;
};

extern struct Profile_Entry __POSIX_Profile_entries[T_POSIX_COUNT];

void Profile_Entry_init( struct Profile_Entry *ent, Wrapped_symbol id );
void Profile_Entry_release( struct Profile_Entry *ent );


void POSIX_profile_init();
void POSIX_profile_release();
void POSIX_profile_render();
void POSIX_profile_reduce();
void POSIX_profile_serialize();


static inline struct Profile_Entry * POSIX_profile_get_entry( Wrapped_symbol id )
{
	if( (0 <= id) && (id < T_POSIX_COUNT) )
	{
		return &__POSIX_Profile_entries[id];
	}
	
	return NULL;
}


static inline POSIX_profile_hit( Wrapped_symbol id, uint64_t duration, int64_t size )
{
	struct Profile_Entry * ent = POSIX_profile_get_entry( id );
	
	if( !ent )
	{
		return;
	}
	
	MALP_Spinlock_lock(&ent->lock);
	
	ent->hits++;
	ent->time += duration;
	ent->size += size;
	
	MALP_Spinlock_unlock(&ent->lock);
}


#endif /* POSIX_PROFILE_H */
