/* ############################ MALP License ############################## */
/* # Fri Jan 18 14:00:00 CET 2013                                         # */
/* # Copyright or (C) or Copr. Commissariat a l'Energie Atomique          # */
/* #                                                                      # */
/* # This software is governed by the CeCILL-C license under French law   # */
/* # and abiding by the rules of distribution of free software.  You can  # */
/* # use, modify and/ or redistribute the software under the terms of     # */
/* # the CeCILL-C license as circulated by CEA, CNRS and INRIA at the     # */
/* # following URL http://www.cecill.info.                                # */
/* #                                                                      # */
/* # The fact that you are presently reading this means that you have     # */
/* # had knowledge of the CeCILL-C license and that you accept its        # */
/* # terms.                                                               # */
/* #                                                                      # */
/* # Authors:                                                             # */
/* #   - BESNARD Jean-Baptiste jean-baptiste.besnard@cea.fr               # */
/* #                                                                      # */
/* ######################################################################## */
#include "Posix_Wrapp.h"

MALPW_GEN_HEADERS();

#include "Timer.h"
#include "Event_Desc.h"
#include "Event_Pack.h"
#include "VMPI.h"
#include "Stream_MGR.h"
#include "Posix_Profile.h"
#include "MALP_Config.h"

#include <stdarg.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>


/* Interception table */
union function_table_elem Tracelib_symbol_table[T_POSIX_COUNT];

static int posix_wrap_gate = 0;

void Wrapp_enable()
{
	posix_wrap_gate = 1;
}


void Wrapp_disable()
{
	posix_wrap_gate = 0;
}

void ___TEST_LOAD_SYMBOL( Wrapped_symbol symb, const char * const symbol )
{
	if( Tracelib_symbol_table[ symb ].p == NULL )
	{
		Tracelib_symbol_table[ symb ].p =  dlsym( RTLD_NEXT , symbol ); 
		if( Tracelib_symbol_table[ symb ].p == NULL )
		{
			return;
		}
	}
}



void Posix_Wrapp_init()
{
	TEST_LOAD_SYMBOL(calloc);
	POSIX_profile_init();
	
	int i;
	
	for( i = 0 ; i < T_POSIX_COUNT ; i++ )
	{
		___TEST_LOAD_SYMBOL( i, Wrapped_symbol_names[i] );
	}
	
}



static inline void Wrapp_Format( struct MALP_Trace_Event * event, Wrapped_symbol s, uint64_t start  , uint64_t  duration, int64_t size, void * ptr )
{
	MALP_Trace_Event_fill( event, MALP_EVENT_WRAPPED, start, duration );

	event->event.wrapped_event.type = s;
	event->event.wrapped_event.size = size;
	event->event.wrapped_event.ptr = ptr;

}


static inline void Wrap_Submit( struct MALP_Trace_Event * event )
{
	if( (event->type == MALP_EVENT_NONE) || !Stream_desc_get() )
		return;

	Event_Packer_push(event);
}

static inline void Wrapp_OTF_enter( uint64_t timestamp, Wrapped_symbol symbol )
{
	/********************************/
	/* EMIT OTF ENTER              */
	/********************************/
	
	struct MALP_Trace_Event otf_event;

	uint64_t _begin =  Timer_tsc ();

	MALP_Trace_Event_fill( &otf_event, MALP_EVENT_OTF, timestamp, 0);
	OTF_Event_forge( &otf_event, OTF_EVENT_ENTER,
					 MALP_EVENT_WRAPPED, symbol );
	Wrap_Submit(&otf_event);
}

static inline void Wrapp_OTF_leave( uint64_t end, uint64_t start, Wrapped_symbol symbol )
{
	/********************************/
	/* EMIT OTF LEAVE               */
	/********************************/
	
	struct MALP_Trace_Event otf_event;

	uint64_t _begin =  Timer_tsc ();

	MALP_Trace_Event_fill( &otf_event, MALP_EVENT_OTF, end, end - start);
	OTF_Event_forge( &otf_event, OTF_EVENT_LEAVE,
					 MALP_EVENT_WRAPPED, symbol );
	Wrap_Submit(&otf_event);	
}

static volatile int is_first_calloc_call = 0;
char temp_buff[1024];

void * calloc(size_t a, size_t b)
{

	if( is_first_calloc_call )
		return (void *)temp_buff;

	if( !pREAL(calloc)  )
	{
		is_first_calloc_call = 1;
		TEST_LOAD_SYMBOL(calloc);
		is_first_calloc_call = 0;
	}

	int64_t ___size = a*b;
	uint64_t ___start = Timer_tsc();

	if( posix_wrap_gate )
	{
		if( TA_Conf_get_key_bool( TA_CONF_INSTRUMENT_POSIX)
		&&  TA_Conf_get_key_bool( TA_CONF_INSTRUMENT_POSIX_OTF) )
		{
			Wrapp_OTF_enter( ___start,
			T_POSIX_ENUM(calloc)
			);
		}
	}

	void * ret = REAL(calloc)(a, b);

	uint64_t ___end = Timer_tsc();

	uint64_t ___duration = ___end - ___start;
	
	if( posix_wrap_gate )
	{
		if( TA_Conf_get_key_bool( TA_CONF_INSTRUMENT_POSIX)
		&&  TA_Conf_get_key_bool( TA_CONF_INSTRUMENT_POSIX_OTF) )
		{
			Wrapp_OTF_leave( ___end, ___start,
			T_POSIX_ENUM(calloc)
			);
		}


		if( TA_Conf_get_key_bool( TA_CONF_INSTRUMENT_POSIX_PROFILE) )
		{
			POSIX_profile_hit(
			T_POSIX_ENUM(calloc)
			, ___duration, ___size );
		}

		
		if( TA_Conf_get_key_bool( TA_CONF_INSTRUMENT_POSIX) )
		{
			struct MALP_Trace_Event event;

			Wrapp_Format( &event,
			T_POSIX_ENUM(calloc)
			, ___start, ___duration, ___size, ret);

			Wrap_Submit(&event);
		}
	}
	
	return ret;
}

/*******
VARIADIC
********/

int fscanf(FILE *stream, const char *format, ...)
{       
	TEST_LOAD_SYMBOL( ftell )

	va_list arg;
	va_start(arg, format);

	long __before_off = REAL(ftell)(stream);

	uint64_t ___start = Timer_tsc();

	if( posix_wrap_gate )
	{
		if( TA_Conf_get_key_bool( TA_CONF_INSTRUMENT_POSIX)
		&&  TA_Conf_get_key_bool( TA_CONF_INSTRUMENT_POSIX_OTF) )
		{
			Wrapp_OTF_enter( ___start,
			T_POSIX_ENUM(fscanf)
			);
		}
	}


	int ret = vfscanf( stream, format, arg );


	uint64_t ___end = Timer_tsc();

	long __after_off = REAL(ftell)(stream);

	int64_t _malp_size = __after_off - __before_off;
	uint64_t ___duration = ___end - ___start;

	if( posix_wrap_gate )
	{
		if( TA_Conf_get_key_bool( TA_CONF_INSTRUMENT_POSIX)
		&&  TA_Conf_get_key_bool( TA_CONF_INSTRUMENT_POSIX_OTF) )
		{
			Wrapp_OTF_leave( ___end, ___start,
			T_POSIX_ENUM(fscanf)
			);
		}


		if( TA_Conf_get_key_bool( TA_CONF_INSTRUMENT_POSIX_PROFILE) )
		{
			POSIX_profile_hit(
			T_POSIX_ENUM(fscanf)
			, ___duration, _malp_size );
		}

		
		if( TA_Conf_get_key_bool( TA_CONF_INSTRUMENT_POSIX) )
		{
			struct MALP_Trace_Event event;

			Wrapp_Format( &event,
			T_POSIX_ENUM(fscanf)
			, ___start, ___duration, _malp_size, NULL);

			Wrap_Submit(&event);
		}
	}
	
	va_end(arg);

    return ret;
}


int printf(const char *format, ...)
{
	va_list arg, arg2;
	va_start(arg, format);
	va_copy(arg2, arg);

	uint64_t ___start = Timer_tsc();

	if( posix_wrap_gate )
	{
		if( TA_Conf_get_key_bool( TA_CONF_INSTRUMENT_POSIX)
		&&  TA_Conf_get_key_bool( TA_CONF_INSTRUMENT_POSIX_OTF) )
		{
			Wrapp_OTF_enter( ___start,
			T_POSIX_ENUM(printf)
			);
		}
	}

	int ret = vprintf( format, arg );
	
	uint64_t ___end = Timer_tsc();

	int64_t _malp_size = vsnprintf( NULL, 0, format, arg2 );
	uint64_t ___duration = ___end - ___start;

	if( posix_wrap_gate )
	{
		if( TA_Conf_get_key_bool( TA_CONF_INSTRUMENT_POSIX)
		&&  TA_Conf_get_key_bool( TA_CONF_INSTRUMENT_POSIX_OTF) )
		{
			Wrapp_OTF_leave( ___end, ___start,
			T_POSIX_ENUM(printf)
			);
		}


		if( TA_Conf_get_key_bool( TA_CONF_INSTRUMENT_POSIX_PROFILE) )
		{
			POSIX_profile_hit(
			T_POSIX_ENUM(printf)
			, ___duration, _malp_size );
		}

		
		if( TA_Conf_get_key_bool( TA_CONF_INSTRUMENT_POSIX) )
		{
			struct MALP_Trace_Event event;

			Wrapp_Format( &event,
			T_POSIX_ENUM(printf)
			, ___start, ___duration, _malp_size, NULL);

			Wrap_Submit(&event);
		}
	}

	va_end(arg);
	va_end(arg2);
	
	return ret;
}



int fprintf(FILE *stream, const char *format, ...)
{
	va_list arg, arg2;
	va_start(arg, format);
	va_copy(arg2, arg);

	uint64_t ___start = Timer_tsc();

	if( posix_wrap_gate )
	{
		if( TA_Conf_get_key_bool( TA_CONF_INSTRUMENT_POSIX)
		&&  TA_Conf_get_key_bool( TA_CONF_INSTRUMENT_POSIX_OTF) )
		{
			Wrapp_OTF_enter( ___start,
			T_POSIX_ENUM(fprintf)
			);
		}
	}

	int ret = vfprintf( stream, format, arg );

	uint64_t ___end = Timer_tsc();

	int64_t _malp_size = vsnprintf( NULL, 0, format, arg2 );
	uint64_t ___duration = ___end - ___start;

	if( posix_wrap_gate )
	{
		if( TA_Conf_get_key_bool( TA_CONF_INSTRUMENT_POSIX)
		&&  TA_Conf_get_key_bool( TA_CONF_INSTRUMENT_POSIX_OTF) )
		{
			Wrapp_OTF_leave( ___end, ___start,
			T_POSIX_ENUM(fprintf)
			);
		}


		if( TA_Conf_get_key_bool( TA_CONF_INSTRUMENT_POSIX_PROFILE) )
		{
			POSIX_profile_hit(
			T_POSIX_ENUM(fprintf)
			, ___duration, _malp_size );
		}

		
		if( TA_Conf_get_key_bool( TA_CONF_INSTRUMENT_POSIX) )
		{
			struct MALP_Trace_Event event;

			Wrapp_Format( &event,
			T_POSIX_ENUM(fprintf)
			, ___start, ___duration, _malp_size, NULL);

			Wrap_Submit(&event);
		}
	}

	va_end(arg);
	va_end(arg2);
	
	return ret;
}


int (*real_open)(const char*, int, ...) = NULL;

int open(const char* path, int flags, ...)
{
	mode_t mode = 0;

	if ( flags & O_CREAT )
	{
		va_list arg;
		va_start(arg, flags);
		mode = va_arg(arg, mode_t);
		va_end(arg);
	}
	else
	{
		if( flags & O_RDONLY )
		{
			mode |= S_IRUSR;
		}
		
		if( flags & O_WRONLY )
		{
			mode |= S_IWUSR;
		}
		
		if( flags & O_RDWR )
		{
			mode |= (S_IRUSR | S_IWUSR);
		}
	}
	
	/* Here load manually */
	if( ! real_open )
	{
		real_open = dlsym(RTLD_NEXT, "open");
		
		if( !real_open )
		{
			perror("dlsym");
			abort();
		}
	}
	
	uint64_t ___start = Timer_tsc();

	if( posix_wrap_gate )
	{
		if( TA_Conf_get_key_bool( TA_CONF_INSTRUMENT_POSIX)
		&&  TA_Conf_get_key_bool( TA_CONF_INSTRUMENT_POSIX_OTF) )
		{
			Wrapp_OTF_enter( ___start,
			T_POSIX_ENUM(open)
			);
		}
	}

	int ret = (real_open)(path, flags, mode);

	uint64_t ___end = Timer_tsc();

	int64_t _malp_size = 0;
	uint64_t ___duration = ___end - ___start;
	void * ___ptr = ret;

	if( posix_wrap_gate )
	{
		if( TA_Conf_get_key_bool( TA_CONF_INSTRUMENT_POSIX)
		&&  TA_Conf_get_key_bool( TA_CONF_INSTRUMENT_POSIX_OTF) )
		{
			Wrapp_OTF_leave( ___end, ___start,
			T_POSIX_ENUM(open)
			);
		}


		if( TA_Conf_get_key_bool( TA_CONF_INSTRUMENT_POSIX_PROFILE) )
		{
			POSIX_profile_hit(
			T_POSIX_ENUM(open)
			, ___duration, _malp_size );
		}

		
		if( TA_Conf_get_key_bool( TA_CONF_INSTRUMENT_POSIX) )
		{
			struct MALP_Trace_Event event;

			Wrapp_Format( &event,
			T_POSIX_ENUM(open)
			, ___start, ___duration, _malp_size, ___ptr);

			Wrap_Submit(&event);
		}
	}
	
	return ret;
}


int (*real_open64)(const char*, int, ...) = NULL;

int open64(const char* path, int flags, ...)
{
	mode_t mode = 0;

	if ( flags & O_CREAT )
	{
		va_list arg;
		va_start(arg, flags);
		mode = va_arg(arg, mode_t);
		va_end(arg);
	}
	else
	{
		if( flags & O_RDONLY )
		{
			mode |= S_IRUSR;
		}
		
		if( flags & O_WRONLY )
		{
			mode |= S_IWUSR;
		}
		
		if( flags & O_RDWR )
		{
			mode |= (S_IRUSR | S_IWUSR);
		}
	}
	
	/* Here load manually */
	if( ! real_open64 )
	{
		real_open64 = dlsym(RTLD_NEXT, "open64");
		
		if( !real_open64 )
		{
			perror("dlsym");
			abort();
		}
	}
	
	uint64_t ___start = Timer_tsc();

	if( posix_wrap_gate )
	{
		if( TA_Conf_get_key_bool( TA_CONF_INSTRUMENT_POSIX)
		&&  TA_Conf_get_key_bool( TA_CONF_INSTRUMENT_POSIX_OTF) )
		{
			Wrapp_OTF_enter( ___start,
			T_POSIX_ENUM(open64)
			);
		}
	}

	int ret = (real_open64)(path, flags, mode);

	uint64_t ___end = Timer_tsc();

	int64_t _malp_size = 0;
	uint64_t ___duration = ___end - ___start;
	void * ___ptr = ret;

	if( posix_wrap_gate )
	{
		if( TA_Conf_get_key_bool( TA_CONF_INSTRUMENT_POSIX)
		&&  TA_Conf_get_key_bool( TA_CONF_INSTRUMENT_POSIX_OTF) )
		{
			Wrapp_OTF_leave( ___end, ___start,
			T_POSIX_ENUM(open64)
			);
		}


		if( TA_Conf_get_key_bool( TA_CONF_INSTRUMENT_POSIX_PROFILE) )
		{
			POSIX_profile_hit(
			T_POSIX_ENUM(open64)
			, ___duration, _malp_size );
		}

		
		if( TA_Conf_get_key_bool( TA_CONF_INSTRUMENT_POSIX) )
		{
			struct MALP_Trace_Event event;

			Wrapp_Format( &event,
			T_POSIX_ENUM(open64)
			, ___start, ___duration, _malp_size, ___ptr);

			Wrap_Submit(&event);
		}
	}
	
	return ret;
}


/* pthread_cond_timedwait & pthread_cond_init seem to have a bug ... 
And we don't need to instruments functions which does not exit */

<MALPW !open,!open64,!fprintf,!printf,!fscanf,!_IO_getc,!_IO_putc,!calloc,!mmap,!pthread_cond_timedwait,!pthread_cond_wait,!pthread_cond_destroy,!pthread_cond_signal,!pthread_cond_init,!abort,!exit,!quick_exit,!pthread_exit,!setjmp,!siglongjmp,!sched_yield,!getpagesize>
MALPW_PROTOTYPE();
{
	TEST_LOAD_SYMBOL(
	MALPW_NAME();
	);
	
	uint64_t ___start = Timer_tsc();

	if( posix_wrap_gate )
	{
		if( TA_Conf_get_key_bool( TA_CONF_INSTRUMENT_POSIX)
		&&  TA_Conf_get_key_bool( TA_CONF_INSTRUMENT_POSIX_OTF) )
		{
			Wrapp_OTF_enter( ___start,
			MALPW_WNAME( T_POSIX_ENUM );
			);
		}
	}

	MALPW_CALL(REAL);

	uint64_t ___end = Timer_tsc();

	uint64_t ___duration = ___end - ___start;

	MALPW_CALL_SIZE( _malp_size );
	MALPW_CALL_PTR( _malp_ptr );

	if( posix_wrap_gate )
	{
		if( TA_Conf_get_key_bool( TA_CONF_INSTRUMENT_POSIX)
		&&  TA_Conf_get_key_bool( TA_CONF_INSTRUMENT_POSIX_OTF) )
		{
			Wrapp_OTF_leave( ___end, ___start,
			MALPW_WNAME( T_POSIX_ENUM );
			);
		}


		if( TA_Conf_get_key_bool( TA_CONF_INSTRUMENT_POSIX_PROFILE) )
		{
			POSIX_profile_hit(
			MALPW_WNAME( T_POSIX_ENUM );
			, ___duration, _malp_size );
		}

		
		if( TA_Conf_get_key_bool( TA_CONF_INSTRUMENT_POSIX) )
		{
			struct MALP_Trace_Event event;

			Wrapp_Format( &event,
			MALPW_WNAME( T_POSIX_ENUM );
			, ___start, ___duration, _malp_size, _malp_ptr);

			Wrap_Submit(&event);
		}
	}


	MALPW_GEN_RET();
}
</MALPW>
