/* ############################ MALP License ############################## */
/* # Fri Jan 18 14:00:00 CET 2013                                         # */
/* # Copyright or (C) or Copr. Commissariat a l'Energie Atomique          # */
/* #                                                                      # */
/* # This software is governed by the CeCILL-C license under French law   # */
/* # and abiding by the rules of distribution of free software.  You can  # */
/* # use, modify and/ or redistribute the software under the terms of     # */
/* # the CeCILL-C license as circulated by CEA, CNRS and INRIA at the     # */
/* # following URL http://www.cecill.info.                                # */
/* #                                                                      # */
/* # The fact that you are presently reading this means that you have     # */
/* # had knowledge of the CeCILL-C license and that you accept its        # */
/* # terms.                                                               # */
/* #                                                                      # */
/* # Authors:                                                             # */
/* #   - BESNARD Jean-Baptiste jean-baptiste.besnard@cea.fr               # */
/* #                                                                      # */
/* ######################################################################## */
var crypt = require('crypto');
var pathParser = require('path');
var fs = require('fs');
var cache = require('nsc');

/* This function is used to generate the trace description form
 * with support for several partitions to do so we save datas in the cache */

function LoadTrace( res, path, arg_mode )
{
	if( !arg_mode )
		res.write("<h1>MALP Trace upload</h1>"); 
	
	res.write("<h1>Please fill in partition informations...</h1>"); 
	
	var data = "";
	var malp_trace = {};
	
	try
	{
		/* Read profile */
		try
		{
			data = fs.readFileSync( path, { encoding : "ascii" }  );
		}
		catch(e)
		{
			console.log(e);
			res.end("Failed to open trace file: " + path );
		}
		
		
		/* Load it in the cache */
		cache.load_json_string( data );
		
		var parts = cache.cache.partitions;
		
		if( parts == undefined )
		{
			res.end("Failed to load TRACE, no partitions found<br>");
			return;
		}
		
		res.write("<form action='/uploadtrace/finalupload' enctype='multipart/form-data' method='post'>");
		res.write("<table class='malptable'>\n");
		res.write("<tr>\n");
		res.write("	<th>Application</th>\n");
		res.write("	<th>Test Case</th>\n");
		res.write("	<th>Size</th>\n");
		res.write("	<th>Command</th>\n");
		res.write("</tr>\n");

		var id = 0;

		for( var i = 0 ; i < parts.length ; i++ )
		{
			if( !parts[i] )
				continue;
			
			if( parts[i].program_name == "malp_an" )
				continue;
				
			var current_profile = {};
			current_profile.profile = cache.cache[ "" + i ];
			current_profile.desc = parts[i];
			current_profile.app = "";
			current_profile.testcase = "";
			current_profile.size = parts[i].partition_size;
			
			/* Recopy Matching streams */
			var streams = {};
			var c_streams = cache.cache["streams"];
			

			for( s in c_streams )
			{
				if( s == undefined )
					continue;
				
				if( c_streams[s].partition_id == i )
					streams[s] = c_streams[s];
				
			}
			
			
			current_profile.streams = streams;
			
			cache.cache["tmp_prof_" + id] = current_profile;
	
			res.write("<tr>\n");
			res.write("	<td>\n");

			res.write("<input type='text' value='"+ parts[i].command_line.split(' ')[0] +"' name='name_" + id + "'>");

			res.write("	</td>\n");
			res.write("	<td>\n");

			res.write("<input type='text' value='default' name='case_" + id + "'>");

			res.write("	</td>\n");
			res.write("	<td>\n");

			res.write( "" + parts[i].partition_size );

			res.write("	</td>\n");
			res.write("	<td>\n");

			res.write( "" + parts[i].command_line );

			res.write("	</td>\n");
			res.write("</tr>\n");

						
			id++;
		}
	
		res.write("</table>\n");
		
		if( !arg_mode )
			res.write( "<input type='submit' value='Upload Trace'>");
		else
			res.write( "<input type='submit' value='Submit Trace'>");
		
		res.write("</form>\n");
		
	}
	catch(e)
	{
		console.log(e);
	}
	
	
}



function repackArray( array )
{
	var tmp = [];
	
	for( var i = 0 ; i < array.length ; i++ )
		if( (array[i] != null) && (array[i] != undefined) )
			tmp.push( array[i] );
	
	return tmp;
}


function Application_run( size, casename, id )
{
	this.size = size;
	this.casename = casename;
	this.id = id;
	
	var sha = crypt.createHash('sha1');
	sha.update(new Buffer(casename + id));
	this.container = sha.digest('hex');	
}


function Application_Case( name )
{
	this.name = name;
	this.runs = [];

	if( name.constructor == Object )
		for (var e in name)
			this[e] = name[e];

	Application_Case.prototype.getRunOffset = function( id )
	{
		for( var i = 0 ; i < this.runs.length ; i++ )
			if( this.runs[i].id == id )
				return i;
		
		return undefined;
	}

	
	
}

/*
	APPLICATION ENTRY
*/

function Application( appname, desc, repository )
{
	this.appname = appname;
	this.desc = desc;
	this.repository = repository;

	this.id = 0;

	this.root_path = "";

	this.cases = [];
	
	if( appname.constructor == Object )
	{
		for (var e in appname)
		{
			if( e == "cases" )
			{
				var tmp = appname[e];
				
				for( var i = 0 ; i < tmp.length ; i++ )
				{
					var tmpc = new Application_Case( tmp[i] );
					this.cases.push( tmpc );
				}
				
			}
			else
			{
				this[e] = appname[e];
			}
		}
	}

	Application.prototype.getcaseOffset = function( casename )
	{
		for( var i = 0 ; i < this.cases.length ; i++ )
			if( this.cases[i].name == casename )
				return i;
		
		return undefined;
	}

	Application.prototype.getcase = function( casename )
	{
		var off = this.getcaseOffset( casename );
		
		if( off != undefined )
			return this.cases[off];
		
		return undefined;
	}
	
	Application.prototype.pushRun = function( size, casename, content )
	{
		/* If needed instantiate a case */
		var targetcase = this.getcase( casename );
		
		if( targetcase == undefined )
		{
			targetcase = new Application_Case( casename );
			this.cases.push( targetcase );
		}
		
		
		/* Then create a new entry for this run */
		var run = new Application_run( size, casename, this.id++ )
		targetcase.runs.push( run );
		
		/* And now store the performance content in a file */
		try
		{
			fs.writeFileSync(this.root_path + "/" + run.container , JSON.stringify(content));
		}
		catch(e)
		{
			console.dir(e);
		}
		
		return run;
	}



	Application.prototype.deleteCase = function( casename )
	{
		var off = this.getcaseOffset( casename );
		var targetcase = this.cases[off];
		
		if( targetcase == undefined )
		{
			console.log( "Could not delete " + casename + " as it was not found");
			return;
		}
		
		console.log( "Deleting Case " + casename + "..." );
		
		for( var i = 0 ; i < targetcase.runs.length ; i++ )
			this.deleteRun( targetcase.runs[i] );
		
		delete this.cases[off];
		
		/* Repack */
		
		this.cases = repackArray( this.cases );
		
		if( this.cases.length == 0 )
		{
			global_catalog.deleteApp( this.appname );
		}
		
	}
	
	Application.prototype.deleteRun = function( run )
	{
		if( run == undefined )
			return;
			
		var casename = run.casename;
		
		var targetcase = this.getcase( casename );
		
		if( targetcase == undefined )
		{
			console.log( "Could not delete run as it is not associated with a case");
			return;
		}
	
		
		var offset = targetcase.getRunOffset( run.id );
		
		console.log( "Deleting run in case " + run.casename + " over " + run.size + "...");
		
		/* Delete its content */
		try
		{
			fs.unlinkSync( this.root_path + "/" + run.container );
		}
		catch( e )
		{
			console.dir( e );
		}
		
		/* Remove it from container */
		delete targetcase.runs[ offset ];
		
		/* Repack */
	
		targetcase.runs = repackArray( targetcase.runs );
		
		console.dir( targetcase.runs );
		
		if( targetcase.runs.length == 0 )
		{
			this.deleteCase( casename );
		}

	}
	
	Application.prototype.clear = function()
	{
		for( var i = 0 ; i < this.cases.length ; i++ )
			this.deletecase( this.cases[i].casename );
	}
}


/*
	CATALOG ENTRY
*/

function Catalog_Entry( appname )
{
	this.appname = appname;
	
	var sha = crypt.createHash('sha1');
	sha.update(new Buffer(appname));
	
	this.repo = sha.digest('hex');	

}

/*
	CATALOG
*/

function safe_create_dir( path )
{
	try
	{
		stat = fs.statSync(path);

		if( !stat.isDirectory() )
		{
			console.log("There is a file " + path + " but it is a regular file.");
			throw("Could not create " + path +" directory");
		}

	}
	catch( e )
	{
		console.log("No " + path + "... creating directory.");
		fs.mkdirSync(path);
	}
}


function Catalog(catalog_path)
{

	if( catalog_path != undefined )
		this.catalog_path = catalog_path;
	else
		this.catalog_path = process.env['HOME'] + "/.malp/catalog/";

	this.catalog = [];
	this.MALP_trace_path = "";

	/* Try loading the catalog */

	var stat = undefined;
	var data = "";

	try
	{
		stat = fs.statSync(this.catalog_path + "/catalog.json");

		/* If we are here the file exists now load the previous catalog */
		try
		{
			data = fs.readFileSync( this.catalog_path + "/catalog.json", { encoding : "ascii" }  );

			var tmp_catalog = JSON.parse( data );

			if( tmp_catalog.constructor != Array )
			{
				throw("Error catalog is not an array... try deleting it : " + this.catalog_path + "/catalog.json");
			}

			this.catalog = tmp_catalog;
		}
		catch(e)
		{
			console.log(e);
		}

	}
	catch(e)
	{
		console.log("No catalog found...");
	}


	/* If no catalog found create one */
	safe_create_dir( process.env['HOME'] + "/.malp" );
	safe_create_dir( process.env['HOME'] + "/.malp/catalog" );

	this.applications = {};
	
	this.current_app = undefined;

	Catalog.prototype.getApp = function(  appname )
	{
		var app = this.applications[ appname ];
		
		return app;
	}
	


	Catalog.prototype.loadFromArgs = function( path_to_trace )
	{
		if( path_to_trace != undefined )
			this.MALP_trace_path = path_to_trace.trim();
		
		return this.MALP_trace_path;
	}

	
	Catalog.prototype.getRun = function(  appname, casename, run_id )
	{
		var app = this.getApp( appname );
		
		if( app == undefined )
		{
			console.log("No app");
			return undefined;
		}
		
		var ccase = app.getcase( casename );
		
		if( ccase == undefined )
		{
			console.log("No case");
			return undefined;
		}
		
		var run = ccase.runs[ run_id ];
		
		if( run == undefined )
		{
			console.log("No run");
			return undefined;
		}
		
		return run;
	}
	
	
	
	Catalog.prototype.deleteProfile = function( appname, casename, run_id )
	{
		var app = this.getApp( appname );
		
		if( app == undefined )
		{
			return;
		}
		
		var run = this.getRun(  appname, casename, run_id );
		
		if( run == undefined )
		{
			return;
		}
		
		app.deleteRun( run );
		
	}
	
	
	Catalog.prototype.loadProfile = function( appname, casename, run_id )
	{
		var app = this.getApp( appname );
		
		if( app == undefined )
		{
			return 0;
		}
		
		var run = this.getRun(  appname, casename, run_id );
		
		if( run == undefined )
		{
			return 0;
		}
		
		var data = "";
		
		/* Now load the target */
		try
		{
			data = fs.readFileSync( app.root_path + "/" + run.container , { encoding : "ascii" }  );

			var tmp_profile = JSON.parse( data );

			this.current_app = tmp_profile;
		}
		catch(e)
		{
			console.log(e);
			return 0;
		}
		
		return 1;
	}
	
	

	Catalog.prototype.loadApp = function( id )
	{
		if( this.catalog[ id ] == undefined )
		{
			console.log( "No such id " + id );
			return;
		}

		var entry = this.catalog[ id ];

		if( entry.appname == undefined )
			return;

		if( entry.repo == undefined )
			return;	

		/* Otherwise load application data */
		try
		{
			data = fs.readFileSync( this.catalog_path + "/" + entry.repo + "/appdesc.json", { encoding : "ascii" }  );

			this.applications[ entry.appname ] = new Application( JSON.parse( data ) );
			
		}
		catch(e)
		{
			console.log(e);
		}
	}	
	
	/* Load the catalog */
	for( var i = 0 ; i < this.catalog.length ; i++ )
		this.loadApp( i );
	
	Catalog.prototype.appList = function()
	{
		var ret = []
		
		for( var i = 0 ; i < this.catalog.length ; i++ )
		{
			if( this.catalog[i] == undefined )
				continue;

			ret.push( this.applications[ this.catalog[i].appname ] );
		}
		
		return ret;
	}

	Catalog.prototype.save = function()
	{
		/* Repack in order to remove null entries */
		
		this.catalog = repackArray( this.catalog );

		try
		{
			fs.writeFileSync(this.catalog_path + "/catalog.json", JSON.stringify(this.catalog));
		}
		catch(e)
		{
			console.dir(e);
		}

		/* Save individual applications */
		for( var i = 0 ; i < this.catalog.length ; i++ )
		{
			if( this.catalog[i] == undefined )
				continue;

			var current_app = this.applications[ this.catalog[i].appname ];

			if( current_app == undefined )
				continue;

		
			console.log( "Saving " + current_app.appname + "...");

			try
			{
				fs.writeFileSync(this.catalog_path + "/" +  this.catalog[i].repo + "/appdesc.json" , JSON.stringify(current_app));
			}
			catch(e)
			{
				console.dir(e);
			}

		}
	}



	Catalog.prototype.getCatalogID = function( appname )
	{
		for( var i = 0 ; i < this.catalog.length ; i++ )
		{
			if( this.catalog[i] == undefined )
				continue;

			if(this.catalog[i].appname == appname )
			{
				return i;
			}

		}

		return undefined;
	}

	Catalog.prototype.getCatalog = function( appname )
	{
		var id = this.getCatalogID( appname );

		if( id == undefined )
			return undefined;

		return this.catalog[id];
	}


	Catalog.prototype.clear = function()
	{
		for( var i = 0 ; i < this.catalog.length ; i++ )
		{
			if( this.catalog[i] == undefined )
				continue;

			this.deleteApp( this.catalog[i].appname );

		}	
	}

	Catalog.prototype.deleteApp = function( appname )
	{
		var id =  this.getCatalogID( appname );
		var entry = this.catalog[id];

		if( entry == undefined )
			return;

		/* delete all files from entry repo */
		var files = [];

		try
		{
			files = fs.readdirSync( this.catalog_path + "/" + entry.repo + "/" );
		}
		catch( e )
		{
			console.dir( e );
			return;
		}

		/* Delete contained files */

		for( var i = 0 ; i < files.length ; i++ )
		{ 
			console.log("Removing: " + this.catalog_path + "/" + entry.repo + "/" + files[i] + "..."); 			
			fs.unlinkSync( this.catalog_path + "/" + entry.repo + "/" + files[i] ); 
		}

		/* Delete repo */
		console.log("Deleting repository: " +  entry.repo + " for application " + appname + "...");
		fs.rmdirSync(this.catalog_path + "/" + entry.repo);

		
		/* Now delete from catalog */
		delete this.applications[ entry.appname ];
		delete this.catalog[id];

	}
	
	Catalog.prototype.getProfile = function()
	{
		if( !this.current_app )
			return undefined;
		
		return this.current_app.profile;
	}
	
	Catalog.prototype.getStreams = function()
	{
		if( !this.current_app )
			return undefined;
		
		return this.current_app.streams;
	}
	
	Catalog.prototype.getDesc = function()
	{
		if( !this.current_app )
			return undefined;
		
		return this.current_app.desc;
	}

	Catalog.prototype.pushApp = function ( app )
	{
		if( app.constructor != Application )
		{
			throw("Bad type of element");
		}

		if( app.appname == undefined )
		{
			console.log("NO Name");
			return undefined;
		}

		var existingapp = this.getCatalog(app.appname);

		/* Check if it already exists */
		if( existingapp != undefined )
		{
			console.dir( existingapp );
			if( existingapp && this.applications[ app.appname ] )
				return this.applications[ app.appname ];
		}

		var ct_entry = new Catalog_Entry( app.appname );

		/* Set app rootpath */
		app.root_path = this.catalog_path + "/" + ct_entry.repo + "/";

		safe_create_dir( this.catalog_path + "/" +  ct_entry.repo )

		this.catalog.push( ct_entry );
		this.applications[ app.appname ] = app;

		/* Save the first catalog app entry */
		try
		{
			fs.writeFileSync(this.catalog_path + "/" +  ct_entry.repo + "/appdesc.json" , JSON.stringify(app));
		}
		catch(e)
		{
			console.dir(e);
			return undefined;
		}
		
		return app;
	}
	

};


global_catalog = new Catalog();

module.exports.LoadTrace = LoadTrace;
module.exports.catalog = global_catalog;
module.exports.Application = Application;
