/* ############################ MALP License ############################## */
/* # Fri Jan 18 14:00:00 CET 2013                                         # */
/* # Copyright or (C) or Copr. Commissariat a l'Energie Atomique          # */
/* #                                                                      # */
/* # This software is governed by the CeCILL-C license under French law   # */
/* # and abiding by the rules of distribution of free software.  You can  # */
/* # use, modify and/ or redistribute the software under the terms of     # */
/* # the CeCILL-C license as circulated by CEA, CNRS and INRIA at the     # */
/* # following URL http://www.cecill.info.                                # */
/* #                                                                      # */
/* # The fact that you are presently reading this means that you have     # */
/* # had knowledge of the CeCILL-C license and that you accept its        # */
/* # terms.                                                               # */
/* #                                                                      # */
/* # Authors:                                                             # */
/* #   - BESNARD Jean-Baptiste jean-baptiste.besnard@cea.fr               # */
/* #                                                                      # */
/* ######################################################################## */
var child = require('child_process');
var fs = require('fs');
var events = require('events');
var util = require('util');

module.exports = Graph;

function Graph()
{
	events.EventEmitter.call(this);
	this.dot = "";
	this.graph = {};
}

util.inherits(Graph, events.EventEmitter);

Graph.prototype.JSON = function (file)
{
	return JSON.stringify(this.graph);
}

Graph.prototype.loadFile = function (file)
{
	try
	{
		data = fs.readFileSync(file, { encoding : "ascii" } );
		this.dot = data;
		this.scanDot();
	}
	catch(e)
	{
		console.log(e);
		return;
	}
	
}

Graph.prototype.setDot = function (dot)
{
	this.dot = dot;
	this.scanDot();
}

if (!String.prototype.trim) {
	 String.prototype.trim = function() {
	  return this.replace(/^\s+|\s+$/g,'');
	 }
}

// --------------------------------	
Graph.prototype.normalize = function ()
{
	var value = 1000;

	if( this.graph.vertex == undefined )
		return;


	// Lets compute the barycentre
	var x = 0;
	var y = 0;
	var z = 0;
	var cnt = 0;
	
	for( i = 0 ; i < this.graph.vertex.length ; i++ )
	{	
		x += this.graph.vertex[i][0];
		y += this.graph.vertex[i][1];
		z += this.graph.vertex[i][2];
		
		cnt ++;
	}
	
	x /= cnt;
	y /= cnt;
	z /= cnt;
	
		
	var max_v = 0;
	
	/* Remove barycentre and compute MAX */
	
	for( i = 0 ; i < this.graph.vertex.length ; i++ )
	{
		this.graph.vertex[i][0] -= x;
		this.graph.vertex[i][1] -= y;
		this.graph.vertex[i][2] -= z;

		if( !max_v || max_v < Math.abs( this.graph.vertex[i][0] ) )
		{
			max_v =  Math.abs( this.graph.vertex[i][0] );
		}

		if( !max_v || max_v < Math.abs( this.graph.vertex[i][1] ) )
		{
			max_v =  Math.abs( this.graph.vertex[i][1] );
		}

		if( !max_v || max_v < Math.abs( this.graph.vertex[i][2] ) )
		{
			max_v =  Math.abs( this.graph.vertex[i][2] );
		}
	}
	
	if( !max_v )
		return;
	
	var factor = value / max_v;

	// Normalize to 0 -- 1
	for( i = 0 ; i < this.graph.vertex.length ; i++ )
	{
		this.graph.vertex[i][0]*=factor;
		this.graph.vertex[i][1]*=factor;
		this.graph.vertex[i][2]*=factor;
	}

	
}

Graph.prototype.parseDot = function()
{
	var lines = this.dot.replace(/\t/g, "").replace(/;\n/g, "\n").replace(/,\n/g, ",").split("\n");
	
	var nodes = [];
	var node_count = 0;
	var edges = [];


	for( i = 0 ; i < lines.length ; i++ )
	{
		m = lines[i].match("([0-9]*) -[->] ([0-9]*) \\[(.*)\\]");
		
		if( m != null )
		{
			if( m.length == 4)
			{
				edge = Array(3);
				
				edge_match = m[0].match("([0-9]*) -[->] ([0-9]*)");
				
				if( edge_match != null )
				{
					
					edge[0] = parseInt( edge_match[1] );
					edge[1] = parseInt( edge_match[2] );
					
					if( isNaN( edge[0]  ) || isNaN( edge[1] ) )
					{
						console.log("WARNING : Graph node IDs must be numeric !\n");
						continue;
					}
						
				}

				val_match = m[0].match('val=\"([0-9.]*)\"');
				edge[2] = 0;
		
				if( val_match != null )
				{
						edge[2] = parseFloat(val_match[1]);
				}
				else
				{
					val_match = m[0].match('val=([0-9.]*)');
					
					if( val_match != null )
					{
							edge[2] = parseFloat(val_match[1]);
					}
					
				}
				
				edges.push( edge );
			}
		}
		else
		{

			m = lines[i].match("([0-9]*) \\[(.*)\\]");

			if( m != null )
			{
				if( m.length == 3)
				{

					node_id = undefined;
					id_match =  m[0].match('^[0-9]*');
					
					if( id_match != null )
					{
						node_id = parseInt( id_match[0] );
						if( isNaN( node_id ) )
							continue;
					}
					
					node_match = m[0].match('pos=\\"([0-9.-]*),([0-9.-]*),([0-9.-]*)\\"');
					
					if( node_match != null )
					{
						if( node_id != undefined )
						{
							nodes[ node_id ] = [ parseFloat(node_match[1]), parseFloat(node_match[2]), parseFloat(node_match[3]) ];
							node_count++;
						}
					}
				}
				
			}
		}
				
				
					
	}
	
	this.graph.vertex = nodes;
	this.graph.vertex_count = node_count;
	this.graph.edges = edges;

	this.normalize();
	
	this.emit('dotParsed');
}



Graph.prototype.scanDot = function ()
{
		var filename = "/tmp/" + Math.round( Math.random() * 1e16 )  + Math.round( Math.random() * 1e16 ) + ".tmp";
	
		fs.writeFileSync(filename, this.dot);
		
		var that = this;
		
		child.exec("sfdp -Gdimen=3 -Tdot " + filename , { maxBuffer : 1024*1024*100 },

				function (error, stdout, stderr) {

					if (error !== null) {
					  console.log('exec error: ' + error);
					  fs.unlink( filename );
					  return;
					}

					that.dot = stdout;
					
					fs.unlink( filename );
					that.parseDot();
				}
		);
}


//
