/* ############################ MALP License ############################## */
/* # Fri Jan 18 14:00:00 CET 2013                                         # */
/* # Copyright or (C) or Copr. Commissariat a l'Energie Atomique          # */
/* #                                                                      # */
/* # This software is governed by the CeCILL-C license under French law   # */
/* # and abiding by the rules of distribution of free software.  You can  # */
/* # use, modify and/ or redistribute the software under the terms of     # */
/* # the CeCILL-C license as circulated by CEA, CNRS and INRIA at the     # */
/* # following URL http://www.cecill.info.                                # */
/* #                                                                      # */
/* # The fact that you are presently reading this means that you have     # */
/* # had knowledge of the CeCILL-C license and that you accept its        # */
/* # terms.                                                               # */
/* #                                                                      # */
/* # Authors:                                                             # */
/* #   - BESNARD Jean-Baptiste jean-baptiste.besnard@cea.fr               # */
/* #                                                                      # */
/* ######################################################################## */
var events = require('events');
var util = require('util');
var fs = require('fs');
var pathParser = require('path');
var child = require('child_process');
//Command line colors (https://github.com/medikoo/cli-color)
var clc = require('cli-color');

var server = require('./server');
var Page = require('./Page');


function default_head(page, GET,  POST, FILES, res)
{
	res.write("<style type=\"text/css\">pagebody {color:red; font-size:22px; font-family:arial; text-decoration:underline}</style>");
}

function default_header(page, GET,  POST, FILES, res)
{
	res.write("<h1>" + page.pageHandler.sitename + "</h1>");
}


function default_navlocal(page, GET,  POST, FILES, res)
{
	for( i = 0 ; i < page.childrenPages.length ; i++ )
	{
		res.write("<a href=\"" + page.childrenPages[i].getUri() +"\" tooltip=\""+ page.childrenPages[i].alt +"\"  ><div class=\"tabs\">" + page.childrenPages[i].title + "</div></a>" );
	}
}


function default_navglobal(page, GET,  POST, FILES, res)
{
	
	uris = [];

	cpa = page;

	while( cpa != undefined )
	{
		uris.push( { "uri" : "" + cpa.getUri() , "title" : "" + cpa.title} );
		cpa = cpa.parentPage;
	}
	
	if( uris.length == 1 )
		return;

	for( i = (uris.length - 1) ; 0 <= i ; i-- )
	{
		
		if( i )
		{
			res.write("<a href=\"" + uris[i].uri + "\"><div class=\"headlink\">" + uris[i].title + "</div></a>" );
		}
		else
		{
			res.write("<a href=\"" + uris[i].uri + "\"><div class=\"headcurrent\">" + uris[i].title + "</div></a>" );
		}
	}

}


function default_body(page, GET,  POST, FILES, res)
{
	res.write("<p>" + page.alt + "</p>");
}

function default_footer(page, GET,  POST, FILES, res)
{
	res.write( page.pageHandler.sitename );
}


function renderPage( page, filename, GET,  POST, FILES, res )
{
	pageview = page.views[ filename ];

	if( pageview )
	{
		if( pageview.is_raw || GET["body_only"]  )
		{
			(pageview.callback)( page,  GET,  POST, FILES, res );
			
			if( !pageview.is_raw )
				res.end();
			
			return;
		}
	}
	
	if( GET["body_only"] )
	{
		page.body( page, GET,  POST, FILES, res );
		res.end();
		return;
	}
	
	res.write("<!DOCTYPE html>\n");
	
	res.write("<head>\n");
	res.write("<meta http-equiv=\"Content-Type\" content=\"text/html; charset=utf-8\">\n");
	page.head( page, GET,  POST, FILES, res );
	res.write("</head>\n");
	
	res.write("<body>\n");
	res.write("<div class=\"website\">\n");	
	
	res.write("<div class=\"pageheader\">\n");	
	page.header( page, GET,  POST, FILES, res );
	res.write("</div>\n");

	res.write("<div class=\"navglobal\">\n");	
	page.navglobal( page, GET,  POST, FILES, res );
	res.write("</div>\n");
	
	res.write("<div class=\"navlocal\">\n");	
	page.navlocal( page, GET,  POST, FILES, res );
	res.write("</div>");
	
	res.write("<div class=\"pagebody\">\n");
	
	if( pageview )
		(pageview.callback)( page,  GET,  POST, FILES, res );
	else
		page.body( page, GET,  POST, FILES, res );

	res.write("</div>\n");
	
	res.write("<div class=\"pagefooter\">\n");	
	page.footer( page, GET,  POST, FILES, res );
	res.write("</div>\n");
	
	res.write("</div>\n");
	
	res.write("</body>\n");
	
	res.write("</html>\n");
	
	res.end();
}

function pageHandler()
{
	events.EventEmitter.call(this);
	this.server = server;
	this.sitename = "";
	this.pages = {};

	this.head = default_head;
	this.header = default_header;
	this.footer = default_footer;
	this.navglobal = default_navglobal;
	this.navlocal = default_navlocal;
	this.body = default_body;
}

util.inherits(pageHandler, events.EventEmitter);

pageHandler.prototype.pushPage = function( page )
{
	if( this.pages[ page.uri ] != undefined )
	{
		throw Error("Uri: " + page.uri + " already exists");
	}

	//Rewire the pageHandler (needed when constructed in template file)
	if( page.pageHandler == undefined )
	{
		page.setPageHandler( this );
		page.setParent( this );
		page.setRender( this );
	}

	server.registerWebModule( page.uri, renderPage , page);

	this.pages[ page.uri ] = page;

	return page;
}


pageHandler.prototype.newPage = function( uri, parentUri, title, alt, ctx )
{
	page = new Page( uri, parentUri, title, alt, ctx, this );

	return this.pushPage( page );
}

pageHandler.prototype.siteName = function( name )
{
	this.sitename = name;
}


pageHandler.prototype.listen = function( port )
{
	this.server.listen( port );
}

pageHandler.prototype.staticResource = function( uri, path )
{
	this.server.addResourceDir( uri, path );
}

pageHandler.prototype.setAuth = function( username, password )
{
	this.server.setAuth( username, password );
}

function extract_field( text, field_name )
{

	var m = text.match( field_name + '=\"([0-9A-Za-z/ ]+)\"[ >]' );
	if( m != null )
	{
		return m[1];
	}

	return "";
}

function Gen_template( data )
{
	var tmp_template = "/tmp/"+ Math.round( Math.random() * 1e16 )  + Math.round( Math.random() * 1e16 ) + ".js.tmp";
	var lines = data.split('\n');
	var output_script = "";
	
	var in_print_area = 0;
	var res_write_begun = 0;
	var page_loaded = 0;

	var in_server = 0;
	
	output_script += "var __Page = require('userver').Page;\n"
	
	for( var i = 0 ; i < lines.length ; i++ )
	{
		current_line = lines[i];
		
		/* Match page begin */
		var match = current_line.match("<page.*/>");
		
		if( match != null )
		{
			if( in_print_area )
			{
				throw Error("Cannot start a page within another page or view @" + match[0] + " line " + (i + 1) );
			}
			
			if( page_loaded )
			{
				throw Error("A template shall define a SINGLE page but might have many views  @" + match[0] + " line " + (i + 1) );
			}
			

			page_loaded = 1;
			
			uri = extract_field( match[0], "uri" );
			parent = extract_field( match[0], "parent" );
			title = extract_field( match[0], "title" );
			desc = extract_field( match[0], "desc" );
			
			if( uri == "" )
			{
				throw Error("Cannot load a page without URI @" + match[0] + " line " + (i + 1) );
			}
			
			if( title == "" )
			{
				throw Error("Cannot load a page without TITLE @" + match[0] + " line " + (i + 1) );
			}
			
			/* Write page begin */
			output_script += 'var __this_page = new __Page( "' + uri + '", "' + parent + '", "' + title +  '", "' + desc + '", {} );\n'
			output_script += 'module.exports = __this_page;\n';
			
			continue;
		}
		
		
		/* Sections begin */
		match = current_line.match("<(body|header|footer|head|navglobal|navlocal)>");
		
		if( match != null )
		{
			var section = match[1];

			if( in_print_area )
			{
				throw Error("Cannot start a body within another page or view @" + match[0] + " line " + (i + 1) );
			}
			
			if( !page_loaded )
			{
				throw Error("You must set a page before loading a view  @" + match[0] + " line " + (i + 1) );
			}
			
			in_print_area = 1;
			
			output_script += '__this_page.' + section + ' = function(page, GET,  POST, FILES, res)\n'
			output_script += '{\n';
			continue;
		}
		
		

		
		/* Match view begin */
		var match = current_line.match("<view.*>");
		
		if( match != null )
		{
			if( in_print_area )
			{
				throw Error("Cannot start a view within another page or view @" + match[0] + " line " + (i + 1) );
			}
			
			if( !page_loaded )
			{
				throw Error("You must set a page before loading a view  @" + match[0] + " line " + (i + 1) );
			}
			
			in_print_area = 1;
			
			uri = extract_field( match[0], "uri" );
			is_raw = extract_field( match[0], "raw" );
			
			if( is_raw == "true" )
				is_raw = 1;
			else
				is_raw = 0;
						
			if( uri == "" )
			{
				throw Error("Cannot load a view without URI @" + match[0] + " line " + (i + 1) );
			}
			
			/* Write view begin */
			output_script += '__this_page.addView( "' + uri +'",' + uri + '_handler, '+ is_raw +' );\n'
			output_script += 'function ' + uri + '_handler(page, GET,  POST, FILES, res)\n'
			output_script += '{\n';
			
			continue;
		}
		
				
		/* Section END */
		match = current_line.match("</(body|header|footer|head|navglobal|navlocal|view)>");
		
		if( match != null )
		{
			if( !in_print_area )
			{
				throw Error("Cannot end a page or view before its start @" + match[0] + " line " + (i + 1) );
			}
			
			if( in_print_area )
			{
				in_print_area = 0;
				
				if( res_write_begun )
				{
					output_script += "'');\n\n";
					res_write_begun = 0;
				}
			}
			
			output_script += '}\n';
			
			continue;
		}
		
		
		/* Server side script handling */
		
		
		match = current_line.match("<server>");
		
		if( match != null )
		{
			if( in_server )
			{
				throw Error("Cannot start a server section inside another one @" + match[0] + " line " + (i + 1) );
			}
			
			
			if( in_print_area )
			{
				in_print_area = 0;
				
				if( res_write_begun )
				{
					output_script += "'');\n\n";
					res_write_begun = 0;
				}
			}
			else
			{
				throw Error("Cannot start a server section outside of a page or view  @" + match[0] + " line " + (i + 1) );
			}
			
			in_server = 1;
			
			continue;
		}
		
		
		match = current_line.match("</server>");
		
		if( match != null )
		{
			if( !in_server )
			{
				throw Error("Cannot end a server section before starting it @" + match[0] + " line " + (i + 1) );
			}
			
			in_server = 0;
			in_print_area = 1;
			continue;
		}
	
		
		/* Handle printing or replay */

		if( in_print_area )
		{
			if( current_line != "" )
			{
				if( ! res_write_begun )
				{
					output_script += "res.write("
					res_write_begun = 1;
				}



				output_script += "'";
				output_script += current_line.replace(/\'/g, "\\'" ) + "\\n";
				output_script += "'+ \n";

			}
			
		}
		else
		{
			output_script += current_line + '\n';
		}
		
	}
	
	fs.writeFileSync(tmp_template, output_script );

	return tmp_template;
}



pageHandler.prototype.parseTemplate = function( template )
{

	var data = "";
	
	ext = pathParser.extname(template);

	if( ext == ".malp" )
	{
		data = fs.readEnc( template );
	}
	else
	{
		try
		{
			data = fs.readFileSync( template  );
		}
		catch(e)
		{
			console.log(e);
		}
	}

	return Gen_template( data + "" );
 }


pageHandler.prototype.loadTemplates = function( directory, recursive )
{
	if( recursive == undefined )
		recursive = 0;

	//console.log(clc.green("Processing directory: " + directory));

	var stat = fs.statSync(directory);

	if( !stat.isDirectory() )
		throw Error("loadTemplates must process a directory ( had " + directory );

	var files = fs.readdirSync( directory );

	var i = 0;

	for( i = 0 ; i < files.length ; i++ )
	{
		stat = fs.statSync(directory + "/" + files[i]);

		if( !stat.isDirectory() )
		{
			ext = pathParser.extname(files[i]);

			if( ext == ".js" || ext == ".malp" )
			{
				//console.log(clc.green("Reading template: " + directory + "/" + files[i]));
				var parsed_template = this.parseTemplate(directory + "/" + files[i]);

				//console.log(clc.yellow("Loading template: " + files[i] ));

				newpage = require(parsed_template);
				
				//Make sure we did not get the empty object
				if( JSON.stringify(newpage) != "{}")
				{
					//Check if this is a Page object
					if(newpage.constructor != Page)
					{
						throw Error("Template files shall return a page object");
					}
				
					//Register page
					this.pushPage( newpage );
				}
				else
				{
					console.log("\t" + files[i] + " defines no template...");
				}
				
				
				fs.unlink( parsed_template );
			}

		}
	}

	//Then if recursive load subdirectories
	if( recursive )
	{
		var i = 0;
		
		for( i = 0 ; i < files.length ; i++ )
		{
			var stat = fs.statSync(directory + "/" + files[i]);
			if( stat.isDirectory() )
			{
					this.loadTemplates( directory + "/" + files[i], recursive );
			}
		}
	}

}



module.exports = pageHandler
