/* ############################ MALP License ############################## */
/* # Fri Jan 18 14:00:00 CET 2013                                         # */
/* # Copyright or (C) or Copr. Commissariat a l'Energie Atomique          # */
/* #                                                                      # */
/* # This software is governed by the CeCILL-C license under French law   # */
/* # and abiding by the rules of distribution of free software.  You can  # */
/* # use, modify and/ or redistribute the software under the terms of     # */
/* # the CeCILL-C license as circulated by CEA, CNRS and INRIA at the     # */
/* # following URL http://www.cecill.info.                                # */
/* #                                                                      # */
/* # The fact that you are presently reading this means that you have     # */
/* # had knowledge of the CeCILL-C license and that you accept its        # */
/* # terms.                                                               # */
/* #                                                                      # */
/* # Authors:                                                             # */
/* #   - BESNARD Jean-Baptiste jean-baptiste.besnard@cea.fr               # */
/* #                                                                      # */
/* ######################################################################## */
/*
 * DEPS
 */

var http = require('http');
var url = require('url');
var fs = require('fs');
var pathParser = require('path');

//Command line colors (https://github.com/medikoo/cli-color)
var clc = require('cli-color');
//MIME handling (https://github.com/broofa/node-mime)
var mime = require('mime');
//Multipart forms handling (https://github.com/felixge/node-formidable)
var formidable = require('formidable');

/*
 * WEB MODULES HANDLING
 */

function HTTPResource(uri, callback, ctx)
{
	this.uri = uri
	this.callback = callback;
	
	//console.log("\t"+ clc.blue("New module " + uri) );
	
	if(typeof ctx != undefined) {
        	this.ctx = ctx;
    	}
	else
	{
		this.ctx = {};
	}
}


/*
 * HTTPResourceHandler Object
 */


// SERVER is a singleton only came up with this 
// quite verbose declaration...
var HTTPResourceHandler = {};

HTTPResourceHandler.web_modules = {};
HTTPResourceHandler.s = undefined;
HTTPResourceHandler.root = "";
HTTPResourceHandler.maximumFileSize = 5*1024*1024*1024;



HTTPResourceHandler.listen = function( port, hostname )
{
	if( this.s == undefined )
		this.s = http.createServer( HTTP_Request_Handler );
	
	
 	if( hostname == undefined )
		this.s.listen( port );
	else
		this.s.listen( port, hostname );

	var addr = this.s.address();

	console.log( "----------------------------------------------");
	console.log( clc.red("Server Is now listening on port " + addr.port +  "...") );
	console.log( "----------------------------------------------");
}



HTTPResourceHandler.chroot = function( serverRoot )
{
	fs.stat(serverRoot, function( err, stats )
			     {
				if(err)
					throw err;
				
				if( !stats.isDirectory() )
					throw new Error("serverRoot must be a directory (" + serverRoot + ")");
			     }
	);

	this.root = serverRoot;

}

function ResourceDirCallback( basedir, filename, query, fields, files, res )
{
	full_path = pathParser.join(basedir, filename);
	HTTP_Send_file( full_path, res )
}


HTTPResourceHandler.addResourceDir = function( uri, path )
{
	fs.stat(path, function( err, stats )
		     {
			if(err)
				throw err;
			
			if( !stats.isDirectory() )
				throw new Error("A resource directory must be a directory (" + serverRoot + ")");
		     }

	);

	//console.log( clc.green("Registering static path: " + uri) );

	HTTPResourceHandler.registerWebModule( uri, ResourceDirCallback, path );
	
}

function countOccurences( haystack , needle )
{
	count = 0;
	pos = haystack.indexOf(needle);

	while ( pos != -1 ) {
	   count++;
	   pos = haystack.indexOf( needle,pos + 1 );
	}

	return count;
}

HTTPResourceHandler.registerWebModule = function( uri, callback, ctx )
{
	if( uri == "/" &&  HTTPResourceHandler.root != "")
	{
		throw new Error("Cannot register a module to / without shadowing root directory: server is already chrooted to "+ HTTPResourceHandler.root );
	}


	if( countOccurences( uri , "/" ) != 1 )
	{
		throw Error("Module URIs must be in the /XXX format got: " + uri);
	}

	if( this.web_modules[uri] != undefined )
	{
		throw new Error("Error Module conflict at URI:" + uri + "\n");
	}

	if(Object.prototype.toString.call(callback) == '[object Function]')
	{
		this.web_modules[uri] =  new HTTPResource( uri, callback, ctx );	
	}
	else
	{
		throw new Error("Callback shall be a function\n");
	}
}

HTTPResourceHandler.callWebModules = function( url, fields, files, res )
{
	uri = url.pathname;
	filename = "";

	if( uri[ uri.length - 1 ] == "/" && uri != "/" )
	{
		//In this case we address a directory
		uri = uri.slice( 0, uri.length - 1 );
	}
	else
	{
		//Here we have a full path
		filename =  pathParser.basename(uri);
		uri = pathParser.dirname( uri );
	}

	// TRY uri
	elem = this.web_modules[ uri ];

	if( elem != undefined )
	{
		//console.log(clc.blue("GET") + " " + clc.yellow( clc.red(url.pathname) +" Redirected to module: " + clc.red(elem.uri) ) );
		elem.callback( elem.ctx, filename, url.query, fields, files, res );
		return true;
	}

	return false;
}

module.exports = HTTPResourceHandler;


/*
 * HTTP SERVER
 */

function do404( path, res )
{
	res.writeHead(404, {'Content-Type': 'text/html'});
	res.end( "<b>ERROR 404</b><br>" + pathParser.basename(path)  + " Not found on this server\n");
}



function HTTP_Send_file( path, res )
{
	//console.log(clc.yellow("Serving static file :" + path) );

	fs.readFile(path, 
		function( err, data )
		{
			if( err )
			{
				console.log("No such file :" + pathParser.basename(path));
				do404( "File", res )
				return;
			}

			//Handle mimetypes
			ext = pathParser.extname(path);
			mimetype = mime.lookup( ext );

			if( ext != "" && ext != "html" )
			{
				res.writeHead(200, {'Content-Type': mimetype});
			}

			//console.log(clc.blue("STATIC") + ": " + clc.red( pathParser.basename(path) + " (" + mimetype + ")"));
			res.end( data );
		}
	);

}


function HTTP_Request_Handler_static_file( url, res )
{
	full_path = pathParser.join( HTTPResourceHandler.root , url.pathname );
	HTTP_Send_file( full_path, res )
}


function HTTP_Map_to_resources( req, res, fields, files )
{
	if( HTTPResourceHandler.callWebModules( parsed_url, fields, files, res ) )
	{
		//console.log("GET " + parsed_url.path +" Redirected to a module");
		return;
	}

	/* Here we handle everything that has not been captured before if there is a server root */
	if( HTTPResourceHandler.root != "" )
	{
		//console.log("GET " + parsed_url.path +" Redirected to a static file");
 		HTTP_Request_Handler_static_file( parsed_url, res );
		return;
	}

	do404(parsed_url.pathname, res );
}



function HTTP_Process_Request( req, res )
{
	parsed_url = url.parse( req.url, true );

	//We default to HTML
	res.writeHead(200, {'Content-Type': 'text/html'});

	//Handle forms if there are some (hopefully done through formidable parser: https://github.com/felixge/node-formidable)
	if (req.method.toLowerCase() == 'post') {

		var form = new formidable.IncomingForm();

		form.on('progress', function(bytesReceived, bytesExpected) {
			if( (HTTPResourceHandler.maximumFileSize < bytesReceived) || ( HTTPResourceHandler.maximumFileSize < bytesExpected ) )
			{
				res.writeHead(413, {'Content-Type': 'text/plain'});
                		res.end("Server does not handle files larger than " + HTTPResourceHandler.maximumFileSize + " Bytes" );
				return;
			}
		});

		form.parse(req, function(err, fields, files) {
			if( err )
			{
				do404("", res );
			}

			HTTP_Map_to_resources( req, res, fields, files );
			return;	

    		});
	}
	else
	{
		HTTP_Map_to_resources( req, res );
	}
}


function HTTP_Request_Handler( req, res )
{
		 HTTP_Process_Request( req, res );
}


