#include <stdio.h>
#include <mpi.h>
#include "nsc.h"

long double compute_pi( long long int nb_points, int rank, int size )
{
	nb_points = nb_points - (nb_points % size);
	double q = 1.0 / nb_points;
	
	long long int i, start, end, per_rank;
	
	per_rank = nb_points / size;
	start = rank * per_rank;
	end = (rank + 1 ) * per_rank;

	long double sum = 0.0;
	double x = 0.0;
	
	for( i = start ; i < end; i++ )
	{
		x = i * q;
		sum +=  q / ( 1.0 + x * x );
	}
	
	sum *= 4.0;

	long double global_sum = 0.0;
	
	MPI_Allreduce(&sum, &global_sum, 1, MPI_LONG_DOUBLE, MPI_SUM, MPI_COMM_WORLD);

	return global_sum;
}


int main( int argc, char ** argv )
{
	MPI_Init( &argc, &argv );
	
	struct nodeSharedCache nsc;
	nodeSharedCache_init( &nsc );
	
	int rank, size;
	
	MPI_Comm_rank( MPI_COMM_WORLD, &rank );
	MPI_Comm_size( MPI_COMM_WORLD, &size );
	
	long int nb_point = 0;
	long int prev_nb_point = -1;
	
	nodeSharedCache_set( &nsc, "points", json_real( 9 ) );
	nodeSharedCache_set( &nsc, "value", json_real( 0.0) );
	
	if(!rank )
	{
		nodeSharedCache_commit( &nsc, "value");
	}
	
	while( 0 <= nb_point )
	{
		json_t * points = nodeSharedCache_get( &nsc, "points");
		
	
		if( points )
		{

			if( points->type == JSON_REAL )
				nb_point = (long int)json_to_real( points )->value;
			else
				nb_point = (long int)json_to_int( points )->value;
			
			//printf("%ld\n", nb_point );
			
			if( nb_point )
			{

				if( prev_nb_point != nb_point )
				{
					prev_nb_point = nb_point;
				
					long double mypi = compute_pi( nb_point, rank, size );


					if( ! rank )
					{
						json_t * res = json_real( mypi );
						
						nodeSharedCache_set( &nsc, "value", res );
						
						json_decref( res );
						
						printf("%.16Le\n", mypi );
					}
				}
			}
		}
		
		usleep(100);
	}


	nodeSharedCache_release( &nsc );
	MPI_Finalize();
}
