var gvz_3d_animation_loop = [];


function graphviz3d( graph, container_id, width, height, mode )
{
	this.graph = graph;
	
	/* Attach to container */
	this.container = document.getElementById( container_id );
	
	if( this.container == undefined )
	{
		alert("Could not locate container");
		return;
	}

	/* Determine Width */
	
	/* Relative to parent ? */
	var parent = $("#" + container_id).parent();
	
	if( width < 0 )
		width = parent.width();
		
	if( height < 0 )
		height = parent.width() * 0.77;	
	
	if( !width )
		width = container.clientWidth;

	if( !height )
		height = container.clientHeight;




	/* Find out mode */
	if( mode == undefined )
	{
		mode = "graph";
	}
	
	switch( mode )
	{
		case "graph" :
			mode = 0;
		break;
		case "matrix" :
			mode = 1;
		break;
		default :
			console.log("No such mode for GV 3D");
			return;
		break;
	}
		
	this.width = width;
	this.height = height;

	/* Init three.js */
	this.camera = new THREE.PerspectiveCamera( 95, this.width / this.height, 1, 10000 );
	
	/* Lets attach the magic orbit renderer =) */
	this.controls = new THREE.OrbitControls( this.camera );
	this.controls.addEventListener( 'change', this.render );
	
	this.scene = new THREE.Scene();
	this.mesh = {};
	
	this.renderer = new THREE.WebGLRenderer();
	this.renderer.setClearColor( 0xffffff, 1 );
	this.renderer.setSize( this.width, this.height );

	this.mode = mode;

	switch( mode )
	{
		case 0 :
			this.init_graph();
			this.camera.position.z = 1800;
			gvz_3d_animation_loop.push( this );
		break;
		case 1 :
			this.init_matrix();
			this.camera.position.x = 0;
			this.camera.position.y = 500;
			this.camera.position.z = 0;
			this.camera.lookAt( this.scene.position );
			this.render();
		break;
		default :
			console.log("No such mode for GV 3D");
			return;
		break;
	}
	
}
// --------------------------------

function gvz3d_animate()
{
	requestAnimationFrame( gvz3d_animate );
	
	for( i = 0 ; i < gvz_3d_animation_loop.length ; i++ )
		gvz_3d_animation_loop[i].render();
}

gvz3d_animate();

function gvz3d_onWindowResize()
{
	for( i = 0 ; i < gvz_3d_animation_loop.length ; i++ )
		gvz_3d_animation_loop[i].resize();
}

window.addEventListener( 'resize', gvz3d_onWindowResize, false );


// ---

graphviz3d.prototype.tempColor = function( col, factor )
{
	h=Math.round(factor * 255);

	h+=0.001;

	if ( h < 60 ) {
		f = h / 60.0;
		col.b = 255;

		col.g = f * 255;
		col.r = 0;

	}

	if ( 60 < h && h <= 120 ) {
		f = (h /60.0) - 1;
		col.b = (1 -f) *255;
		col.g = 255;
		col.r = 0;

	}

	if ( 120 < h && h <= 180 ) {
		f = (h /60.0) - 2;
		col.b = 0;
		col.g = 255;
		col.r = f * 255;

	}

	if ( h > 180) {
		f = ((h /60.0) - 3)/1.25;
		col.b = 0;
		col.g = (1-f)*255;
		col.r = 255;

	}

	col.r /= 255;
	col.g /= 255;
	col.b /= 255;
}

// ---

graphviz3d.prototype.init_graph = function()
{
	var geometry = new THREE.BufferGeometry();
	var material = new THREE.LineBasicMaterial({ vertexColors: true  });

	geometry.attributes = {
		position: {
			itemSize: 3,
			array: new Float32Array(2 * this.graph.edges.length * 3)
		},
		color: {
			itemSize: 3,
			array: new Float32Array(2 * this.graph.edges.length * 3)
		}
	};

	var vertexes = geometry.attributes.position.array;
	var lineColors = geometry.attributes.color.array;
	var cnt = 0;
	var color = new THREE.Color();

	for ( var i = 0; i < this.graph.edges.length; i ++ )
	{
		var e_start = this.graph.edges[i][0];
		var e_end = this.graph.edges[i][1];
		var e_color = this.graph.edges[i][2];

		var edge_start = this.graph.vertex[ e_start ];
		var edge_end = this.graph.vertex[ e_end ];

		if( edge_start != undefined  && edge_end != undefined && e_color != undefined)
		{
				this.tempColor( color, e_color );

				vertexes[ cnt * 3 ] = edge_start[0];
				vertexes[ cnt * 3 + 1 ] = edge_start[1];
				vertexes[ cnt * 3 + 2 ] = edge_start[2];

				lineColors[ cnt * 3 ] = color.r;
				lineColors[ cnt * 3 + 1 ] = color.g;
				lineColors[ cnt * 3 + 2 ] = color.b;
				
				cnt++;
				
				vertexes[ cnt * 3 ] = edge_end[0];
				vertexes[ cnt * 3 + 1 ] = edge_end[1];
				vertexes[ cnt * 3 + 2 ] = edge_end[2];
				
				lineColors[ cnt * 3 ] = color.r;
				lineColors[ cnt * 3 + 1 ] = color.g;
				lineColors[ cnt * 3 + 2 ] = color.b;
				
				cnt++;
		}
	}

	this.mesh = new THREE.Line( geometry, material );
	this.mesh.type = THREE.LinePieces;

	
	this.scene.add( this.mesh );
	
	// ---
	this.container.appendChild( this.renderer.domElement );
}


graphviz3d.prototype.init_matrix = function()
{
	var geometry = new THREE.BufferGeometry();

	var scale = 900;
	var y_scale = 1;
	var cnt = this.graph.vertex_count;
	
	var spacedelta = scale / cnt;
	
	console.log( "Cube is " + spacedelta + " wide for " + cnt );
	
	var color = new THREE.Color();

	for ( var i = 0; i < this.graph.edges.length; i ++ )
	{		
		var e_start = this.graph.edges[i][0];
		var e_end = this.graph.edges[i][1];
		var e_color = this.graph.edges[i][2];

		y = Math.floor(e_end * spacedelta );
		x = Math.floor(e_start * spacedelta );
		
		y -= scale / 2;
		x -= scale / 2;

		this.tempColor( color, e_color );

		var cube = new THREE.Mesh(new THREE.CubeGeometry(spacedelta, y_scale, spacedelta),  new THREE.MeshBasicMaterial({color: color}));
		cube.position.x = x;
		cube.position.y = e_color * y_scale/2; 
		cube.position.z = y;
		this.scene.add(cube);

	}

	
	// ---
	this.container.appendChild( this.renderer.domElement );
}


graphviz3d.prototype.render = function()
{
	var time = Date.now() * 0.0007;
	
	if( this.mode == 0 )
	{
		if(0)
		{
			/* GRAPH MODE ==> MOVE MESH */
			this.camera.position.x = Math.sin( time * 0.5 ) * 1600;
			this.camera.position.z = Math.cos( time * 0.1 ) * 1600;
			this.camera.position.y = 500 + 1000 * Math.sin( time * 0.5 );
			this.camera.lookAt( this.scene.position );
		}
	}
	else
	{
		/* MATRIX MODE ==> MOVE CAM */

		//
	}

	if( this.renderer )
		this.renderer.render( this.scene, this.camera );
}

graphviz3d.prototype.resize = function()
{
	if( this.full_window )
		this.renderer.setSize( this.container.clientWidth, this.container.clientHeight );
}




