var catalog = module.require("malp_catalog").catalog;
var Application = module.require("malp_catalog").Application
var fs = require('fs');
var cache = require('nsc');

<page uri="/catalog" parent="/" title="Performance Catalog" />


<view uri="load" raw="true">

<server>
	var a = GET["a"];
	var c = GET["c"];
	var r = GET["r"];

	if( a && c && r )
	{
		if( catalog.loadProfile( a , c, r ) )
			res.end("1");
		else
			res.end("0");
	}

	res.end("0");
</server>

</view>

<view uri="delete" raw="true">

<server>
	var a = GET["a"];
	var c = GET["c"];
	var r = GET["r"];

	if( a && c && r )
	{
		if( catalog.deleteProfile( a , c, r ) )
		{
			res.end("1");
		}
		else
		{
			res.end("0");
		}
		
		catalog.save();
	}

	res.end("0");
</server>

</view>

<body>

<script type="text/javascript" src="/scripts/jquery.js"></script>

<h1>MALP Performance Catalog</h1>

<p>In this section you can manage the performance measurements which are performed with MALP.</p>

<div id="messages"></div>

<div>
	<div class="inline">
	<div id="Apps"></div>
	</div>

	<div class="inline">
	<div id="Cases"></div>
	</div>

	<div class="inline">
	<div id="Runs"></div>
	</div>

	<div class="inline">
	<div id="Controls"></div>
	</div>
</div>

<script>

var apps = [];


function load_run()
{
	var appid = $(this).attr("appid");
	var caseid = $(this).attr("caseid");
	var runid = $(this).attr("runid");
	var procs = $(this).attr("procs");
		
	var app = apps[appid];
	var cs = apps[appid].cases[caseid];
	
	var appname = app.appname;
	var casename = cs.name;
		
	$.getJSON("./load?a=" + appname + "&c=" + casename + "&r=" + runid , function(data)
	{
		if( data == "1" )
		{
			$("#messages").html("Sucessfully loaded " + appname + " ( " + casename + " )");
			$("#current_title").html( appname + " - " + casename + " @ " + procs ); 
		}
		
	});
}

function delete_run()
{
	var appid = $(this).attr("appid");
	var caseid = $(this).attr("caseid");
	var runid = $(this).attr("runid");

	
	var app = apps[appid];
	var cs = apps[appid].cases[caseid];
	
	var appname = app.appname;
	var casename = cs.name;
	
	var answer = confirm("Do you really want to delete " + appname + " ( " + casename + " ) ?")
    if (answer){
		
		$.getJSON("./delete?a=" + appname + "&c=" + casename + "&r=" + runid , function(data)
		{
			if( data == "1" )
			{
				$("#messages").html("Sucessfully loaded " + appname + " ( " + casename + " )"); 
			}
			
		});
	
		$.getJSON('./apps', gen_apps );
	}
}

var current_run = undefined;

function gen_actions()
{
	var appid = $(this).attr("appid");
	var caseid = $(this).attr("caseid");
	var runid = $(this).attr("runid");
	var procs = $(this).attr("procs");

	if( current_run )
	{
		current_run.css("color", "#FFFFFF");
	}
	
	$(this).css("color", "#FF0000");
	
	current_run = $(this);
	
	$("#Controls *").remove();
	
	$("#Controls").append( "<div class='action catalogentry' id='load' appid='" + appid + "'  caseid='" + caseid + "'  runid='" + runid  + "' + procs='" + procs + "'>Load</div>" );
	$("#load").bind( 'click', load_run );
	
	$("#Controls").append( "<div class='action catalogentry' id='delete' appid='" + appid + "'  caseid='" + caseid + "'  runid='" + runid  + "' + procs='" + procs + "'>Delete</div>" );
	$("#delete").bind( 'click', delete_run );
}

var current_case = undefined;


function gen_runs()
{
	var appid = $(this).attr("appid");
	var caseid = $(this).attr("caseid");

	var runs = apps[appid].cases[caseid].runs;

	$("#Runs *").remove();
	$("#Controls *").remove();
	
	if( current_case )
	{
		current_case.css("color", "#FFFFFF");
	}
	
	$(this).css("color", "#FF0000");
	
	current_case = $(this);

	for( var i = 0 ; i < runs.length ; i++ )
	{
		$("#Runs").append( "<div class='run catalogentry' appid='" + appid + "'  caseid='" + caseid + "'  runid='" + i + "' + procs='" + runs[i].size + "'>"+ runs[i].size + "</div>" );
	}
	
	$(".run").bind( 'click', load_run );
	$(".run").bind( 'mouseover', gen_actions);
}

var current_app = undefined;

function gen_cases()
{
	var appid = $(this).attr("appid");

	var cases = apps[appid].cases;
	
	if( current_app )
	{
		current_app.css("color", "#FFFFFF");
	}
	
	$(this).css("color", "#FF0000");
	
	current_app = $(this);

	$("#Cases *").remove();
	$("#Runs *").remove();
	$("#Controls *").remove();

	for( var i = 0 ; i < cases.length ; i++ )
	{
		$("#Cases").append( "<div class='case catalogentry' appid='" + appid + "'  caseid='" + i + "'>"+ cases[i].name + "</div>" );
	}

	$(".case").bind( 'mouseover', gen_runs);
							
}


function gen_apps( data )
{
/* Fill in apps */
	apps = data;
	
	$("#Apps *").remove();
	$("#Cases *").remove();
	$("#Runs *").remove();
	$("#Controls *").remove();

	for( var i = 0 ; i < apps.length ; i++ )
	{
		if( apps[i] != undefined )
			$("#Apps").append( "<div class='app catalogentry' appid='" + i + "'>"+apps[i].appname+"</div>" );
	}

	$(".app").bind( 'mouseover', gen_cases );
}



$(document).ready( function()
{
	
	$.getJSON('./apps', gen_apps );
	
});

</script>


</body>


<view uri="apps"  raw="true">
<server>
	var appList = catalog.appList();

	res.end(JSON.stringify( appList ));
</server>

</view>


