var catalog = module.require("malp_catalog").catalog;
var prof = require("malp_rendering").profile;
var meta = require("malp_rendering").metadata;
var conv = require("malp_rendering").converter;

<page uri="/perrank" parent="/" title="Per Rank Analysis" />

var Profile_Entries = [];


function generate_graph_data()
{	
	var current_prof = catalog.getProfile();

	if( current_prof == undefined )
	{
		return;
	}

	Profile_Entries = [];

	var mpip = current_prof["MPI_Profile"];
	var posixp = current_prof["POSIX_Profile"];

	/*
	 * Profile : {
		 * 
		 * prof : 
		 * [
		 * {
			 * hits : 16
			 * size : 0
			 * name : ""
			 * time : 0 
		* } ]
		* 
			* streams:
			* [
			* 		{
				* 		tid : "1"
				* 		prof:
				* 		[
				* 			{
					* 			hits: 11
					* 			...
					* 		}
			* 			]
			* 		}
			* ]
		* 
		* }
	 * 
	 * 
	 */


	/* Generate a MAP indexed by TIDs */

	var stream_map = {};

	/* Walk MPI */
	for( var i = 0 ; i < mpip.streams.length ; i++ )
	{
		var tid = mpip.streams[i].tid;

		stream_map[ tid ] = [];
		stream_map[ tid ] = stream_map[ tid ].concat( mpip.streams[i].profile );
	}

	/* Walk POSIX */
	for( var i = 0 ; i < posixp.streams.length ; i++ )
	{
		var tid = posixp.streams[i].tid;
	
		if( stream_map[ tid ] == undefined )
			stream_map[ tid ] = [];

		stream_map[ tid ] = stream_map[ tid ].concat( posixp.streams[i].profile );
	}
	
	/* We now compute the "Other" time metric by using the walltime */
	for( var tid in stream_map )
	{
		if( !tid )
			continue;

		var total_time = 0;
		var p = stream_map[ tid ];

		for( var j = 0 ; j < p.length ; j++ )
		{
			total_time += p[j].time;
		}
		
		/* Push the remaining time */
		p.push({name : "Other" , hits : 0 , time : meta.stream_duration( tid ) - total_time , size : 0});
	}
	
	/* At this point we have an object indexed with tids with merged POSIX & MPI profiles */
	
	var _Profile_Entries = {};

	/* Flag possible keys Profile_Entries is then an array indexed by possible calls
	 * either posix or mpi we do this to make sure that each call is acounted
	 * for only once */

	for( var tid in stream_map )
	{
		if( !tid )
			continue;

		var p = stream_map[ tid ];

		for( var j = 0 ; j < p.length ; j++ )
		{
			_Profile_Entries[ p[j].name ] = { "key" : p[j].name,  "values" : [] };
		}
	}
	
	/* We now build a rank to TID table */
	var rank_list = [];
	
	var id = 0;
	
	for( var tid in stream_map )
	{
		if( !tid )
			continue;
			
		var d = meta.getdesc( tid );
		
		if( d.type == "process" )
		{
			rank_list.push( [ d.mpi_rank, tid ] );
		}
	}
	
	/* We sort it in terms of rank */
	rank_list.sort( function( a , b )
					{
						if( a[0] < b[0] )
							return -1;
						if( b[0] < a[0] )
							return 1;
						
						return 0;
					});
	
	var function_thieves = {};
	
	/* And now we can fill the _Profile_Entries object */
	for( var i = 0 ; i < rank_list.length ; i++ )
	{
		var tid = rank_list[i][1];
		var rank = rank_list[i][0];
		
		if( !tid )
			continue;

		var d = meta.getdesc( tid );

		if( d.type == "process" )
		{
			/* Retrieve the profile for current rank */
			var p = stream_map[ tid ];
			
			for( var k in _Profile_Entries )
				function_thieves[k] = 1;

			/* Push all profile infos at the right place */
			for( var j = 0 ; j < p.length ; j++ )
			{
				_Profile_Entries[ p[j].name ].values.push( [rank, p[j].time, p[j].hits, p[j].size] );
				function_thieves[ p[j].name ] = 0;
			}
			
			for( var k in _Profile_Entries )
			{
				if( function_thieves[k] )
				{
					_Profile_Entries[k].values.push( [rank, 0.0, 0.0, 0.0] );
				}
			}
		}
		
	}
	

	/* Now that we used the object for unicity put it back to an array */
	for( var entry in _Profile_Entries )
	{
		if( !entry )
			continue;
		
		Profile_Entries.push( _Profile_Entries[entry] );
	}
		
	generate_graph_data_done = 1;
}


<view uri="functions">

	<server>
		generate_graph_data();
	</server>
	<script src='/scripts/jquery.js'></script>
	<script type="text/javascript" src="/scripts/d3.js"></script>
	<link rel="stylesheet" type="text/css" href="/css/nv.d3.css"/>
	<script type="text/javascript" src="/scripts/nv.d3.js"></script>

	<server>
		if(! GET["body_only"] )
			res.write("<h1>Time Breakdown over Functions</h1>");
	</server>
	

	<svg id="chart_fct" width='80%' height='800px'></svg>

	<script>
	
	<server>
		res.write("var data = " + JSON.stringify(Profile_Entries) + ";");
	</server>
	
	$(document).ready( function()
	{
		var chart = nv.models.stackedAreaChart()
						.margin({right: 100})
						.x(function(d) { return d[0]; }) //We can modify the data accessor functions...
						.y(function(d) { return d[1]; }) //...in case your data is formatted differently.
						.useInteractiveGuideline(true) //Tooltips which show all data points. Very nice!
						.transitionDuration(500)
						.showControls(true);
						 
		chart.xAxis.tickFormat(d3.format('d'));
		chart.yAxis.tickFormat(d3.format('d'));
		 
		d3.select('#chart_fct')
		.datum(data)
		.call(chart);
		 
		nv.utils.windowResize(chart.update);
	});
	</script>
</view>


<body>
	<script src='/scripts/jquery.js'></script>
	<div id="cat"></div>
	<div id="func"></div>

	<script>
	$(document).ready( function()
	{
		    $("#breakdown").hide();
			$("#cat").hide();
			$("#func").hide();
			
			$.ajax({url: "./functions?body_only=1"})
			.done(function( data ) {
					$("#func").html( data ).fadeIn();
				  });
	});
	</script>
</body>
