var nsc = require('nsc');

<page uri="/pi" parent="" title="The Interactive PI example" />


<body>

	<h1>Pi Computation Interface</h1>
	
	<h2>Principle</h2>
	
	Use the derivative of arctan to compute a multiple of Pi:
	<br>
	<br>
	<img src="/img/comp.gif"></img>
	<br>	
	<br>	
	The approximation is done using the rectangle method:
	<br>
	<img src="/img/rect.gif"></img>
	<br>
	<span style="font-size:10px; color:grey">
	Illustration by Khurram Wadee (MKWadee), created with wxmaxima found on Wikipedia
	</span>
	
	

	<h2>Interpolation Parameters</h2>

	<span style="font-size : 30px">

    Points: <input type="number" id="points" value="1e9">
    <button id="comp">compute</button>
	
	</span>
	
	<h2>A closer Value of Pi</h2>
	
	<div id="actualpi"></div>
	
	<h2>PI Computation Results</h2>

	<span style="font-size : 30px">

	
	<table class="malptable" style="font-size : 30px">
		<tr>
			<td>Points</td>
			<td>Errors</td>
			<td>Value</td>
		</tr>
		<tr>
			<td><div id="pointsval">1e9</div></td>
			<td><div id="errors"></div></td>
			<td><div id="pi"></div></td>
		</tr>
	</table>

	</span>
	

	<script src='/scripts/jquery.js'></script>
	<script type="text/javascript" src="/scripts/d3.js"></script>
	<link rel="stylesheet" type="text/css" href="/css/nv.d3.css"/>
	<script type="text/javascript" src="/scripts/nv.d3.js"></script>

	
	<script type="text/javascript">
		var pi = 0.0;
	    var actualpi = "3.14159265358979323846264338327950288419716939";
	
		function renderpi( data )
		{
			data = data + "";
	
			var i;
			var html = "";

			for( i = 0 ; i < data.length; i++ )
			{
				var canditate =  data.substring( i , i + 1);
				var actual =  actualpi.substring( i , i + 1);
				
				if( canditate == actual )
				{
					html += "<font color='green'>" + canditate + "</font>"
				}
				else
				{
					html += "<font color='red'>" + canditate + "</font>"
				}
			}
			
			
			return html;
		}
	
		function errorspi( data )
		{
			data = data + "";

			var i;
			var html = "";
	
	        var errors = 0;		
	
			for( i = 0 ; i < data.length; i++ )
			{
				var canditate =  data.substring( i , i + 1);
				var actual =  actualpi.substring( i , i + 1);
				
				if( canditate != actual )
				{
					errors++;
				}
			}
			
			
			return errors;
		}
	
		$(document).ready( function()
		{
			$("#actualpi").html(actualpi);
			
			setInterval(function() 
			{
				$.getJSON( "getpi", function( data ) 
				{
					if( data != "" )
					{
						$("#errors").html(errorspi( data ) );
						$("#pi").html(renderpi( data ) );
					}
				});
			}, 1000);
			
			$("#comp").on("click", function()
			{
				$("#pointsval").html($("#points").val());
				$("#pi").empty();
				$("#errors").empty();
				$.ajax({
				  url: "setpoints?p=" + $("#points").val(),
				  context: document.body
				});
			});
		});
	</script>



</body>


<view uri="setpoints" raw="true">

	<server>
		console.dir( GET );
		nsc.set("value", "" );
		nsc.set("points", parseFloat(GET.p) );
		res.end("OK");
	</server>

</view>

<view uri="getpi" raw="true">
	<server>
		res.end(JSON.stringify( nsc.cache.value ));
	</server>
</view>
