var cache = require('nsc');
var gvz3d = require('nodeGraphviz');
var catalog = module.require("malp_catalog").catalog;
var conv = require("malp_rendering").converter;
var timeMatrixScale = require("malp_rendering").timeMatrixScale;

<page uri="/networktopo" parent="/" title="Network Topology" desc="This modules displays network topology in 3D" />



<body>

<h1>Topology Analysis</h1>

<p>
Choose a type of MPI points to points call in the following table in order to visualize
its topology.
</p>

<table class="malptable">

<tr>
	<th>
		MPI Call
	</th>
	<th>
		Global View
	</th>
	<th>
		Calls
	</th>
	<th>
		Time
	</th>
	<th>
		Size
	</th>
</tr>

<server>
	var current_prof = catalog.getProfile();

	if( !current_prof )
	{
		res.write("<h1>No profile loaded</h1>");
		return;
	}

	var collector = current_prof["Topology"];

	if( !collector )
		return;

	var i;

	for( e in collector )
	{
		if( e == "name" )
			continue;
		
		res.write("<tr>");
		res.write("<td>" + e + "</td>");
		res.write("<td><a href=\"/networktopo/render?t="+ e + "\">Combined</a></td>");
		res.write("<td><a href=\"/networktopo/renderfs?t="+ e + "&m=hits\">Fullscreen</a></td>");
		res.write("<td><a href=\"/networktopo/renderfs?t="+ e + "&m=time\">Fullscreen</a></td>");
		res.write("<td><a href=\"/networktopo/renderfs?t="+ e + "&m=size\">Fullscreen</a></td>");
		res.write("</tr>");
	}

</server>


</table>

</body>


<view uri="graph" raw="true">

	<server>

		var g = new gvz3d();
		g.loadFile( "./m.dot" );

		g.on('dotParsed', function()
		{
				res.end( g.JSON() );
		});
	</server>

</view>



<view uri="renderfs">

<server>
var type = GET.t;

if( type == undefined )
	return;
	
var metric = GET.m;

if( metric == undefined )
	return;

</server>


<script src='/scripts/three.js'></script>
<script src='/scripts/OrbitControls.js'></script>
<script src='/scripts/jquery.js'></script>
<script src='/scripts/graphviz3d.js'></script>


<div id="container" class="gl"></div>

<script>


	function genscale( target_cell, func, metric )
	{
		$.getJSON('./minmax?t='+ func +'&m=' + metric, function(data)
		{
			$(target_cell).append("<div class='centered'><table width='100%'><tr><td align='center'>"+data[0]+"</td><td><img  border='1px' width='100%' height='80px' src='/networktopo/scale'/></td><td align='center'>"+data[1]+"</td></tr></table></div>");
		} );
		
		
	}

<server>
	res.write("var type = \"" + type + "\";\n");
	res.write("var metric = \"" + metric + "\";\n");
</server>

	$( document ).ready( function ()
	{
		$.getJSON("/networktopo/data?q=" + Math.random() + "&t="+ type +"&m="+ metric +"", function(json){
			var graph1 = new graphviz3d( json, "container", -1 , -1 );
			genscale("#container", type, metric);
		});
	});

</script>



</view>



<view uri="render">

<server>
var type = GET.t;

if( type == undefined )
	return;


res.write("<h1>" + type + " Topology</h1>");

</server>

<table class="malptable">
<tr>
	<th>Calls</th>
	<th>Time</th>
	<th>Size</th>
</tr>
<tr>

	<td>
		<div id="cont_hits" class="gl"></div>
	</td>

	<td>
		<div id="cont_time" class="gl"></div>
	</td>

	<td>
		<div id="cont_size" class="gl"></div>
	</td>

</tr>
</table>


<script src='/scripts/three.js'></script>
<script src='/scripts/OrbitControls.js'></script>
<script src='/scripts/jquery.js'></script>
<script src='/scripts/graphviz3d.js'></script>

<script>

	function genscale( target_cell, func, metric )
	{
		$.getJSON('./minmax?t='+ func +'&m=' + metric, function(data)
		{
			$(target_cell).append("<div class='centered'><table width='100%'><tr><td align='center'>"+data[0]+"</td><td><img  border='1px' width='100%' height='80px' src='/networktopo/scale'/></td><td align='center'>"+data[1]+"</td></tr></table></div>");
		} );
		
		
	}


<server>
		res.write("var type=\"" + type + "\";\n"); 
</server>

		$( document ).ready( function ()
		{
			var w = (0.55 * $(this).width())/3;
		 
			 $.getJSON("/networktopo/data?q=" + Math.random() + "&t="+ type +"&m=hits", function(json){
				var graph1 = new graphviz3d( json, "cont_hits",  w ,  w );
				genscale( "#cont_hits", type, "hits" );
			});
			$.getJSON("/networktopo/data?q=" + Math.random() + "&t="+ type +"&m=time", function(json){
				var graph2 = new graphviz3d( json, "cont_time",  w,  w );
				genscale( "#cont_time", type, "time" );
			});
			$.getJSON("/networktopo/data?q=" + Math.random() + "&t="+ type + "&m=size", function(json){
				var graph3 = new graphviz3d( json, "cont_size",  w,  w );
				genscale( "#cont_size", type, "size" );
			});
		
		});
</script>


</view>



<view uri="data" raw="true">

	<server>
		var current_prof = catalog.getProfile();

		if( !current_prof )
		{
			res.write("<h1>No profile loaded</h1>");
			return;
		}

		var collector = current_prof["Topology"];
	
		var type = GET.t;
		var metric = GET.m;
		
		if( type == undefined )
			type = "MPI_Send";

		if( metric == undefined )
			metric = "hits";

		if( collector == undefined )
		{
			res.end( "{}" );
			return;
		}
		
		var comm_array = collector[ type ];
		
		if( comm_array == undefined )
		{
			res.end( "{}" );
			return;
		}

		var dot = "Digraph G{\n";
	
	
		var max = 0;
		var i = 0;
		
		
		for( i=0; i < comm_array.length; i++ )
		{			
			if(max < comm_array[i][metric])
				max = comm_array[i][metric];
		}
	
		
		
		for( i=0; i < comm_array.length; i++ )
		{
				dot += comm_array[i].src + " -> " + comm_array[i].dest + " [val=\"" + comm_array[i][metric]/max + "\"]\n";
		}
		
		dot += "\n}";

		var g = new gvz3d();
		
		g.setDot( dot );
		
		g.on('dotParsed', function()
		{
				res.end( g.JSON() );
		});
	</server>

</view>

<view uri="scale" raw="true">
	<server>
	var width = GET.w;
	var height = GET.h;
	
	
	if( width == undefined )
		width = 512;
	
	if( height == undefined )
		height = 80;

	scale_type = "temperature";

	/* Gen scale image */
	timeMatrixScale( res, scale_type, width,  height);
</server>


</view>



<view uri="minmax" raw="true">

	<server>
		var current_prof = catalog.getProfile();

		if( !current_prof )
		{
			res.write("<h1>No profile loaded</h1>");
			return;
		}

		var collector = current_prof["Topology"];
	
		var type = GET.t;
		var metric = GET.m;
		
		if( type == undefined )
			type = "MPI_Send";

		if( metric == undefined )
			metric = "hits";

		if( collector == undefined )
		{
			res.end( "{}" );
			return;
		}
		
		var comm_array = collector[ type ];
		
		if( comm_array == undefined )
		{
			res.end( "{}" );
			return;
		}

		var min = -1;
		var max = -1;
		

		
		for( i=0; i < comm_array.length; i++ )
		{			
			if((max < comm_array[i][metric]) || ( max == -1) )
				max = comm_array[i][metric];
			
			if((comm_array[i][metric] < min) || ( min == -1))
				min = comm_array[i][metric];
		}
	
		var min_string = "";
		var max_string = "";
		
		switch( metric )
		{
			case "time":
				min_string = conv.time_unit( min );
				max_string = conv.time_unit( max );
			break;
			
			case "hits":
				min_string = min + " hit(s)";
				max_string = max + " hit(s)";
			break;
			
			case "size":
				min_string = conv.size_unit( min );
				max_string = conv.size_unit( max );	
			break;
		}
		
		var ret = [min_string, max_string];
		
		res.end(JSON.stringify( ret ));	
	</server>

</view>






 
