/* ############################ MALP License ############################## */
/* # Fri Jan 18 14:00:00 CET 2013                                         # */
/* # Copyright or (C) or Copr. Commissariat a l'Energie Atomique          # */
/* #                                                                      # */
/* # This software is governed by the CeCILL-C license under French law   # */
/* # and abiding by the rules of distribution of free software.  You can  # */
/* # use, modify and/ or redistribute the software under the terms of     # */
/* # the CeCILL-C license as circulated by CEA, CNRS and INRIA at the     # */
/* # following URL http://www.cecill.info.                                # */
/* #                                                                      # */
/* # The fact that you are presently reading this means that you have     # */
/* # had knowledge of the CeCILL-C license and that you accept its        # */
/* # terms.                                                               # */
/* #                                                                      # */
/* # Authors:                                                             # */
/* #   - BESNARD Jean-Baptiste jean-baptiste.besnard@cea.fr               # */
/* #                                                                      # */
/* ######################################################################## */

/**
 * @addtogroup CRC64_
 * @{
 */

#ifndef MALP_CRC64_H
#define MALP_CRC64_H

#ifdef __cplusplus
extern "C"
{
#endif

#include <stdio.h>
#include <stdint.h>

/**
 * @brief This initializes the CRC64 internals
 *
 * This function fills the table (crc_table)
 */
void MALP_Trace_crc64_init();

/**
 * @brief crc_table
 */
extern uint64_t crc_table[256];

/**
 * @brief Computes the hash of a given data
 * @param source buffer containing the data to be hashed
 * @param size the size of the buffer
 * @return the hashcode (64 bits)
 */
static inline uint64_t MALP_Trace_crc64(char *source, uint64_t size)
{
	uint64_t crc = 0xFFFFFFFFFFFFFFFFULL;

	uint64_t i = 0;

	for( i = 0 ; i < size ; i++ )
		crc = crc_table[(crc ^ source[i]) & 0xff] ^ (crc >> 8);

	//printf ("%16lX\n", crc);

	return crc;
}

#ifdef __cplusplus
}
#endif

#endif

/**
 * @}
 */
