#include "Encodings.h"
#include "CRC64.h"

#include <string.h>
#include <stdlib.h>
#include <unistd.h>


static int CRC_INIT = 0;

/* Here we use a simple one time pad strategy */
void Encrypt_apply_pad( char * string , uint64_t seed )
{

	/* Initialize the one time pad generator */
	
	srandom( seed );
	
	int i = 0;
	
	/* Create the one time pad */
	int string_l = strlen( string );
	
	if( !string_l )
	{
		/* Nothing to do */
		return;
	}
	
	char * pad = (char *)malloc( string_l );
	
	if( ! pad )
	{
		perror("malloc");
		abort();
	}

	/* In order to complexify decription we pull off some numbers */
	while( i < (seed%16384) )
	{
		pad[i%string_l] = random()%128;
		i++;
	}
	
	/* This code is useless just here to jam the ASM */
	char moo1[16];
	char moo2[32];
	char moo3[64];
	char moo4[16];
	char moo5[32];
	char moo6[64];
	int u;
	
	for( u = 0 ; u < string_l ; u++ )
	{
		moo1[u%16] = string[u];
		moo2[u%32] = string[u] + 2;
		moo3[u%64] = string[u] + 3;
		moo4[u%16] = string[u] + 4;
		moo5[u%32] = string[u] + 5;
		moo6[u%64] = string[u] + 6;
	}
	/*----------------------*/
	
	for( i = 0 ; i < string_l ; i++ )
	{
		/* This code is useless just here to jam the ASM */
		pad[i] = moo1[i%16] ^ moo2[i%16] ^ moo3[i%16] ^ moo4[i%16] ^ moo5[i%16] ^ moo6[i%16];
		pad[i] = string[i] ^ i;
		pad[i] = moo1[i%16] - moo2[i%16] * moo3[i%16] + moo4[i%16] ^ moo5[i%16] * moo6[i%16];
		/*----------------------*/
		pad[i] = (i + seed)%128;
	}

	/* Apply the pad */
	for( i = 0 ; i < string_l ; i++ )
	{
		/* Restore NULL chars */
		if( string[i] == (char)-1 )
			string[i] = '\0';
		
		/* This code is useless just here to jam the ASM */
		moo3[0] = string[i];
		string[i] =  moo1[i%4] - moo2[i%8] * moo3[i%4] + moo4[i%8] ^ moo5[i%4] * moo6[i%8];
		string[i] ^=  moo1[i%4] - moo2[i%8] * moo3[i%4] + moo4[i%8] ^ moo5[i%4] * moo6[i%8];
		string[i] &=  moo1[i%4];
		moo1[i%16] = string[i];
		memfrob( moo1, 16 );
		string[i] = moo3[0];
		/*----------------------*/
		char c = string[i];
		string[i] ^= pad[i];

		/* Make sure we do not truncate the string using char signedness to store NULLs */
		if( string[i] == '\0' )
			string[i] = (char)-1;
	}
	
	
	
	free( pad );
}

char * Encrypt_string( char * string , char * key )
{
	int string_l = strlen( string );
	char *ret = (char *)realloc( string, string_l + 64 );
	
	if( !ret )
	{
		perror("realloc");
		return NULL;
	}
	
	if( !CRC_INIT )
	{
		MALP_Trace_crc64_init();
		CRC_INIT = 1;
	}
	
	int key_l = strlen(key);
	uint64_t first_seed = MALP_Trace_crc64( key,  key_l);
	uint64_t second_seed = MALP_Trace_crc64( (char *)&first_seed,  sizeof( uint64_t) );

	/* We do the first encryption */
	Encrypt_apply_pad( ret, first_seed );

	/* We adjunct the CRC64 of the encrypted string */
	uint64_t first_pass_crc = MALP_Trace_crc64( ret,  string_l);
	char scrc[64];
	sprintf(scrc, "%016llx",first_pass_crc );
	strncat( ret, scrc, string_l + 16 );

	/* And we do the second pass encryption using the second seed */
	Encrypt_apply_pad( ret, second_seed );

	return ret;
}

void frighten()
{
		printf("ERROR : Data have been tampered with !! This will be reported...\n");
		/* Print a message to frighten the attacker */
		printf("Sending report...\n");
		char host[128];
		char cwd[128];
		char user[128];
		gethostname(host, 128);
		getcwd(cwd, 128);
		getlogin_r(user, 128);
		printf("-------------------------------------------\n");
		printf("%s@%s in (%s) tampered a file\n", user, host, cwd );
		printf("------------------------------------------\n");
		printf("OK !\n");
}


void Decrypt_string( char * string , char * key )
{
	if( !CRC_INIT )
	{
		MALP_Trace_crc64_init();
		CRC_INIT = 1;
	}

	
	int key_l = strlen(key);
	uint64_t first_seed = MALP_Trace_crc64( key,  key_l);
	uint64_t second_seed = MALP_Trace_crc64( (char *)&first_seed,  sizeof( uint64_t) );
	
	/* First decrypt the container with the second seed */
	Encrypt_apply_pad( string, second_seed );
	
	/* We now perform CRC comparison */
	
	/* Compute the CRC we do this to hide our quite linear 
	 * pad which would be sensitive to bbox attacks */
	int payload_length = strlen( string ) - 16;
	
	if( payload_length < 0 )
	{
		frighten();
		return;
	}

	uint64_t first_pass_crc = MALP_Trace_crc64( string,  payload_length);
	
	/* Render it */
	char scrc[64];
	memset( scrc, 0 , 64 );
	sprintf(scrc, "%016llx",first_pass_crc );

	/* Compare with payload */
	char *pl_crc = NULL;
	pl_crc = string + payload_length;
	
	if( strcmp( pl_crc, scrc ) )
	{
		frighten();
		return;
	}
	else
	{
		*pl_crc = '\0';
	}
	
	/* Now that we verified the signature we can decrypt */
	Encrypt_apply_pad( string, first_seed );
	
}


void Read_file( char * buff, char *path, size_t max_size )
{
	if( !buff||!max_size )
		return;
		
	
	buff[0]='\0';

	/* Read file */	
	FILE *in = fopen( path, "r" );
	
	if( !in )
	{
		perror("fopen");
		return;
	}

	char tmp_buff[500];
	size_t off = 0;

	while( fgets(tmp_buff, 500, in) != NULL )
	{
		strcat( buff, tmp_buff );
		off+=strlen(tmp_buff);
		
		if( max_size <= off )
		{
			fclose( in );
			return;
		}
	}
	
	fclose( in );
	
}

void Dump_string( char *string, FILE * out )
{
	if( !out )
	{
		return;
	}	
	
	int to_write = strlen( string );
	
	int ret = fwrite( string, sizeof( char ) , to_write, out );

	if( ret != (to_write * sizeof( char ) ) )
	{
		perror( "fwrite" );
		return;
	}
	
	fflush( out );
}


#define MAX_BUFF_SIZE 1024*1024*10


void Encrypt_File( char *in_path, FILE *out, char *key )
{
	char * buff = (char *)malloc( MAX_BUFF_SIZE );

	if( !buff )
	{
		perror("malloc");
		return;
	}

	Read_file( buff, in_path, MAX_BUFF_SIZE );

	Encrypt_string( buff, key );
	
	Dump_string( buff, out );

	free( buff );
}

void Decrypt_File( char *in_path, FILE *out, char *key )
{
	char * buff = (char *)malloc( MAX_BUFF_SIZE );

	if( !buff )
	{
		perror("malloc");
		return;
	}

	Read_file( buff, in_path, MAX_BUFF_SIZE );

	Decrypt_string( buff, key );
	
	Dump_string( buff, out );

	free( buff );
}
