/* ############################ MALP License ############################## */
/* # Fri Jan 18 14:00:00 CET 2013                                         # */
/* # Copyright or (C) or Copr. Commissariat a l'Energie Atomique          # */
/* #                                                                      # */
/* # This software is governed by the CeCILL-C license under French law   # */
/* # and abiding by the rules of distribution of free software.  You can  # */
/* # use, modify and/ or redistribute the software under the terms of     # */
/* # the CeCILL-C license as circulated by CEA, CNRS and INRIA at the     # */
/* # following URL http://www.cecill.info.                                # */
/* #                                                                      # */
/* # The fact that you are presently reading this means that you have     # */
/* # had knowledge of the CeCILL-C license and that you accept its        # */
/* # terms.                                                               # */
/* #                                                                      # */
/* # Authors:                                                             # */
/* #   - BESNARD Jean-Baptiste jean-baptiste.besnard@cea.fr               # */
/* #                                                                      # */
/* ######################################################################## */
#include "MALP_Blackboard.h"

#include <dlfcn.h>

uint64_t  MALP_Blackboard_level_id( char *MALP_Blackboard_level_name, char *object1_name, char *object2_name )
{
	uint64_t tab[3];
	
	tab[0] = MALP_Trace_crc64( MALP_Blackboard_level_name, strlen(MALP_Blackboard_level_name) );
	tab[1] = MALP_Trace_crc64( object1_name, strlen(object1_name) );
	tab[2] = MALP_Trace_crc64( object2_name, strlen(object2_name) );

	return MALP_Trace_crc64( (char *)tab, sizeof( uint64_t ) * 3 );
}



struct MALP_Blackboard_level * MALP_Blackboard_level_load(  struct MALP_blackboard *bb , char *plugin_soname, void *arg )
{
	void *lib_handle = dlopen(plugin_soname, RTLD_NOW);

	if( !lib_handle )
	{
		 printf("Failed to load %s BB plugin\n", plugin_soname );
		 perror("dlopen");
		 return NULL;
	}

	struct MALP_Blackboard_level *ret = malloc( sizeof( struct MALP_Blackboard_level ) );

	if( !ret )
	{
		 perror("Failled to allocate BLackboard level");
		 dlclose( lib_handle );
		 return NULL;
	}

	ret->BB_level_setup = dlsym(lib_handle, "BB_level_setup");

	if( !ret->BB_level_setup  )
	{
		 perror("Could not load 'BB_level_setup' function from plugin" );
		 dlclose( lib_handle );
		 return NULL;
	}

	ret->BB_level_teardown = dlsym(lib_handle, "BB_level_teardown");

	if( !ret->BB_level_teardown )
	{
		 perror("Could not load 'BB_level_teardown' function from plugin" );
		 dlclose( lib_handle );
		 return NULL;
	}
	
	ret->arg = arg;
	ret->lib_handle = lib_handle;
	
	printf("MALP Blackboard sucessfully loaded plugin %s\n", plugin_soname );
	
	int tmp = (ret->BB_level_setup)( bb, arg );
	
	if( tmp )
	{
		printf("Error setting up %s plugin\n", plugin_soname );
		dlclose( lib_handle );
		return NULL;
	}
	
	
	return ret;
}


struct MALP_Blackboard_level * MALP_Blackboard_level(  struct MALP_blackboard *bb , 
														   int (*BB_level_setup)( struct MALP_blackboard *bb, void *arg ),
														   int (*BB_level_teardown)( struct MALP_blackboard *bb, void *arg ) , void *arg )
{
   
	struct MALP_Blackboard_level *ret = malloc( sizeof( struct MALP_Blackboard_level ) );

	if( !ret )
	{
		 perror("Failled to allocate BLackboard level");
		 return NULL;
	}
	
	
	ret->arg = arg;
	ret->BB_level_setup = BB_level_setup;
	ret->BB_level_teardown = BB_level_teardown;
	ret->lib_handle = NULL;
 
	if( (ret->BB_level_setup)( bb, arg ) )
	{
		printf("Error setting up plugin\n" );
		return NULL;
	}
 
	return ret;
}


int MALP_Blackboard_level_unload(  struct MALP_blackboard *bb , struct MALP_Blackboard_level *level )
{
	if( !level )
		return 0;
	
	int ret = (level->BB_level_teardown)( bb, level->arg );
	
	if( ret )
	{
		printf("Error releasing a plugin\n");
	}
	
	dlclose( level->lib_handle );
	level->lib_handle = NULL;
	
	free( level );
	
	return 1;
}
