/* ############################ MALP License ############################## */
/* # Fri Jan 18 14:00:00 CET 2013                                         # */
/* # Copyright or (C) or Copr. Commissariat a l'Energie Atomique          # */
/* #                                                                      # */
/* # This software is governed by the CeCILL-C license under French law   # */
/* # and abiding by the rules of distribution of free software.  You can  # */
/* # use, modify and/ or redistribute the software under the terms of     # */
/* # the CeCILL-C license as circulated by CEA, CNRS and INRIA at the     # */
/* # following URL http://www.cecill.info.                                # */
/* #                                                                      # */
/* # The fact that you are presently reading this means that you have     # */
/* # had knowledge of the CeCILL-C license and that you accept its        # */
/* # terms.                                                               # */
/* #                                                                      # */
/* # Authors:                                                             # */
/* #   - BESNARD Jean-Baptiste jean-baptiste.besnard@cea.fr               # */
/* #                                                                      # */
/* ######################################################################## */
#ifndef MALP_BB_L_H
#define MALP_BB_L_H

#include <stdint.h>
#include "MALP_Blackboard.h"
#include "CRC64.h"

/**
 * @ingroup MALP_Blackboard_Level_
 * @brief The MALP_Blackboard_level structure
 */
struct MALP_Blackboard_level
{
    void *arg;          /**< Arguments */
    void *lib_handle;   /**< Pointer to a handler's mapping library */
    int (*BB_level_setup)( struct MALP_blackboard *bb, void *arg );     /**< Handler */
    int (*BB_level_teardown)( struct MALP_blackboard *bb, void *arg );  /**< Handler */
};


/**
 * @ingroup MALP_Blackboard_Level_
 * @brief MALP_Blackboard_level_load [UNUSED?]
 * @param bb
 * @param plugin_soname Handler's mapping library
 * @param arg
 * @return
 */
struct MALP_Blackboard_level * MALP_Blackboard_level_load(  struct MALP_blackboard *bb , char *plugin_soname, void *arg );

/**
 * @ingroup MALP_Blackboard_Level_
 * @brief Create a MALP_Blackboard_level structure using the parameters with NULL value for the handler's mapping library
 * @param bb the blackboard
 * @param BB_level_setup the setup handler
 * @param BB_level_teardown the teardown handler
 * @param arg argument to give to setup and teardown handlers
 * @return a MALP_Blackboard_level structure's pointer
 */
struct MALP_Blackboard_level * MALP_Blackboard_level(  struct MALP_blackboard *bb , 
														   int (*BB_level_setup)( struct MALP_blackboard *bb, void *arg ),
														   int (*BB_level_teardown)( struct MALP_blackboard *bb, void *arg ) , void *arg );

/**
 * @ingroup MALP_Blackboard_Level_
 * @brief MALP_Blackboard_level_unload [UNUSED?]
 * @param bb
 * @param level
 * @return
 */
int MALP_Blackboard_level_unload(  struct MALP_blackboard *bb , struct MALP_Blackboard_level *level );

/**
 * @ingroup MALP_Blackboard_Level_
 * @brief Generate the hashcode for a 3 dimensions table buffer containing a blackboard level and two object names
 * @param MALP_Blackboard_level_name
 * @param object1_name
 * @param object2_name
 * @return CRC_64 hash of a data buffer
 */
uint64_t  MALP_Blackboard_level_id( char *MALP_Blackboard_level_name, char *object1_name, char *object2_name );


/**
 * @ingroup MALP_Blackboard_Level_
 * @brief MALP_Blackboard_level_id redefinition with "*" for the second object name
 */
#define KS_ID(bl,a) MALP_Blackboard_level_id( (char *)bl, a , "*" )
/**
 * @ingroup MALP_Blackboard_Level_
 * @brief KS_ID redefinition
 */
#define DE_ID(bl,t) KS_ID(bl,t)
/**
 * @ingroup MALP_Blackboard_Level_
 * @brief MALP_Blackboard_level_id redefinition with casting of bl as (char*)
 */
#define DE_ID_S(bl,t,s) MALP_Blackboard_level_id( (char *)bl, t , s )


/**
 * @ingroup MALP_Blackboard_Level_
 * @brief MALP_Blackboard_level_name redefinition with "*" for the second object name and "MALP_Blackboard_level_name" as first parameter
 */
#define KS_LID(a) MALP_Blackboard_level_id( (char *)MALP_Blackboard_level_name, a , "*" )
/**
 * @ingroup MALP_Blackboard_Level_
 * @brief The same as KS_LID
 */
#define DE_LID(t) KS_LID(t)
/**
 * @ingroup MALP_Blackboard_Level_
 * @brief Redefinition of MALP_Blackboard_level_id with "*" for the second object name
 */
#define DE_LID_S(t,s) MALP_Blackboard_level_id( (char *)MALP_Blackboard_level_name , t , s )


#endif /* MALP_BB_L_H */
