/* ############################ MALP License ############################## */
/* # Fri Jan 18 14:00:00 CET 2013                                         # */
/* # Copyright or (C) or Copr. Commissariat a l'Energie Atomique          # */
/* #                                                                      # */
/* # This software is governed by the CeCILL-C license under French law   # */
/* # and abiding by the rules of distribution of free software.  You can  # */
/* # use, modify and/ or redistribute the software under the terms of     # */
/* # the CeCILL-C license as circulated by CEA, CNRS and INRIA at the     # */
/* # following URL http://www.cecill.info.                                # */
/* #                                                                      # */
/* # The fact that you are presently reading this means that you have     # */
/* # had knowledge of the CeCILL-C license and that you accept its        # */
/* # terms.                                                               # */
/* #                                                                      # */
/* # Authors:                                                             # */
/* #   - BESNARD Jean-Baptiste jean-baptiste.besnard@cea.fr               # */
/* #                                                                      # */
/* ######################################################################## */
#ifndef PBB_WORKER_H
#define PBB_WORKER_H

#include <pthread.h>
#include <stdint.h>
#include <Spinlock.h>

#include "Knowledge_System.h"
#include "Buffered_FIFO.h"

#ifdef __cplusplus
  extern "C"
  {
#endif


struct MALP_blackboard;

/**
 * @ingroup Pbb_Workers_
 * @brief structure describing a workers descriptor
 */
struct Pbb_workers_desc
{
	int id;                 /**< @brief the id of the workers */
	struct Pbb_workers *wk; /**< @brief the workers */
};


/**
 * @ingroup Pbb_Workers_
 * @brief Structure defining a parallel blackboard worker set
 */
struct Pbb_workers
{
	uint32_t worker_count;          /**< @brief the number of workers */
	
	struct Pbb_workers_desc *descs; /**< @brief the workers descriptors */
	pthread_t *threads;             /**< @brief the threads of the workers */
	uint32_t running;               /**< @brief indicates if the workers are running or not */

	struct MALP_blackboard *bb;     /**< @brief the blackboard where the workers run */
	struct Buffered_FIFO *fifos;    /**< @brief a set of FIFOs where Knowledge_system_job are stored */
};

/**
 * @ingroup Pbb_Workers_
 * @brief Initializes workers
 * @param wk the workers to initialize
 * @param bb the associated blackboard
 * @param wk_count the number of wanted parallel workers
 */
void Pbb_workers_init( struct Pbb_workers *wk, struct MALP_blackboard *bb, uint32_t wk_count );

/**
 * @ingroup Pbb_Workers_
 * @brief releases workers
 * @param wk the workers to release
 */
void Pbb_workers_release( struct Pbb_workers *wk );

/**
 * @ingroup Pbb_Workers_
 * @brief sumbits a job on workers
 * @param wk the workers where to submit jobs
 * @param ksj the job to submit
 */
void Pbb_workers_submit( struct Pbb_workers *wk, struct Knowledge_system_job *ksj );

/**
 * @ingroup Pbb_Workers_
 * @brief Waits until all workers are done
 * @param wk the workers to wait
 */
void Pbb_workers_wait( struct Pbb_workers *wk );

#ifdef __cplusplus
  }
#endif

#endif
