#include "Data_Pack.h"

#include <stdlib.h>
#include <stdio.h>
#include <errno.h>
#include <string.h>

/*
 * 
 * Data Unpacker
 * 
 */


void Data_pack_print( struct Data_pack *pack )
{
	printf("=====================\n");
	printf("Source : %lld\n", (long long int)pack->source_id );
	printf("Is_eof : %d\n", (int)pack->is_eof );
	printf("size : %ld\n", pack->size );
	printf("total_size : %ld\n", pack->total_size );
	printf("data[0] : %c\n", pack->data[0] );
	printf("data[1] : %c\n", pack->data[1] );
	printf("data[2] : %c\n", pack->data[2] );
	printf("data[3] : %c\n", pack->data[3] );
	printf("=====================\n");
}


struct Data_Unpacker_entry * Data_Unpacker_entry( struct Data_pack *pack, struct Data_Unpacker_entry * next )
{
	struct Data_Unpacker_entry * ret = calloc( 1, sizeof( struct Data_Unpacker_entry ) );
	
	if( ! ret )
	{
		perror("calloc");
		return NULL;
	}
	
	ret->id = pack->source_id;
	ret->offset = 0;
	ret->size = pack->total_size;
	ret->next = next;
	
	if( ret->size )
	{
		ret->data = malloc( ret->size );
		
		if( !ret->data )
		{
			perror("malloc");
			abort();
		}
	}
	
	return ret;
}

void Data_Unpacker_entry_release( struct Data_Unpacker_entry *ent )
{
	if( !ent )
		return;
	
	/* Note that data are not free it is the charge of the user */
	
	memset( ent, 0, sizeof( struct Data_Unpacker_entry ) );
	free( ent );
}

char * Data_Unpacker_entry_push( struct Data_Unpacker_entry *de , struct Data_pack *pack )
{
	if( !de )
	{
		printf("Warning pushing a pack to an empty de \n");
		abort();
	}
	
	if( !pack )
	{
		printf("Warning pushing a NULL pack \n");
		abort();
	}
	
	if( de->size < de->offset + pack->size )
	{
		printf("Error pushing is going to overflow buffer\n");
		abort();
	}
	
	if( PACK_SIZE_UNIT < pack->size )
	{
		printf("Warning this pack seems to be erroneous\n");
		return NULL;
	}
	
	if( de->size != pack->total_size )
	{
		printf("Error size mismatch in pack\n");
		abort();
	}
	
	//printf("Data %p Offset %d / size %ld Pack data %p pack size %ld\n", de->data, de->offset, de->size,  pack->data, pack->size );
	
	memcpy( de->data + de->offset , pack->data, pack->size );
	de->offset += pack->size;
	
	if( de->offset == de->size )
	{
		return de->data;
	}
	
	return NULL;
}





void Data_Unpacker_init( struct Data_Unpacker *du, uint64_t ht_size )
{
	if( !ht_size )
	{
		printf("HT cannot be empty\n");
		abort();
	}
		
	du->entries = calloc( sizeof( struct Data_Unpacker_entry * ), ht_size );
	
	if( !du->entries )
	{
		perror("calloc");
		abort();
	}
	
	du->ht_size = ht_size;

	if( pthread_mutex_init(&du->unpacker_lock, NULL) != 0 )
	{
		perror("pthread_mutex_init");
		abort();
	}
	
}




void Data_Unpacker_release( struct Data_Unpacker *du )
{
	/* Warn for unfinised transactions */
	int i;
	
	for( i = 0 ; i < du->ht_size ; i++ )
	{
		struct Data_Unpacker_entry *ent = du->entries[i];
		
		if( ent )
		{
			do
			{
				printf("Warning unfinished entry from %lld\n", (long long int)ent->id );
				struct Data_Unpacker_entry * to_free = ent;
				ent = ent->next;
				Data_Unpacker_entry_release( to_free );
			}while( ent );	
		}
	}
	
	
	/* Release */
	pthread_mutex_destroy(&du->unpacker_lock);
	free( du->entries );
	du->ht_size = 0;
}

char * Data_Unpacker_push( struct Data_Unpacker *du, struct Data_pack *pack )
{
	uint64_t id = pack->source_id;
	uint64_t cell = pack->source_id % du->ht_size;
	
	pthread_mutex_lock(&du->unpacker_lock);
	
	struct Data_Unpacker_entry *ent = du->entries[cell];
	
	while( ent )
	{
		if( ent->id == id )
		{
			break;
		}
	}
	
	/* At this point we did not find the entry so create a new one */
	int create_a_new_entry = 0;
	
	if( !ent )
	{
		create_a_new_entry = 1;
	}
	else
	{
		if( ent->id != id )
		{
			create_a_new_entry = 1;
		}
	}
	
	char * ret = NULL;
	
	if( create_a_new_entry )
	{
		du->entries[cell] = Data_Unpacker_entry( pack, du->entries[cell] );
		ent = du->entries[cell];
	}

	ret = Data_Unpacker_entry_push( ent, pack );
	
	/* Entry is completed remove from HT */
	if( ret && ent )
	{
		/* First on the list */
		if( ent == du->entries[cell] )
		{
			du->entries[cell] = ent->next;
		}
		else
		{
			/* Walk until being next and bypass */
			struct Data_Unpacker_entry *tmp = du->entries[cell];
			while( tmp )
			{
				if( tmp->next == ent )
				{
					tmp->next = ent->next;
					break;
				}
			}
		}
		
		Data_Unpacker_entry_release( ent );
	}

	pthread_mutex_unlock(&du->unpacker_lock);
	
	return ret;
}
